/*
   M T P C K G


   History:

   11th Mar 1988.  CKL / CIT. Converted to Convex Unix. Length of
                   Fortran character variables added to argument list.
*/

#include <errno.h>

#include <sys/types.h>

#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <fcntl.h>

#include "mtercs.h"                 /* MTPCKG error codes */

#define MTCMAX 8                    /* maximum number of tape drives */
#define TRUE   (1)                  /* truth is non-zero */
#define FALSE  (0)                  /* false is zero */

extern  int  errno;
extern  int  sys_nerr;
extern  char *sys_errlist[];

int     mtfd[MTCMAX];               /* tape file descriptors */
char    mtpnam[MTCMAX][64+1];       /* physical tape device names */
int     mtwtrt[MTCMAX];             /* last function return code */
int     mtwtva[MTCMAX];             /* actual # of bytes transferred */
int     mtwtop[MTCMAX];             /* operation code called in wait mode */
int     mtopnd[MTCMAX];             /* logical array for opened id */
int     mtwtmd[MTCMAX];             /* logical array for async wait mode */
int     mtnoer[MTCMAX];             /* logical array for no error recovery */
int     mtunixerr;                  /* last unix error returned */
struct  mtop  op;                   /* tape operation io structure */
struct  mtget opstat;               /* tape status io structure */

/*
2 MTNAME
CALL MTNAME(ID, NAME, LENGTH, RESULT) -- To obtain the physical name of
=====================================    the actual tape drive that is
            allocated to this value of ID.

      NAME   - CHARACTER:  (output) Receives the name string of the
                           allocated tape drive.  If NAME is not long
                           enough to hold the complete name string,
                           then the name string is truncated at the
                           right so that what is left of it will fit.
                           NAME should have length at least 10 to avoid
                           this.

      LENGTH - INTEGER:  (output) receives the actual length of the name
                           string.

      RESULT - INTEGER:  (output) Can receive a non-zero warning code:

                   MTRTRN - NAME is not long enough to hold all of the
                              name string.
*/

void
mtname_(id,name,length,result,namelen)
int *id;
char *name;
int *length,*result;
int namelen;
{
   char *strncpy();
   int strlen();

   int len;

   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      len = strlen(&mtpnam[*id][0]);
      if (namelen < len ) {
         *length = namelen;
         *result = MTRTRN;
      } else {
         *length = len;
         *result = 0;
      }
      (void) strncpy(name,&mtpnam[*id][0],*length);
      mtwtva[*id] = *length;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTPREP
CALL MTPREP(ID, DENSIT, RESULT) -- To set the tape density for writing.
===============================    this should be done when the writing
            is to commence at the beginning of the tape.  The default
            density is 800 BPI on PHOBOS and 1600 on DEIMOS.
	    If the tape has been read or spaced,
            then the density will have been set to whatever value pre-
            vailed on the tape itself.  MTPREP rewinds the tape so that
            it is impossible to write a tape with records of varying
            density.

            In Unix, it only rewinds the tape.  Different densities
            are selected by the device name.

      DENSIT - INTEGER: The value for the recording density: 800 or 1600
                        or 6250.
      RESULT - INTEGER:  (output) can receive non-zero values:

                   MTRERR - An I/O error occurred
                   MTROFL - The tape drive is offline
                   MTRDBN - The density value was other than 800 or
                              1600 or 6250). 
                   MTRUNX - Unexpected failure
*/

void
mtprep_(id,density,result)
int *id,*density,*result;
{
    void mtrewi_(),mtwait_();

    int dummy;

    if (*density != 800 && *density != 1600 && *density != 6250) {
       *result = MTRBDN;
       return;
    }
    mtrewi_(id,result);
    if (mtwtmd[*id])
       mtwait_(id,&dummy,result);
}

/*
2 MTCLOS
CALL MTCLOS(ID, RESULT) -- To deallocate the tape drive associated with
=======================    this value of ID.  If there is a tape reel
            currently mounted on the drive it is not rewound or unloaded.
            The ID value becomes invalid for further use in MTPCKG.

      RESULT - INTEGER:  (output) is always zero.
*/

void
mtclos_(id,result)
int *id,*result;
{
   int ioerr;

   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      ioerr = close(mtfd[*id]);
      if (ioerr < 0)
         mtunixerr = errno;
      mtopnd[*id] = FALSE;
   }
   *result = 0;
}

/*
2 MTSERM
CALL MTSERM(ID, INHERC, RESULT) -- To set the I/O error recovery mode.
===============================

      INHERC - LOGICAL:  .TRUE. To inhibit error recovery procedures on
               I/O errors;  .FALSE. for normal error recovery attempts.
               The error recovery mode set by this function will remain
               in effect until changed by another call of MTSERM.  The
               default error recovery mode is RETRY (INHERC = .FALSE.).

      RESULT - INTEGER:  (output) possible non-zero values:

                   MTRERU - The error recovery mode is unchanged,i.e.,
                              the error recovery mode specified by the
                              parameter, INHERC, was already in effect.
*/

void
mtserm_(id,inherc,result)
int *id,*inherc,*result;
{
   int ioctl();

   int ioerr;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      if ( *inherc == mtnoer[*id] )
         *result = MTRERU;
      else if ( *inherc )  {
         ioerr = ioctl(mtfd[*id],MTIOCNRTY);
      }
      else {
         ioerr = ioctl(mtfd[*id],MTIOCRTY);
      }
      mtnoer[*id] = *inherc;
      mtwtva[*id] = 0;
      mtwtrt[*id] = *result;
   }
}   

/*
2 MTSETW
CALL MTSETW(ID, EXTERN, RESULT) -- To set the I/O completion mode.
===============================  The I/O completion mode has two states,
            internal and external.  When the I/O completion mode is
            internal, then each of the procedures of MTPCKG returns to
            its caller only after all of its operations have completed,
            including data transfer and tape motion, at which time the
            caller receives the appropriate completion status inform-
            ation.  When the I/O completion mode is external, then those
            procedures of MTPCKG which involve tape motion (except
            MTPREP), i.e., MTREAD, MTRDBK, MTWRIT, MTWEOF, MTREWI,
            MTUNLD, MTSKRE, and MTSKFI, return to the caller as soon as
            the I/O operation has been initiated; no completion status
            information is communicated to the caller. At that time,
            values of zero are assigned to the output parameters,
            RESULT, ACTLEN OR ACTCNT.  In this case the caller
            must subsequently call either MTWAIT or MTCHCK (see below),
            and this procedure will await the completion of the prev-
            iously initiated I/O operation, then returning the
            completion status via the output parameters, RESULT and
            VALUE.  The state of the I/O completion mode has no effect
            on the operation of other procedures of MTPCKG than those
            listed above.

      EXTERN - LOGICAL:.TRUE. To set the I/O completion mode to external
               .FALSE. To set the I/O completion mode to internal.  The
               default state of the I/O completion mode is internal.

      RESULT - INTEGER:  (output) possible non-zero values:

                   MTRWMU - The I/O completion mode is unchanged,i.e.,
                              the I/O completion mode was already in the
                              state specified by the input parameter,
                              EXTERN.
*/

void
mtsetw_(id,wait,result)
int *id,*wait,*result;
{
   int fcntl();

   int ioerr;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      if (*wait && mtwtmd[*id])
         *result = MTRWMU;
      else {
         ioerr = fcntl(mtfd[*id],F_SETFL,FASIO);
      }
      mtwtmd[*id] = *wait;
      mtwtva[*id] = 0;
      mtwtrt[*id] = *result;
   }        
}

/*
2 MTWAIT
CALL MTWAIT(ID, VALUE, RESULT) -- To await the completion of the most
==============================    recently initiated I/O operation of
            MTPCKG (with the same ID) and to obtain its completion
            status.  This operation involves a physical wait only for
            I/O operations which entail tape motion.  The operation
            MTCHCK, above, is identical to MTWAIT.  These functions may
            be called no matter what the state of the I/O completion
            mode, but it is mandatory that they be called when the I/O
            completion mode is external and the immediately preceding
            call of MTPCKG was to MTREAD, MTRDBK, MTWRIT, MTWEOF,
            MTREWI, MTUNLD, MTSKRE, or MTSKFI, except when the next call
            is to MTCLOS, MTWAIT, or MTCHCK.

      VALUE  - INTEGER: (output) Receives the value that in internal I/O
               completion mode would have been returned to the caller
               via the output parameter ACTLEN or ACTCNT.  The value
               zero is returned if the preceding I/O operation would not
               have returned such a value (e.g., MTWEOF).

      RESULT - INTEGER: (output) Receives the value that in internal I/O
               completion mode would have been returned to the caller
               via the output parameter RESULT.


      Calls of MTWAIT (or MTCHCK) Do not reset the stored values for the
      output parameters, VALUE and RESULT, so that several calls of
      MTWAIT will all return the same results providing that no other
      calls of MTPCKG with the same ID intervene.

      The table given below summarizes the requirements for calling MTWAIT
      When the I/O completion mode is external.  The entries in the (A, B)
      positions of the table signify as follows:

            * :  Not Applicable;
            + :  A call of MTWAIT or MTCHCK is mandatory between a call
                 of A and a call of B;
            - :  A call of MTWAIT or MTCHCK is not required between a call
                 of A and a call of B.




\     O | C | W | C | N | S | P | S | S | N | R | R | W | W | R | U | S | S |
 \B   P | L | A | H | A | E | R | E | E | O | E | D | R | E | E | N | K | K |
 A\   E | O | I | C | M | N | E | R | T | O | A | B | I | O | W | L | R | F |
   \  N | S | T | K | E | S | P | M | W | P | D | K | T | F | I | D | E | I |
    |---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
OPEN| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
CLOS| * | * | * | * | * | * | * | * | * | * | * | * | * | * | * | * | * | * |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
WAIT| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
CHCK| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
NAME| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
SENS| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
PREP| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
SERM| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
SETW| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
NOOP| * | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - | - |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
READ| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
RDBK| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
WRIT| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
WEOF| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
REWI| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
UNLD| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
SKRE| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
SKFI| * | - | - | - | + | + | + | + | + | + | + | + | + | + | + | + | + | + |
----+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|

    MTOPID is the same as MTOPEN for the purposes of this table, and
    MTVMER, MTMESS, MTGETN, should be regarded as utilities which can
    be ignored as far as MTWAIT is concerned.
*/

void
mtwait_(id,count,result)
int *id,*count,*result;
{
   void mtgetstat();
   int asiostat();

   int retval;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      if (mtwtmd[*id]) {
         switch (mtwtop[*id]) {

         case 1:            /* Read Forward, Read Backward */
            retval = asiostat(mtfd[*id]);
            if (retval == 0)  /* END-OF-FILE */
               *result = MTREOF;
            else if (retval < 0) {      /* check tape status */
               mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
               retval = 0;
            }
            mtwtva[*id] = retval;
            break;
         case 2:            /* Write */
            retval = asiostat(mtfd[*id]);
            if (retval < 0) {           /* check tape status */
               mtgetstat(id,result,2);  /* Returns MTROFL, etc... */
               retval = 0;
            }
            mtwtva[*id] = retval;
            break;
         case 3:            /* Write End-Of-File */
            retval = asiostat(mtfd[*id]);
            if (retval < 0) {           /* check tape status */
               mtgetstat(id,result,2);  /* Returns MTROFL, etc... */
               retval = 0;
            } else
               retval = 1;
            mtwtva[*id] = retval;
            break;
         case 4:            /* Rewind, Rewind-unload */
            retval = asiostat(mtfd[*id]);
            if (retval < 0) {           /* check tape status */
               mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
            }
            mtwtva[*id] = 0;
            break;
         case 5:            /* Skip File */
         case 6:            /* Skip Record */
                  /* mtwtva not changed so asked count can be returned */
            retval = asiostat(mtfd[*id]);
            if (retval < 0) {           /* check tape status */
               mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
               if (*result == MTREOF && mtwtop[*id] == 5)
                  *result = 0;
            }
            break;
         }
         mtwtrt[*id] = *result;
         mtwtop[*id] = 0;
      }   
      *count  = mtwtva[*id];
      *result = mtwtrt[*id];
   }
}

void
mtchck_(id,count,result)
int *id,*count,*result;
{
    void mtwait_();

    mtwait_(id,count,result);
}

/*
2 MTNOOP
CALL MTNOOP(ID, VALUE, RESULT) -- To perform no operation.  This has the
==============================    effect of insuring that an immediately
            succeeding call of MTWAIT or MTCHCK (see below) will return
            with the specified "VALUE" and RESULT = 0.

      VALUE  - INTEGER:  The "VALUE" to be returned by a succeeding call
               of MTWAIT or MTCHCK.

      RESULT - INTEGER:  (output) is always zero.
*/

void
mtnoop_(id,value,result)
int *id,*value,*result;
{
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      mtwtva[*id] = *value;
      mtwtrt[*id] = 0;
   }
   *result = 0;                        
}

/*
2 MTREAD
CALL MTREAD(ID,BUFLEN,BUFFER,ACTLEN,RESULT) -- To read the next record.
===============================================    in the forward direc-
            tion from the currently loaded tape reel into the buffer.

      BUFLEN - INTEGER:  The length, in bytes, of the buffer area into
               which the record data are to be read.  This value is the
               number of bytes which MTPCKG will endeavor to read.
  
      BUFFER - LOGICAL*1(BUFLEN):(output) Buffer which receives the data
               from the read operation.

      ACTLEN - INTEGER:  (output) The actual number of bytes transferred
               from the tape, but not necessarily into the buffer. If no
               error occurred, then this is the actual length of the
               record. The number of bytes of data read into the buffer
               is always MIN(BUFLEN, ACTLEN).  If ACTLEN is > BUFLEN,
               then the buffer will contain the first BUFLEN bytes of
               data from the record.

      RESULT - INTEGER:  (output) possible non-zero values:

                   MTRERR - An I/O error occurred
                   MTROFL - The tape is offline
                   MTREOF - The record was an End-of-File mark
                   MTREOT - The End-of-Tape mark was encountered
                              during the read, or the tape was position-
                              ed past the End-of-Tape mark at the start
                              of the read. MTREOF takes precedence
                              over MTREOT when both situations occur
                              together.
                   MTRNOR - No Start of Record was encountered within
                              several feet of forward motion of the tape
                              presumably the read was initiated when the
                              tape was positioned past the physical end
                              of data.
                   MTRUNX - Unexpected failure
*/

void
mtread_(id,buflen,buffer,actlen,result)
int *id,*buflen;
unsigned char *buffer;
int *actlen,*result;
{
   void mtgetstat();

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      *actlen = read(mtfd[*id],buffer,*buflen);
      if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 1;
         *actlen = 0;
         *result = 0;
      } else if ( *actlen == 0 )  /* END-OF-FILE */
         *result = MTREOF;
      else if (*actlen <= 0) {     /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
         *actlen = 0;
      }
      mtwtva[*id] = *actlen;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTRDBK
CALL MTRDBK(ID, BUFLEN, BUFFER, ACTLEN, RESULT) -- To read the previous
===============================================    record in the reverse
            direction from the currently loaded tape into a buffer.

      BUFLEN - INTEGER:  The length of the buffer, as above.
        
      BUFFER - LOGICAL*1(BUFLEN):  (output) The buffer to receive the
            data.

      ACTLEN - INTEGER: (output) The actual number of bytes transferred,
               as above.  if ACTLEN is > BUFLEN, then the buffer will
               contain the last BUFLEN bytes of data from the record.

      RESULT - INTEGER:  (output) possible non-zero values:

                   MTRERR - An I/O error occurred
                   MTROFL - The tape is offline
                   MTREOF - The record was an End-of-File mark, or the
                              tape was positioned at Beginning-of-Tape
                              before the read.
                   MTRUNX - Unexpected failure
*/

void     
mtrdbk_(id,buflen,buffer,actlen,result)
int *id,*buflen;
unsigned char *buffer;
int *actlen,*result;
{                  
   void mtgetstat();
   int ioctl();

   int ioerr;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      op.mt_op = MTBSR;
      op.mt_count = (daddr_t) 1;
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if ( ioerr < 0 ) {          /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
         *actlen = 0;
         mtwtva[*id] = *actlen;
         mtwtrt[*id] = *result;
         return;
      }   
      *actlen = read(mtfd[*id],buffer,*buflen);
      if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 1;
         *actlen = 0;
         *result = 0;
      } else if ( *actlen == 0 )  /* END-OF-FILE */
         *result = MTREOF;
      else if (*actlen < 0) {     /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
         *actlen = 0;
      }
                                  /* The following defeats asynch io */
                                  /* since it will block until io is done */
      op.mt_op = MTBSR;
      op.mt_count = (daddr_t) 1;
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if ( ioerr < 0 ) {          /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
         *actlen = 0;
      }   
      mtwtva[*id] = *actlen;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTWRIT
CALL MTWRIT(ID, BUFLEN, BUFFER, ACTLEN, RESULT) -- To write a record on
===============================================    the currently loaded
            tape from a buffer.

      BUFLEN - INTEGER:  The length of the buffer, e.g., the number of
               bytes to be written.  BUFLEN must be .GE. 14.

      BUFFER - LOGICAL*1(BUFLEN): The buffer containing the BUFLEN bytes
               of data to be written.

      ACTLEN - integer:  (output) the actual number of bytes written; If
               RESULT=0, then ACTLEN will equal BUFLEN.

      RESULT - INTEGER:  (output) possible non-zero values:

                   MTRERR - An I/O error occurred
                   MTROFL - The tape is offline
                   MTRWLK - The tape is write protected
                   MTREOT - End-of-Tape:  The tape entered the End-of-
                              Tape region during the write, or the tape
                              was positioned past the End-of-Tape
                              position before the write.
                   MTRUNX - Unexpected failure
*/

void
mtwrit_(id,buflen,buffer,actlen,result)
int *id,*buflen;
unsigned char *buffer;
int *actlen,*result;
{
   void mtgetstat();

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      *actlen = write(mtfd[*id],buffer,*buflen);
      if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 2;
         *actlen = 0;
         *result = 0;
      }
      else if (*actlen < 0) {     /* check tape status */
         mtgetstat(id,result,2);  /* Returns MTROFL, etc... */
         *actlen = 0;
      }
      mtwtva[*id] = *actlen;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTWEOF
CALL MTWEOF(ID, RESULT) -- To write an End-of-File mark on the currently
=======================    loaded tape.

      RESULT - INTEGER:  (output) the possible non-zero values of result
               are as for MTWRIT, above.
*/                  

void
mtweof_(id,result)
int *id,*result;
{
   void mtgetstat();
   int ioctl();

   int ioerr,count;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      op.mt_op = MTWEOF;
      op.mt_count = (daddr_t) 1;
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if ( ioerr < 0 ) {          /* check tape status */
         mtgetstat(id,result,2);  /* Returns MTROFL, etc... */
         count = 0;
      } else if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 3;
         count = 0;
         *result = 0;
      } else
         count = 1;
      mtwtva[*id] = count;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTREWI
CALL MTREWI(ID, RESULT) -- To rewind the currently loaded tape. The tape
=======================    is left at Beginning-of-Tape, i.e., at the
            beginning of the first file.

      RESULT - INTEGER:  (output) the possible non-zero values:

                   MTRERR - An I/O error occurred
                   MTROFL - The tape is offline.
*/

void
mtrewi_(id,result)
int *id,*result;
{
   void mtgetstat();
   int ioctl();

   int ioerr;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      op.mt_op = MTREW;
      op.mt_count = (daddr_t) 1;
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if ( ioerr < 0 ) {          /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
      } else if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 4;
         *result = 0;
      }
      mtwtva[*id] = 0;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTUNLD
CALL MTUNLD(ID, RESULT) -- To rewind the currently loaded tape and then
=======================    take the tape drive offline.  The tape drive 
            cannot go online without subsequent manual intervention.

      RESULT - INTEGER:  (output) as for MTREWI, above.
*/

void
mtunld_(id,result)
int *id,*result;
{
   void mtgetstat();
   int ioctl();

   int ioerr;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      op.mt_op = MTOFFL;
      op.mt_count = (daddr_t) 1;
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if ( ioerr < 0 ) {          /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
      } else if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 4;
         *result = 0;
      }
      mtwtva[*id] = 0;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTSKFI
CALL MTSKFI(ID, COUNT, ACTCNT, RESULT) -- To skip over a number of End-
======================================    of-File marks on the currently
            loaded tape in either the forward or reverse direction.

      COUNT  - INTEGER:  The number of End-of-File marks to be skipped:
               forward, if > 0, reverse, if < 0.  the operation is
               terminated by Beginning-of-Tape in the reverse direction;
               the operation is not terminated by End-of-Tape in the
               forward direction. Note that, for skipping in the reverse
               direction not terminated by beginning-of-tape, the tape
               is left positioned at the end of a file;  in the other
               cases the tape is left positioned at the start of a file.

      ACTCNT - INTEGER:  (output) The actual number of End-of-File marks
               skipped.  ACTCNT is always .ge. 0.

      RESULT - INTEGER:  (output) The possible non-zero values are:

                   MTRERR - An I/O error occurred
                   MTROFL - The tape is offline
                   MTREOT - The End-of-Tape marker was encountered
                              while skipping forward.
                   MTRNOR - No End-of-File mark or data record was en-
                              countered within several feet of tape mo-
                              tion.
                   MTRUNX - Unexpected failure
*/

void
mtskfi_(id,count,actcnt,result)
int *id,*count,*actcnt,*result;
{
   void mtgetstat();
   int ioctl();

   int ioerr;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      *actcnt = *count;           /* Can't get true count from system */
      op.mt_op = (*count < 0) ? MTBSF : MTFSF;
      op.mt_count = (daddr_t) abs(*count);
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if ( ioerr < 0 ) {          /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
         if (*result == MTREOF)
            *result = 0;
      } else if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 5;
         *actcnt = 0;
         *result = 0;
      }
      mtwtva[*id] = *count;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTSKRE
CALL MTSKRE(ID,COUNT,ACTCNT,RESULT) -- To skip over a number of records
======================================   of the currently loaded tape in
            either the forward or reverse direction.

      COUNT  - INTEGER:  The number of records to be skipped:  forward,
               if > 0, reverse, If < 0.  The operation is terminated
               if an End-of-File mark is skipped, if the End-of-Tape
               mark is sensed in the forward direction, or if beginning-
               of-tape encountered in the reverse direction.

      ACTCNT - INTEGER:  (output) the actual number of records skipped,
               including a terminating End-of-File mark, if any.  ACTCNT
               is always .ge. 0.

      RESULT - INTEGER: (output) the possible non-zero values are as for
               MTREAD, above, when skipping forward;  when skipping in
               reverse, the possible values of result are as for MTRDBK,
               above, except that MTRNOR (no record encountered) is
               also possible.
*/

void
mtskre_(id,count,actcnt,result)
int *id,*count,*actcnt,*result;
{
   void mtgetstat();
   int ioctl();

   int ioerr;

   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      *actcnt = *count;           /* Can't get true count from system */
      op.mt_op = (*count < 0) ? MTBSR : MTFSR;
      op.mt_count = (daddr_t) abs(*count);
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if ( ioerr < 0 ) {          /* check tape status */
         mtgetstat(id,result,1);  /* Returns MTROFL, etc... */
      } else if ( mtwtmd[*id] ) {        /* asynchronous io */
         mtwtop[*id] = 6;
         *actcnt = 0;
         *result = 0;
      }
      mtwtva[*id] = *count;
      mtwtrt[*id] = *result;
   }
}

/*
2 MTSENS
CALL MTSENS(ID, STATUS, RESULT) -- To obtain information about the
===============================    current status of the tape drive
                                   associated with this value of ID.

      STATUS - INTEGER: (output) receives 32 bits of status information:

                   00000000000ABCDE000FFFFFGGGGH000
  
               Which have the following significance:

                   A    =1     - Tape position lost
                        =0     - Tape position OK
                   B    =1     - Tape is write locked
                        =0     - Tape is not write locked
                   C    =1     - End-of-Tape marker encountered
                        =0     - Not End-of-Tape
                   D    =1     - End-of-File encountered
                        =0     - Not End-of-File
                   E    =1     - Beginning of tape
                        =0     - Not beginning of tape
                   FFFFF=00011 - Density is 800 BPI
                        =00100 - Density is 1600 BPI
                        =00101 - Density is 6250 BPI
                   GGGG =1100  - Tape format is normal 11
                         0000  - Tape format is default (normal 11)
                   H    =1     - Even parity
                        =0     - Odd parity.

      RESULT - INTEGER:  (output) can receive a non-zero return code:

                   MTRERR - An I/O error occurred while obtaining the
                              status
                   MTROFL - The tape drive is offline
                   MTRVIN - The tape drive is online but has been
                              offline sometime between now and when you
                              last called MTPCKG, i.e., the tape reel
                              has been manually rewound or unloaded or
                              switched.
                   MTRUNX - Unexpected failure
*/

void
mtsens_(id,status,result)
int *id,*status,*result;
{
   int ioctl();

   int ioerr;

   *status = 0;
   *result = 0;
   if ( (*id >= 0) && (*id < MTCMAX) && mtopnd[*id] ) {
      op.mt_op = MTNOP;
      op.mt_count = (daddr_t) 1;
      ioerr = ioctl(mtfd[*id],MTIOCTOP,&op);
      if (ioerr < 0) {
         *result = MTRUNX;
         mtunixerr = errno;
      }
      ioerr = ioctl(mtfd[*id],MTIOCGET,&opstat);
      if (ioerr < 0) {
         *result = MTRUNX;
         mtunixerr = errno;
      } else {
         if (!(opstat.mt_dsreg & MT_ONL))      /* Not Online */
            *result = MTROFL;
         if (!(opstat.mt_dsreg & MT_RDY)) /* Operation incomplete ? */
            *result = MTRNOR;
         if (opstat.mt_dsreg & MT_EOT)  /* End-Of-Tape */
            *status |= 0x40000;         /* bit 18 starting from 0 */
         if (opstat.mt_dsreg & MT_FPT)  /* Write Protect Status */
            *status |= 0x80000;         /* bit 19 */
         if (opstat.mt_erreg & MT_LOST) /* Tape Position Lost */
            *status |= 0x100000;        /* bit 20 */
         if (opstat.mt_dsreg & MT_TM)   /* Tape Mark(End-of-File) */
            *status |= 0x20000;         /* bit 17 */
         if (opstat.mt_dsreg & MT_BOT)  /* Beginning-Of-Tape */
            *status |= 0x10000;         /* bit 16 */
                                        /* density is at bits 12..8 */
                                        /* tape format is at bits 7..4 */
                                        /* parity is at bit 3 */
      }
   }
}

/*
2 MTGETN
CALL MTGETN(DRIVE,NAME,RESULT) -- To obtain the name of one of the
=============================     tape drives on the system.  These
            are defined by the system environment name TAPES, and the
            order in which the drive names appear in TAPES 
            defines how the values for DRIVE map onto the actual
            device names. (If TAPES is defined as "/dev/rmt16 /dev/rmt12",
            a call with DRIVE=1 will return NAME='/dev/rmt16', a call
            with DRIVE=2 will return NAME='/dev/rmt12')

      DRIVE  - INTEGER:   (input) The number of the drive whose name
                          is to be returned.  This should be between 1
                          and n, where n is the number of drives on
                          the system. 

      NAME   - CHARACTER: (output) The name of the tape drive. Note 
                          that this will be a 10 character string, eg
                          '/dev/rmt16'

      RESULT - INTEGER:   (output) Receives a code indicating success
                          or failure. Possible failure codes are:

                            = MTRNDV - The value of DRIVE is either
                                       not positive, or is greater than
                                       the number of drives on the
                                       system.
                            = MTRNTR - Unable to translate the environment
                                       name TAPES.
*/

void
mtgetn_(ndrive,name,result,namelen)
int *ndrive;
char *name;
int *result;
int namelen;
{
   int strlen();
   char *getenv(),*strncpy(),*index();

   int i,len,length;
   char *tapes,*space;

   tapes = getenv("TAPES");
   if ((int) tapes == 0) {
      mtunixerr = errno;
      *result = MTRNTR;
      return;
   }
   *result = MTRNDV;
   for (i = 1; i <= *ndrive; i++) {
       space = index(tapes,' ');
       len = strlen(tapes);
       len = space == 0 ? len : space - tapes;

       if (i == *ndrive) {
          length = namelen < len ? namelen : len;
          (void) strncpy(name,tapes,length);
          *result = 0;
       }
       if (space == 0)      /* break out if space not found in TAPES */
          break;
       tapes += (len+1);
   }
}

/*
2 MTMOUN
CALL MTMOUN(NAME,RESULT) -- To mount a tape on a named drive.  This
=======================     routine is used by MTOPID and MTOPEN, and
is probably only of limited use when called directly.  

In Unix, this is a no op.

      NAME   - CHARACTER: (input) The name of the drive on which a
                          tape is to be mounted.  The drive must not
                          be allocated to another user, and it must
                          have a tape physically loaded.  If the
                          drive has already been mounted by the user
                          using DCL - or even a previous MTMOUN call -
                          MTMOUN will leave it as it is and return
                          RESULT=0.  

      RESULT - INTEGER:   (output) Receives a success or failure
                          indication.  Non-zero codes are

                            MTROFL - Tape is off-line
                            MTRALL - Drive was already allocated to
                                     another user
                            MTRIDV - Device name is illegal
                            MTRNMT - Failed to mount tape (use MTVMER
                                     for more details)
                            MTRUNX - Unexpected failure
*/

void
mtmoun_(name,result,namelen)
char *name;
int *result;
int namelen;
{
   *result = 0;
}

/*
2 MTOPEN
CALL MTOPEN(ID, RESULT) -- To allocate and connect a tape drive to the
=======================    calling process.

      ID     - INTEGER:  (output) Receives an INTEGER value from MTOPEN
               which is to be used to identify the particular tape drive
               that has been allocated to your use by this call.  The
               value of ID is used as an input parameter to every subse-
               quent call upon the I/O primitives of MTPCKG.  By several
               calls of MTOPEN you may allocate up to 8 tape drives, or
               up to as many tape drives as are available if there are
               less than 8 of them.

      RESULT - INTEGER:  (output) Receives an indication of success or
               failure.  The modes of failure can be as follows:

                   = MTRNDV - All of the tape drives have been allocated
                              to you or to others, so that there is none
                              to fulfill this request.
                   = MTRNEF - The operating system run out of event
                              flags and therefore lacks the means of
                              synchronizing I/O operations on the
                              allocated channel.
                   = MTRNTR - Cannot translate SYS$TAPES to determine
                              which drives are on the system.
                   = MTRUNX - Unexpected failure.

               If RESULT is .NE. 0, then the ID value is invalid for any
               subsequent calls of MTPCKG.
*/

void
mtopen_(id,result)
int *id,*result;
{
   void mtgetn_();
   int strncmp(),open();
   char *strncpy();

   int slot,i,status,drive,len,ioerr;
   char *space,name[64+1];

   for ( slot=0 ; slot < MTCMAX ; slot++ ) {   /* find free slot */
      if (!mtopnd[slot]) {                         
         *id = slot;
         break;
      } else if (slot == MTCMAX-1) {
         *result = MTRNDV;     /* return if there are no free slots */
         *id = -1;
         return;
      }
   }
   for (i=0; i<64; i++)
       mtpnam[slot][i] = ' ';
   mtpnam[slot][64] = '\0';
   for (status=0,drive=1; status == 0; drive++) {
       mtgetn_(&drive,&mtpnam[slot][0],&status,64);
       if (status == 0) {   /* check we don't already have this drive */
          status = -1;
          for (i=0; i<MTCMAX; i++) {
              if (mtopnd[i]) {
                 if (strncmp(&mtpnam[slot][0],&mtpnam[i][0],64) == 0)
                    status = 0;
              }
              if (status == 0)
                 break;
          }
       }
       if (status == -1) {    /* found one */
          space = index(&mtpnam[slot][0],' ');
          len = strlen(&mtpnam[slot][0]);
          len = space == 0 ? len : space - &mtpnam[slot][0];
          (void) strncpy(name,&mtpnam[slot][0],len);
          ioerr = open(name,O_RDWR,600);
          if (ioerr < 0)            /* may be write-protected */
             ioerr = open(name,O_RDONLY,400);
          if (ioerr < 0) {
             mtunixerr = errno;
             switch (errno) {

             case EMFILE:
                 *result = MTRNCH;
                 break;
             case ENOTDIR:
             case ENXIO:
                 *result = MTRIDV;
                 break;
             default:
                 *result = MTRUNX;
                 break;
             }
             *id = -1;
             return;
          }
          mtfd[*id] = ioerr;
          mtwtrt[*id] = 0;
          mtopnd[*id] = TRUE;
          mtnoer[*id] = FALSE;
          *result = 0;
       }
   }
}

/*
2 MTOPID
CALL MTOPID(NAME,ID,RESULT) -- To open a tape drive for subsequent use
===========================    by MTPCKG, given the name of the drive.
            This is an alternative to MTOPEN, and allows the calling
            program greater control over precisely which tape drives
            are used.  However, it requires that the program know the
            drive device names, either via a call to MTGETN, or by
            prompting the user.  Actually building device names into
            a program is not recommended; it is possible to use logical
            names instead.

      NAME   - CHARACTER:  (input) Either the actual name of the tape
                           drive to be opened, or a logical name which
                           will translate to that name.  Actual names
                           should be 6 characters, eg '_MTA0:'.
                       
      ID     - INTEGER:    (output) Receives an integer value to be used
                           in all subsequent calls to MTPCKG routines 
                           to refer to the tape drive that has been 
                           opened by this call to MTOPID.  This is
                           exactly equivalent to the ID parameter of
                           MTOPEN.

      RESULT - INTEGER:    (output) Receives an indication of success or
                           failure.  The possible values are:

                             = MTRNDV - There are no more available
                                        device slots in the common areas
                                        used by MTPCKG - ie you have
                                        already opened 8 tape drives!
                             = MTROFL - The tape drive specified is
                                        off line.
                             = MTRNEF - The system has run out of event
                                        flags and cannot allocate one
                                        for use with this drive.
                             = MTRNMT - The system has failed to mount
                                        the tape.  (For more details,
                                        use MTVMER)
                             = MTRALL - The drive specified is already
                                        allocated to another user.
                             = MTRIDV - The value given for NAME does
                                        not represent a valid tape
                                        drive.
                             = MTRUNX - Unexpected failure
*/

void
mtopid_(name,id,result,namelen)
char *name;
int *id,*result;
int namelen;
{
   char *getenv();
   int strncmp(),open();

   char envname[64+1],*trname,*space;
   int slot,len,i,ioerr;

   len = namelen;
   (void) strncpy(envname,name,len);
   envname[len] = '\0';
   trname = getenv(envname);               /* translate name */
   if ((int) trname == 0)
      trname = envname;
   else
      len = strlen(envname);
   space = index(trname,' ');
   len = space == 0 ? len : space - trname;
   trname[len] = '\0';
   for ( slot=0 ; slot < MTCMAX ; slot++ ) {   /* find free slot */
      if ( mtopnd[slot] ) {                         
         if ( strncmp(&mtpnam[slot][0],trname,len) == 0 ) {  
              *id = slot;            /* name opened already */
              *result = 0;
              return;
         }
         if ( slot == MTCMAX-1 ) {   /* return if there are no free slots */
              *result = MTRNDV;
              return;
         }
      }
      else {
         *id = slot;
         break;
      }
   }
   for (i=0; i<64; i++)
       mtpnam[*id][i] = ' ';
   mtpnam[*id][64] = '\0';
                                       /* put physical name in array */
   (void) strncpy(&mtpnam[*id][0],trname,len); 
   ioerr = open(trname,O_RDWR,600);    /* get a file descriptor */
   if (ioerr < 0)                      /* may be write-protected */
      ioerr = open(trname,O_RDONLY,400);
   if ( ioerr < 0 ) {
      mtunixerr = errno;
      switch (errno) {

      case EMFILE:
          *result = MTRNCH;
          break;
      case ENOTDIR:
      case ENXIO:
          *result = MTRIDV;
          break;
      default:
          *result = MTRUNX;
          break;
      }
      return;
   }
   mtfd[*id] = ioerr;
   mtwtrt[*id] = 0;
   mtopnd[*id] = TRUE;
   mtnoer[*id] = FALSE;
   *result = 0;
}

/*
2 MTMESS
CALL MTMESS(RESULT,MTSUBR,ERROR) -- To obtain a description of the
===============================     error condition that has caused
an MTPCKG routine to return a particular error code.

      RESULT - INTEGER:    (input) The result code returned by an
                           MTPCKG subroutine.

      MTSUBR - CHARACTER:  (input) The name of the routine that 
                           returned that code.

      ERROR  - CHARACTER:  (output) A description of the error,
                           suitable for output to the user.
                           Should be at least 32 characters long.
*/

void
mtmess_(result,mtsubr,error,mtsubrlen,errorlen)
int *result;
char *mtsubr,*error;
int mtsubrlen,errorlen;
{
   char *strcat(),*strncat(),*strncpy(),*sprintf();
   int strncmp();

#define MRTMAX 19           /* Number of possible result codes */
#define RESMAX 7            /* Number of routines */

     /* List of possible subroutines */

     static char mtsubs[MRTMAX][6+1] = {
        "MTOPEN","MTCLOS","MTNOOP","MTSERM",
        "MTNAME","MTREAD","MTSKRE","MTRDBK",
        "MTWRIT","MTWEOF","MTSKFI","MTREWI",
        "MTUNLD","MTPREP","MTSENS","MTSETW",
        "MTOPID","MTGETN","MTMOUN"
     };

     /* List of possible error messages */

     static char errors[MRTMAX][32+1] = {
        "No available tape drive         ",    /*  1 */
        "No available I/O channel        ",    /*  2 */
        "No available event flag         ",    /*  3 */
        "Error mode unchanged            ",    /*  4 */
        "Name had to be truncated        ",    /*  5 */
        "I/O error                       ",    /*  6 */
        "Tape is offline                 ",    /*  7 */
        "End of file encountered         ",    /*  8 */
        "End of tape encountered         ",    /*  9 */
        "No record read                  ",    /* 10 */
        "Tape is write locked            ",    /* 11 */
        "Illegal density                 ",    /* 12 */
        "Volume is invalid               ",    /* 13 */
        "Wait mode unchanged             ",    /* 14 */
        "Tape not allocated/mounted      ",    /* 15 */
        "Device allocated to another user",    /* 16 */
        "Illegal device name             ",    /* 17 */
        "Unexpected system service code  ",    /* 18 */
        "Unable to translate TAPES       "     /* 19 */
     };

/*
      Table to give error message corresponding to result
      code for each routine.
*/
      static int lookup[MRTMAX][RESMAX] = {
                   1, 2, 3, 0, 0, 0,18,     /* MTOPEN */
                   0, 0, 0, 0, 0, 0,18,     /* MTCLOS */
                   0, 0, 0, 0, 0, 0,18,     /* MTNOOP */
                   4, 0, 0, 0, 0, 0,18,     /* MTSERM */
                   5, 0, 0, 0, 0, 0,18,     /* MTNAME */
                   6, 7, 8, 9,10, 0,18,     /* MTREAD */
                   6, 7, 8, 9,10, 0,18,     /* MTSKRE */
                   6, 7, 8, 0, 0, 0,18,     /* MTRDBK */
                   6, 7,11, 9, 0, 0,18,     /* MTWRIT */
                   6, 7,11, 9, 0, 0,18,     /* MTWEOF */
                   6, 7, 0, 9,10, 0,18,     /* MTSKFI */
                   6, 7, 0, 0, 0, 0,18,     /* MTREWI */
                   6, 7, 0, 0, 0, 0,18,     /* MTUNLD */
                   6, 7,12, 0, 0, 0,18,     /* MTPREP */
                   6, 7,13, 0, 0, 0,18,     /* MTSENS */
                  14, 0, 0, 0, 0, 0,18,     /* MTSETW */
                   1, 7, 3,15,16,17,18,     /* MTOPID */
                   1,19, 0, 0, 0, 0,18,     /* MTGETN */
                   0, 7, 0,15,16,17,18      /* MTMOUN */
      };

   int isub,messge,len;
   char msg[33];

   /* Search for valid subroutine name */

   for (isub=0; isub<MRTMAX; isub++) {
      if (strncmp(mtsubr,&mtsubs[isub][0],mtsubrlen) == 0)
         break;
      else if (isub == MRTMAX-1) {
         (void) sprintf(msg,"Illegal routine name (");
         len = strlen(msg)+mtsubrlen;
         (void) strncat(msg,mtsubr,mtsubrlen);
         msg[len] = ')';
         msg[len+1] = '\0';
         (void) strncpy(error,msg,errorlen);
         return;
      }
   }
   if (*result == 0) {
      (void) strncpy(error,"OK",errorlen);
      return;
   }
   if (*result < 1 || *result > RESMAX) {
         (void) sprintf(msg,"Illegal result code (%d)",*result);
         (void) strncpy(error,msg,errorlen);
         return;
   }

   /* Got valid subroutine name and result code, lookup msg # */

   messge = lookup[isub][(*result)-1] -1; /* C indices start at 0 */
   if (messge < 0) {
      msg[0] = '\0';
      (void) strncat(msg,mtsubr,mtsubrlen);
      (void) sprintf(msg+mtsubrlen," shouldn't give code (%d)",*result);
      (void) strncpy(error,msg,errorlen);
      return;
   } else {
      (void) strncpy(error,&errors[messge][0],errorlen);
   }
}

/*
2 MTSEOF
IF (MTSEOF(RESULT)) THEN ..  -- To test if the last error code returned
==========================      represented an end-of-file error.  This
is the most common error condition that occurs, and provision of this
function (note, a LOGICAL FUNCTION, which will have to be declared as
LOGICAL in any calling routine) means that if this is the only error
code tested for explicitly, a routine need not bother with including the
file giving the MTPCKG error codes.

      RESULT - INTEGER:  (input) The code returned by an MTPCKG read
                         routine.

      MTEOST - LOGICAL:  (function value) True if RESULT=MTREOF, 
                         false otherwise.
*/

mtseof_(result)
int *result;
{
   if (*result == MTREOF)
      return(TRUE);
   else
      return(FALSE);
}

/*
2 MTVMER
CALL MTVMER(FLAGS,STATUS,ERROR)  -- In some cases, the precise reason
==============================      for the failure of an MTPCKG
routine will not be clear from the RESULT code it returns (routines
returning MTRUNX, for example).  Generally, these unexpected errors
follow from a VMS system service call that has failed.  MTPCKG
maintains the last system service error code that it encounters, and
this can be accessed via MTVMER.

      FLAGS   - INTEGER:   (input) The bits set in flags control the
                           format of the VMS error message returned
                           in ERROR.  If the least significant bit
                           is bit 0, the bits have the following
                           meaning -
                           0  set => Include message text
                           1  set => Include error identification
                           2  set => Include severity code
                           3  set => Include facility code
                           So if FLAGS=1 only the message text is 
                           output; if FLAGS=3 the message text and
                           the message identification are output.

      STATUS  - INTEGER:   (output) The last VMS error code encountered
                           by MTPCKG as a result of a system service
                           call.

      ERROR   - CHARACTER  (output) A description of the error, formatted
                           as specified by the value of FLAGS.
                           Should be about 64 characters long to 
                           accomodate most error messages.
*/

void
mtvmer_(flags,status,error,errorlen)
int *flags,*status;
char *error;
int errorlen;
{
   char *strncpy();

   int i;

   for (i=0; i<errorlen; i++)
      error[i] = ' ';
   error[errorlen] = '\0';
   if (mtunixerr == 0)
      *status = 0;
   else {
      if (mtunixerr > 0 && mtunixerr < sys_nerr)
         (void) strncpy(error,sys_errlist[mtunixerr],errorlen);
      *status = mtunixerr;
   }
}

/* 
  mtgetstat

  Internal routine that saves last unix error number, gets status from
  tape drive and returns the corresponding MTPCKG error code in RESULT.

  Parameters      ("<" input, ">" output)

  (<) ID          (Integer,ref) File descriptor to get status of.
  (>) RESULT      (Integer,ref) MTPCKG error corresponding to unix error.
  (<) CODE        (Integer,val) 1 = READ error codes, 2 = WRITE error codes

  History:

  16th Mar 1988.  CKL / CIT.  Original.
*/

void
mtgetstat(id,result,code)
int *id,*result,code;
{
      int ioctl();

      int ioerr;
         
      mtunixerr = errno;           /* save unix error number */
      *result = 0;
      ioerr = ioctl(mtfd[*id],MTIOCGET,&opstat);
      if (ioerr < 0)
         *result = MTRUNX;
      else {
         if (!(opstat.mt_dsreg & MT_ONL))      /* Not Online */
            *result = MTROFL;
         if (opstat.mt_dsreg & MT_EOT)  /* End-Of-Tape */
            *result = MTREOT;
         if (opstat.mt_dsreg & MT_TM)   /* Tape Mark(End-of-File) */
            *result = MTREOF;
         if (!(opstat.mt_dsreg & MT_RDY) &&
                   code == 1)                /* Operation incomplete ? */
            *result = MTRNOR;
         if ((opstat.mt_dsreg & MT_FPT) &&
                   code == 2)                /* Write Protect Status */
            *result = MTRWLK;
      }
}
