      PROGRAM ADDCLO
C-----------------------------------------------------------------------
C ADDCLO creates a copy of a MERGE-format file, computing the closure
C phases from the visibility phases in the input file. If the input file
C already includes closure phases, they are ignored.  Alternatively,
C the program can be used to delete the closure phases from a file
C (specify OPCODE=D).
C
C Algorithm: The program computes closure phases for all possible
C station triangles LMN (L < M < N) for which baselines LM, MN, and LN
C are present in the input file.  The closure phase CLPHS(LMN) for
C triangle LMN is defined in terms of the visibility phases PHS(LM), 
C PHS(MN), and PHS(LN) on the constituent baselines by
C        CLPHS(LMN) = PHS(LM) + PHS(MN) - PHS(LN),
C and the error estimate for the closure phase is the quadrature sum
C of the error estimates of the visibility phases.  The closure phase
C is reduced modulo 360 degrees to the range -180 to +180 degrees.
C The closure phase is undefined if any of the constituent visibility
C phases is absent or deleted (flagged).
C
C Parameters:
C  INPUT = input file name
C  OUTPUT = output file name
C  OPCODE = "C" to compute closure phases (default); "D" to delete
C            closure phases
C  HISTORY = text to be written in header
C
C Limitations:
C   Maximum number of stations is 40.
C
C Author:
C   T. J. Pearson, 105-24 Caltech, Pasadena California 91125.
C
C Language:
C   Standard Fortran-77 except for non-standard
C   keywords in OPEN and CLOSE statements.
C
C Subroutines required:
C   ERROR, KEYIN, LEN1, LOOKUP, MKHIST, UPCASE.
C
C History:
C   Version 1.0:  1986 Mar 17 - new program.
C   Version 1.1:  1986 Apr  1 - correct error in output parm REC1(7).
C   Version 1.2:  1986 Nov 15 - omit unnecessary history records.
C   Version 1.3:  1987 Dec  3 - use MKHIST subroutine.
C   Version 1.4:  1988 Jun 21 - use VLBOPE subroutine.
C   Version 1.5:  1990 Nov 13 - remove unused variables.
C   Version 1.6:  1990 May 15 - increase to 40 stations.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER INC,OUTC,INDAT,OUTDAT,MAXS,MAXB,MAXT,NPARS
C            Program version and date:
      PARAMETER (PROGNM='ADDCLO')
      PARAMETER (VERSN='1.6 - 1991 May 15')
C            Fortran unit numbers used:
      PARAMETER (INC=5,OUTC=6,INDAT=8,OUTDAT=9)
C            Maximum numbers of stations, baselines, and triangles:
      PARAMETER (MAXS=40)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C            Number of input parameters:
      PARAMETER (NPARS=32)
C-----------------------------------------------------------------------
C Variables.
C
      CHARACTER*8   STNAME(MAXS)
      CHARACTER*16  SOURCE, OPCODE
      CHARACTER*80  HISTRY, ADDHIS(3)
      CHARACTER*255 INDSN, OUTDSN
      INTEGER   IUT, IS, IB, IT, NS, NB, NT, I, J, IER, MODE
      INTEGER   IZ, IY, NHIST, NNEW, LIN, LOUT, I1, I2, I3, P1, P2, P3
      INTEGER   NVER, NDWPB, STFLAG
      INTEGER   NBI(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      INTEGER   CONST(MAXT,3)
      REAL      INREC(7*MAXB), CLPHS(MAXT), CLERR(MAXT)
      DOUBLE PRECISION STPARM(9)
      DOUBLE PRECISION RA, DEC, RAP, DECP, GASTM, FREQ
      DOUBLE PRECISION TINTG, AVG, BW, FLUX
      DOUBLE PRECISION PARS(NPARS), VALS(NPARS), ENDMRK
C
C Functions called.
C
      INTEGER   LEN1, LOOKUP, VLBOPE
C-----------------------------------------------------------------------
      DATA  PARS/ 5HInput,   9*1H , 6HOUtput,  9*1H , 7HHistory,9*1H ,
     1            6HOPcode, 1H  /
      DATA  VALS/ 8H***UNSET,9*1H , 8H***UNSET,9*1H , 10*1H , 1HC, 1H  /
      DATA  ENDMRK/ 1H/ /
C-----------------------------------------------------------------------
  650 FORMAT (' - Computing',I5,' closure phases.')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//' computes/deletes closure phases')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
  100 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT('Parameters (INPUT, OUTPUT, OPCODE=C or D, HISTORY):')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=21,30)
      WRITE (OPCODE,'(2A8)')  (VALS(I),I=31,32)
      CALL UPCASE(OPCODE)
      IF (OPCODE.NE.'C' .AND. OPCODE.NE.'D') THEN
          CALL PUTOUT('Invalid OPCODE specified: '//OPCODE)
          CALL PUTOUT('OPCODE must be "C" or "D"')
          GOTO 100
      END IF
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 100
      LIN = LEN1(INDSN)
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          GOTO 100
      END IF
      LOUT = LEN1(OUTDSN)
C
C Make the new history records.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      IF (OPCODE.EQ.'D') THEN
          ADDHIS(2) = '- Closure phases deleted (OPCODE=D)'
      ELSE
          ADDHIS(2) = '- Closure phases computed (OPCODE=C)'
      END IF
      IF (HISTRY.NE.' ') THEN
          NNEW = 3
          ADDHIS(3) = '- '//HISTRY(1:78)
      ELSE
          NNEW = 2
      END IF
C
C Print the new history records.
C
      CALL PUTOUT(' ')
      CALL PUTOUT('Input file: '//INDSN(1:LIN))
      CALL PUTOUT('Output file: '//OUTDSN(1:LOUT))
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
C
C Read the first three header records. Merge format level must be 1.
C
      READ (INDAT,END=190,ERR=190) (REC1(J),J=1,20)
      IF (REC1(1).EQ.1) THEN
          NVER = 1
          NDWPB = 4
      ELSE IF (REC1(1).EQ.2) THEN
          NVER = 2
          NDWPB = 7
      ELSE
          GOTO 190
      END IF
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) GOTO 190
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (NB.GT.0) READ (INDAT,END=190,ERR=190) (NBI(IB),IB=1,NB)
      IF (NT.GT.0) READ (INDAT,END=190,ERR=190) (NTI(IT),IT=1,NT)
C
C Find out what closure phases can be calculated, and how to calculate 
C them.  Array CONST gives the baseline numbers of the constituent
C baseline of each closure triangle.
C
      NT = 0
      IF (OPCODE.EQ.'C') THEN
        DO 130 I3=3,NS
          DO 120 I2=2,I3-1
              DO 110 I1=1,I2-1
                  P1 = LOOKUP(100*I1 + I2, NBI, NB)
                  P2 = LOOKUP(100*I2 + I3, NBI, NB)
                  P3 = LOOKUP(100*I1 + I3, NBI, NB)
                  IF (P1.NE.0 .AND. P2.NE.0 .AND. P3.NE.0) THEN
                      NT = NT+1
                      NTI(NT) = 10000*I1 + 100*I2 + I3
                      CONST(NT,1) = P1
                      CONST(NT,2) = P2
                      CONST(NT,3) = P3
                  END IF
  110         CONTINUE
  120     CONTINUE
  130   CONTINUE
        WRITE (OUTC,650) NT
      END IF
      REC1(6) = NT
      REC1(2) = REC1(2) + NNEW
      REC1(3) = REC1(2) + NS + 2
      IF (NB.GT.0) REC1(3) = REC1(3)+1
      IF (NT.GT.0) REC1(3) = REC1(3)+1
      REC1(7) = 1 + NDWPB*NB + 2*NT
C
C Rewrite the first three header recorders, modified to reflect
C the new closure phases.
C
      WRITE (OUTDAT) (REC1(J),J=1,20)
      IF (NB.GT.0) WRITE (OUTDAT) (NBI(IB),IB=1,NB)    
      IF (NT.GT.0) WRITE (OUTDAT) (NTI(IT),IT=1,NT)
C
C Copy the history records and add the new ones.
C
      DO 140 I=1,NHIST
          READ (INDAT,END=190,ERR=190) HISTRY
          WRITE (OUTDAT) HISTRY
  140 CONTINUE
      DO 145 I=1,NNEW
          WRITE (OUTDAT) ADDHIS(I)
  145 CONTINUE
C
C Copy the remaining header records unchanged.
C
      READ (INDAT,END=190,ERR=190) SOURCE,RA,DEC,RAP,DECP,FLUX
      WRITE (OUTDAT) SOURCE,RA,DEC,RAP,DECP,FLUX
      DO 150 IS=1,NS
          IF (NVER.EQ.1) THEN
              READ (INDAT,END=190,ERR=190) STNAME(IS), STPARM(1),
     1            STPARM(2), STPARM(3)
              DO 148 I=4,9
                  STPARM(I) = 0D0
  148         CONTINUE
              STFLAG = 0
          ELSE
              READ (INDAT,END=190,ERR=190) STNAME(IS), STPARM,
     1            STFLAG
          END IF
          WRITE (OUTDAT) STNAME(IS), STPARM, STFLAG
  150 CONTINUE
      READ (INDAT,END=190,ERR=190) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
      WRITE (OUTDAT) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C
C Read and re-write data records, computing closure phases.
C
  160 READ(INDAT,END=180,ERR=190) IUT,(INREC(J),J=1,NDWPB*NB)
      IF (IUT.LT.0) GOTO 180
      DO 170 J=1,NT
          P1 = NDWPB*CONST(J,1) - NDWPB + 3
          P2 = NDWPB*CONST(J,2) - NDWPB + 3
          P3 = NDWPB*CONST(J,3) - NDWPB + 3
          CLPHS(J) = AMOD(INREC(P1)+INREC(P2)-INREC(P3),360.0)
          IF (CLPHS(J).GT.180.0)  CLPHS(J) = CLPHS(J)-360.0
          IF (CLPHS(J).LE.-180.0) CLPHS(J) = CLPHS(J)+360.0
          P1 = P1+1
          P2 = P2+1
          P3 = P3+1
          IF (INREC(P1).LE.0.0 .OR. INREC(P2).LE.0.0 .OR.
     1        INREC(P3).LE.0.0) THEN
              CLPHS(J) = 0.0
              CLERR(J) = 0.0
          ELSE
              CLERR(J) = SQRT(INREC(P1)**2+INREC(P2)**2+INREC(P3)**2)
          END IF
  170 CONTINUE
      WRITE (OUTDAT) IUT,(INREC(J),J=1,NDWPB*NB),
     1               (CLPHS(J),CLERR(J),J=1,NT)
      GOTO 160
C
C Write final record and quit.
C
  180 WRITE (OUTDAT) -1,(0.0,J=1,NDWPB*NB+2*NT)
      CLOSE (UNIT=INDAT)
      CLOSE (UNIT=OUTDAT)
      STOP
C
C Error exit. Delete the nascent output file.
C
  190 CALL PUTOUT('Something wrong with input MERGE file.')
      CLOSE (UNIT=OUTDAT, STATUS='DELETE')
      CLOSE (UNIT=INDAT)
      CALL ERROR(PROGNM//' aborted')
C-----------------------------------------------------------------------
      END
