      PROGRAM AVERAG
C-----------------------------------------------------------------------
C AVERAG reads data from an input MERGE-format file, averages data
C within bins of specified length, and writes the averaged data in a
C new MERGE-format file.
C
C Algorithm: Three different algorithms can be used.
C
C Parameters:
C  INPUT = input file name
C  OUTPUT = output file name
C  METHOD = "I" (incoherent), "C" (coherent, errors from input), or
C           "S" (coherent, errors from scatter)
C  TIMESCALE = average time (seconds)
C  HISTORY = text to be written in header
C
C Limitations:
C   1. Maximum number of stations is 40.
C   2. If the input file is MERGE format 2 (i.e., it contains the
C      u,v,w coordinates) the averaging time cannot be more than
C      100 times the integration time in the input file.
C
C Author:
C   Original author: R. C. Walker.
C   Modified by S. C. Unwin, D. L. Meier, and T. J. Pearson.
C   105-24 Caltech, Pasadena, California 91125.
C
C Language:
C   Standard Fortran-77 except for name HEADX40 and %REF.
C
C History:
C   1.1  1982 Apr  1 : Coherent averaging added - RCW.
C   1.2  1982 Apr 15 : Changed weighting - RCW.
C   1.3  1982 Sep 22 : Some bugs fixed (?), old versions no good - RCW.
C   2.0  1983 Jul 20 : Extended to include merge version 2 files - DLM.
C   2.1              : Added w to ver 2 files; interpolate u,v,w - DLM.
C   2.2  1985 Mar  5 : Changed parameters for up to 20 stations - TJP.
C   2.3  1985 Apr  9 : Fixed bug: Coherent average inquiry required 
C                      'y' in uppercase only! - SCU.
C   3.0  1985 Apr 11 : Change coherent average: errors now calculated 
C                      from scatter in data, not input errors; write 
C                      averaging information into history records - SCU.
C   3.1  1986 Apr  4 : More flexible input (see 1985 Apr 9!) - SCU.
C   3.2  1986 Oct  9 : Fix rounding error in s.d. - SCU.
C   4.0  1986 Nov 15 : Revised to use parameter input - TJP.
C   4.1  1988 Jun 21 : Changes for Convex - TJP.
C   4.2  1992 Aug  7 : when errors are calculated from scatter of data,
C                      the case of only one input point must be treated
C                      specially; change to flag the point - TJP.
C   4.3  1993 Dec  4 : increase to 40 stations.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
C     -- program version and date
      PARAMETER (PROGNM='AVERAGE')
      PARAMETER (VERSN='4.3 - 1993 Dec 4')
      INTEGER MS, MB, MC, MAXWPB, MR, MJB
      PARAMETER (MS=40)
      PARAMETER (MB=MS*(MS-1)/2)
      PARAMETER (MC=MS*(MS-1)*(MS-2)/6)
      PARAMETER (MAXWPB=7)
      PARAMETER (MR=MAXWPB*MB+2*MC)
C     -- MJB = max number of data records that can be accumulated for
C     -- Version 2 data files
      PARAMETER (MJB=100)
      INTEGER NPARS
C     -- number of input parameters
      PARAMETER (NPARS=33)
      INTEGER INDAT, OUTDAT, INC, OUTC
      PARAMETER (INDAT=1)
      PARAMETER (OUTDAT=2)
      PARAMETER (INC=5)
      PARAMETER (OUTC=6)
      DOUBLE PRECISION DEGRAD
      PARAMETER (DEGRAD=57.29577951D0)
C-----------------------------------------------------------------------
      LOGICAL    END, COHER, SCATTR
      CHARACTER*80 HISTRY, ADDHIS(3)
      INTEGER    OUTREC, ITSTOR(MJB,MB), JB(MB), NFLAG
      INTEGER    MODE, I, INSEC, INTAVG, IER, LIN, LOUT, NNEW, IPROUT
      INTEGER    IZERO, IDAY, IYEAR, IUNITS, IFILE, N, NDWPB, NBM, NCM
      INTEGER    INREC, INAVG, IFIRST, ILAST, IT, JBI, NVER, J, K, L
      INTEGER    NSTA, NB1, NB2, NC1, NC2, NC3, ITOUT
      REAL       SECAVG, SDCOS2, SDSIN2, DOT, PH, COSPHS, SINPHS
      REAL       ERA, ERP, ERC
      REAL       TAKE(MR),SEND(MR)
      REAL       AMP(MB),ERRA(MB),COSPH(MB),SINPH(MB),ACCUM(MB)
      REAL       ERRPH(MB),COSCL(MC),SINCL(MC),ERRCL(MC)
      REAL       COSPH2(MB),SINPH2(MB)
      REAL       U(MJB,MB),V(MJB,MB),W(MJB,MB),COEFF(MJB)
      DOUBLE PRECISION XVALS(210)
      DOUBLE PRECISION  GASTM,FREQ,STNAME(MS),STX(MS),STY(MS),STZ(MS)
      DOUBLE PRECISION  SOURCE(2), RAP,DECP,FLUX
      COMMON /PARMS/STNAME,STX,STY,STZ,SOURCE,RAP,DECP,GASTM,FREQ,FLUX
      COMMON /ORDER/NSTA,NBM,NCM,NB1(MB),NB2(MB),NC1(MC),NC2(MC),
     1   NC3(MC),NVER,NDWPB
      CHARACTER*16 METHOD
      CHARACTER*7  X
      CHARACTER*255  INDSN, OUTDSN
      DOUBLE PRECISION PARS(NPARS), VALS(NPARS), ENDMRK
C
      INTEGER LEN1, VLBOPE
C-----------------------------------------------------------------------
      DATA  PARS/ 'Input',   9*' ', 'Output',  9*' ', 'History',9*' ',
     1            'Method', ' ', 'Timescal' /
      DATA  VALS/ '***UNSET',9*' ', '***UNSET',9*' ', 10*' ',
     1            '***UNSET', ' ',  '***UNSET' /
      DATA  ENDMRK/ '/' /
      DATA  XVALS/210*0.D0/
C-----------------------------------------------------------------------
 1060 FORMAT ('- ',A,' average, ',I5,' sec; errors from ',A, 
     1        ' (METHOD=',A,')')
 1080 FORMAT (2A8,' Averaging of ',I6,' records completed;',
     1        3X,I6,' output records')
 1090 FORMAT ('++WARNING++ ',I5,' visibilities were flagged ',
     1        '(error could not be calculated)')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL PUTOUT(PROGNM//' computes time-averaged visibility data')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
      MODE = 0
   10 IF (MODE.NE.0) CALL ERROR('Bad input parameters in '//PROGNM)
      CALL PUTOUT(' ')
      CALL PUTOUT(
     1'Parameters (INPUT, OUTPUT, TIME (sec), METHOD=I/C/S, HISTORY):')
      MODE = 0
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      WRITE (INDSN,'(10A8)') (VALS(I),I=1,10)
      WRITE (OUTDSN,'(10A8)') (VALS(I),I=11,20)
      WRITE (HISTRY,'(10A8)') (VALS(I),I=21,30)
      WRITE (METHOD,'(2A8)')  (VALS(I),I=31,32)
      CALL UPCASE(METHOD)
      IF (METHOD(1:1).EQ.'C') THEN
          COHER = .TRUE.
          SCATTR = .FALSE.
      ELSE IF (METHOD(1:1).EQ.'I') THEN
          COHER = .FALSE.
          SCATTR = .FALSE.
      ELSE IF (METHOD(1:1).EQ.'S') THEN
          COHER = .TRUE.
          SCATTR = .TRUE.
      ELSE
          CALL PUTOUT('++WARNING++ Invalid METHOD specified: '//METHOD)
          GOTO 10
      END IF
      INSEC = NINT(VALS(33))
      SECAVG = INSEC
      INTAVG = SECAVG * 60
C
C Open input Merge file.
C
      IER = VLBOPE(INDAT, INDSN, 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 10
      LIN = LEN1(INDSN)
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, OUTDSN, 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) GOTO 10
      LIN = LEN1(INDSN)
      LOUT = LEN1(OUTDSN)
C
C Make the new history records.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      IF (SCATTR) THEN
          X = 'scatter'
      ELSE
          X = 'input'
      END IF
      IF (COHER) THEN
          WRITE (ADDHIS(2),1060) 'Coherent', INSEC, X, METHOD(1:1)
      ELSE
          WRITE (ADDHIS(2),1060) 'Incoherent', INSEC, X, METHOD(1:1)
      END IF
      IF (HISTRY.NE.' ') THEN
          NNEW = 3
          ADDHIS(3) = '- '//HISTRY(1:78)
      ELSE
          NNEW = 2
      END IF
C
C Print the new history records.
C
      CALL PUTOUT(' ')
      CALL PUTOUT('Input file:  '// INDSN(1:LIN))
      CALL PUTOUT('Output file: '// OUTDSN(1:LOUT))
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
C
C Process header records.
C
      IPROUT = 0
      XVALS(45) = SECAVG
      CALL HEADX40(.FALSE.,INDAT,IZERO,IDAY,IYEAR,%REF(ADDHIS(1)),
     +        NNEW,OUTDAT,IUNITS,XVALS,IFILE,IPROUT)
      IF (IFILE.EQ.1) CALL ERROR ('Bad header')
C
C Set size parameters.
C
      N = NDWPB*NBM  + 2*NCM
C
C Initialize counters.
C
      INREC = 0
      OUTREC = 0
      INAVG = 0
      IFIRST = -1
      ILAST = -1
      NFLAG = 0
      END = .FALSE.
C
C Process data.
C
C     Read record.
C
  100 CONTINUE
      READ (INDAT,END=110) IT,(TAKE(I),I=1,N)
      IF (IT.EQ.-1) GO TO 110
      INREC = INREC + 1
      IF (IT.GE.IFIRST.AND.IT.LE.ILAST) GO TO 300
         IF (INREC.EQ.1) GO TO 200
         GO TO 120
C
C     Prepare output.
C
  110 END = .TRUE.
      IF (END.AND.INAVG.EQ.0) GO TO 900
  120 OUTREC = OUTREC + 1
C
C     Output section.
C
      ITOUT = IFIRST + INTAVG/2
      DO 129 I=1,N
          SEND(I) = 0.0
  129 CONTINUE
      DO 130 I=1,NBM
          K = NDWPB*(I-1)
C                                      !   ***** COHERENT AVERAGE
          IF (COHER.AND.(ERRA(I).NE.0. .AND. ERRPH(I).NE.0.)) THEN
              SEND(K+1) = SQRT( (COSPH(I)/ACCUM(I))**2 +
     *                          (SINPH(I)/ACCUM(I))**2 )
              SEND(K+2) = SQRT( 1./ERRA(I) )
              SEND(K+3) = ATAN2( SINPH(I),COSPH(I) ) * DEGRAD
              SEND(K+4) = SQRT( 1./ERRPH(I) )
              IF (SCATTR) THEN
                IF (ACCUM(I).GT.1) THEN
C                   ! calculate standard deviation
                  SDCOS2 = ( ACCUM(I)*COSPH2(I) - COSPH(I)**2 ) /
     *                     ( ACCUM(I)*(ACCUM(I)-1) )
                  SDSIN2 = ( ACCUM(I)*SINPH2(I) - SINPH(I)**2 ) /
     *                     ( ACCUM(I)*(ACCUM(I)-1) )
C        Amplitude error = sample s.d.;  phase error = amp err / amp
                  IF ( (SDCOS2+SDSIN2)/(2*ACCUM(I)) .GT. 1E-16 ) THEN
                      SEND(K+2) = SQRT((SDCOS2+SDSIN2)/(2*ACCUM(I)))
                      SEND(K+4) = SEND(K+2)*DEGRAD/SEND(K+1)
                  END IF
                ELSE
C                    ! only one point: cannot calculate sd; flag it
                  SEND(K+2) = -1.0*ABS(SEND(K+2))
                  SEND(K+4) = -1.0*ABS(SEND(K+4))
                  NFLAG = NFLAG+1
                END IF
              END IF
C                                      !   ***** INCOHERENT AVERAGE
          ELSE IF (.NOT.COHER) THEN
              IF (ERRA(I).NE.0) THEN
                  SEND(K+1) = AMP(I) / ERRA(I)
                  SEND(K+2) = SQRT(1/ERRA(I))
              END IF
              IF(.NOT.((SINPH(I).EQ.0.0.AND.COSPH(I).EQ.0.0) .OR.
     +               (ERRPH(I).EQ.0.0))) THEN
                  SEND(K+3) = ATAN2(SINPH(I),COSPH(I)) * DEGRAD
                  SEND(K+4) = SQRT(1/ERRPH(I))
              END IF
         END IF
C
C     Interpolate (u,v,w) if this is a version-2 file.
C
          JBI = JB(I)
          IF ((NVER.EQ.2) .AND. (JBI.GT.0)) THEN
              CALL LAGRAN(ITOUT,ITSTOR(1,I),JBI,COEFF)
              SEND(K+5) = DOT(COEFF,U(1,I),JBI)
              SEND(K+6) = DOT(COEFF,V(1,I),JBI)
              SEND(K+7) = DOT(COEFF,W(1,I),JBI)
          END IF
  130 CONTINUE
C
C     Compute averaged closure phases (if necessary).
C
      IF (NCM.GT.0) THEN
          DO 140 I=1,NCM
              K = NDWPB*NBM + 2*(I-1)
              IF ((SINCL(I).EQ.0.0.AND.COSCL(I).EQ.0.0).OR.
     +                       (ERRCL(I).EQ.0.0))       GO TO 140
              SEND(K+1) = ATAN2(SINCL(I),COSCL(I)) * DEGRAD
              SEND(K+2) = SQRT(1/ERRCL(I))
  140     CONTINUE
      END IF
C
C     Write out the new record.
C
      WRITE (OUTDAT) ITOUT,(SEND(I),I=1,N)
      IF (END) GOTO 900
C
C     Reset accumulation.
C
  200 INAVG = 0
      IF (NVER.EQ.2) THEN
          DO 202 J=1,MJB
              COEFF(J) = 0.0
  202     CONTINUE
      END IF
      IFIRST = IT/INTAVG
      IFIRST = IFIRST * INTAVG
      ILAST = IFIRST + INTAVG - 1
      DO 210 I=1,NBM
          ACCUM(I) = 0.0
          AMP(I) = 0.0
          ERRA(I) = 0.0
          COSPH(I) = 0.0
          SINPH(I) = 0.0
          ERRPH(I) = 0.0
          COSPH2(I) = 0.0
          SINPH2(I) = 0.0
          IF (NVER.EQ.2) THEN
              JB(I) = 0
              DO 206 J=1,MJB
                  ITSTOR(J,I) = 0
                  U(J,I) = 0.0
                  V(J,I) = 0.0
                  W(J,I) = 0.0
  206         CONTINUE
          END IF
  210 CONTINUE
      DO 220 I=1,NCM
          COSCL(I) = 0.0
          SINCL(I) = 0.0
          ERRCL(I) = 0.0
  220 CONTINUE
C
C     Accumulate.
C
  300 INAVG = INAVG + 1
C     -- accumulate baseline data --
      DO 310 I = 1,NBM
         K = NDWPB*(I-1)
         IF(COHER) THEN                
C            !   ***** COHERENT AVERAGE
            IF(TAKE(K+2).GT.0. .AND. TAKE(K+4).GT.0.) THEN
               PH = TAKE(K+3)/DEGRAD
               COSPHS = COS(PH)
               SINPHS = SIN(PH)
               COSPH(I) = COSPH(I) + TAKE(K+1)*COSPHS
               SINPH(I) = SINPH(I) + TAKE(K+1)*SINPHS
               ERRA(I)  = ERRA(I)  + 1./(TAKE(K+2)**2)
               ERRPH(I) = ERRPH(I) + 1./(TAKE(K+4)**2)
               ACCUM(I) = ACCUM(I) + 1.0
               COSPH2(I)= COSPH2(I) + (TAKE(K+1)*COSPHS)**2
               SINPH2(I)= SINPH2(I) + (TAKE(K+1)*SINPHS)**2
               IF (NVER.EQ.2) THEN
                  J = JB(I) + 1
                  IF (J.GT.MJB) CALL ERROR
     1               (' Too many data points in averaging interval ')
                  ITSTOR(J,I) = IT
                  U(J,I)       = TAKE(K+5)
                  V(J,I)       = TAKE(K+6)
                  W(J,I)       = TAKE(K+7)
                  JB(I) = J
               END IF
            END IF
         ELSE                          
C           !   ***** INCOHERENT AVERAGE
            IF(TAKE(K+2).LE.0.) GO TO 304
               ERA = 1/(TAKE(K+2)*TAKE(K+2))
               AMP(I) = AMP(I) + TAKE(K+1)*ERA
               ERRA(I) = ERRA(I) + ERA
               IF (NVER.EQ.2) THEN
                  J = JB(I) + 1
                  IF (J.GT.MJB) CALL ERROR
     1               (' Too many data points in averaging interval ')
                  ITSTOR(J,I) = IT
                  U(J,I)       = TAKE(K+5)
                  V(J,I)       = TAKE(K+6)
                  W(J,I)       = TAKE(K+7)
                  JB(I) = J
               END IF
  304       IF(TAKE(K+4).LE.0.) GO TO 310
               ERP = 1/(TAKE(K+4)*TAKE(K+4))
               PH = TAKE(K+3)/DEGRAD
               COSPH(I) = COSPH(I) + COS(PH) * ERP
               SINPH(I) = SINPH(I) + SIN(PH) * ERP
               ERRPH(I) = ERRPH(I) + ERP
         END IF
  310 CONTINUE
C     -- accumulate closure phase data --
      DO 320 I=1,NCM
          K = NDWPB*NBM + 2*(I-1)
          IF (TAKE(K+2) .GT. 0.0) THEN
              ERC = 1/(TAKE(K+2)*TAKE(K+2))
              PH  = TAKE(K+1)/DEGRAD
              COSCL(I) = COSCL(I) + COS(PH)*ERC
              SINCL(I) = SINCL(I) + SIN(PH)*ERC
              ERRCL(I) = ERRCL(I) + ERC
          END IF
  320 CONTINUE
C
      GOTO 100
C
C Termination: write final output data record, close file, and send 
C message to user.
C
  900 ITOUT = -1
      WRITE (OUTDAT) ITOUT,(0.0,I=1,N)
      CLOSE (UNIT=OUTDAT)
      CLOSE (UNIT=INDAT)
      WRITE (HISTRY,1080) SOURCE,INREC,OUTREC
      CALL SSPACE(HISTRY,L)
      CALL PUTOUT(HISTRY(1:L))
      IF (NFLAG.GT.0) THEN
          WRITE (HISTRY,1090) NFLAG
          CALL SSPACE(HISTRY,L)
          CALL PUTOUT(HISTRY(1:L))
      END IF
C-----------------------------------------------------------------------
      END

      SUBROUTINE LAGRAN (IXWANT,IX,NVAR,COEFF)
      INTEGER IXWANT, NVAR, IX(NVAR)
      REAL    COEFF(NVAR)
C-----------------------------------------------------------------------
C Compute Lagrangian polynomial coefficients given an independent
C integer variable vector IX and a desired position IXWANT.
C-----------------------------------------------------------------------
      INTEGER I, II, IXI, IXII
      REAL COEFFI
C-----------------------------------------------------------------------
      DO 20 I=1,NVAR
          IXI = IX(I)
          COEFFI = 1.
          DO 10 II=1,NVAR
              IXII = IX(II)
              IF (I.NE.II) COEFFI = COEFFI *
     1                              FLOAT(IXWANT-IXII)/FLOAT(IXI-IXII)
   10     CONTINUE
          COEFF(I) = COEFFI
   20 CONTINUE
C-----------------------------------------------------------------------
      END

      REAL FUNCTION DOT (A,B,N)
      INTEGER N
      REAL  A(N), B(N)
C-----------------------------------------------------------------------
C Dot product of two vectors A and B.
C-----------------------------------------------------------------------
      INTEGER M
C-----------------------------------------------------------------------
      DOT = 0.0
      DO 10 M=1,N
          DOT = DOT + A(M)*B(M)
   10 CONTINUE
C-----------------------------------------------------------------------
      END
