      PROGRAM CAL
C-----------------------------------------------------------------------
C CAL applies amplitude calibration corrections to a dataset in standard
C MERGE format, to convert correlation coefficients into correlated flux
C densities.
C
C Author:
C  T.J. Pearson, California Institute of Technology.
C
C Program History:
C  1.0:  1979 Jun  3 - new program [TJP].
C  2.0:  1979 Sep 29 - change printout format; add
C                      parameter TCAL.
C  3.0:  1980 Jan 28 - make unbiasing correction optional;
C                      default CALERR=0.0; add BRIEF option.
C  3.1:  1982 Aug 30 - correct bug in testing for negative
C                      antenna temps.
C  3.2:  1982 Oct 20 - allow comments (!) in TSYS/TANT tables;
C                      change format of history record.
C  3.3:  1982 Dec 22 - remove CALERR, add BDEF[ault], allow
C                      superfluous stations; add mean
C                      visibility and noise calculation.
C  3.4:  1983 Mar 20 - correct bug in noise calculation for
C                      negative BW.
C  3.5:  1984 Feb 14 - correct bug to allow up to 2500 system
C                      temperatures per station (formerly died
C                      after 250).
C  4.0:  1985 Mar  7 - Add GCNRAO gain curve for NRAO 140-ft 
C                      telescope (after John Benson). Add
C                      execution summary to listing. Put
C                      scratch file on scratch disk. Suppress
C                      echo of parameter file to SYS$OUTPUT.
C                      Correct bug in and improve format of
C                      map noise estimates. Format TSYS data in
C                      columns; move bfac to summary table.
C                      Reformat source code to be closer to
C                      Fortran-77 [TJP].
C  4.1:  1986 Aug 28 - Increase from 15 to 20 stations.
C  4.2:  1987 Dec 16 - Standardization for Convex.
C  4.3:  1988 Jun  2 - More changes for Convex.
C  4.4:  1990 Feb  1 - BASELINE group was handled incorrectly
C                      if stations were in reverse order.
C                      Change param B to BFAC to avoid conflict
C                      with station name.
C  5.0:  1990 Sep  4 - allow tables in external files; more
C                      informative error messages; change BRIEF
C                      to LONG and invert meaning; do not
C                      echo Tsys when LONG omitted; different
C                      FLUX values for different sources; add
C                      DELTAT parameter; add plots [TJP].
C  5.1:  1990 Nov 14 - Add TSCURV as alternative to TSYS table.
C                      Works like GAIN group except applies to 
C                      TSYS instead of TANT. Add consistency check 
C                      for parameters and groups [DLM].
C                      Add TANTFT to TSCURV group [JER,DLM].
C  5.1:  1991 Jan 16 - new FFREAD routine [TJP].
C  5.1:  1991 Jan 30 - minmatch problems [TJP].
C  5.2:  1992 Feb 17 - improved graphs [TJP].
C  5.3:  1992 Mar  9 - Added another gain parameter to make it possible
C		       for the NRAO gain curve to be 2nd order harmonic
C		       [jdell@maggie.mit.edu].
C  5.4:  1992 Mar  9 - Added GTABLE tables [TJP].
C  5.4:  1992 Mar 31 - Correct bug [TJP].
C  5.5:  1992 Jul  2 - Display bad data when not in order [TJP].
C  5.6:  1992 Oct 29 - correct bug in application of GCNRAO [TJP].
C  5.7:  1992 Nov  6 - change GTABLEs to be tables of *degrees per Jy*
C                      and correct interpolation error [TJP].
C  5.8:  1992 Nov  9 - calibrate flagged data [TJP].
C  5.9:  1993 Mar 20 - non-fatal error if "already calibrated" [TJP].
C 5.10:  1993 Sep  1 - accept bigger TSYS tables [TJP].
C 5.11:  1993 Dec  4 - increase to 40 stations [TJP].
C-----------------------------------------------------------------------
C Constants
      CHARACTER*(*) PROGNM, VERSN
      INTEGER   INC,OUTC,PR,PS,INDAT,OUT,INPARS,INTAB
      INTEGER   NPARS,NHPARS,MAXS, MAXB,MAXT,MAXL,MAXTAB,MAXTBL
      INTEGER   BASEGR,TSYSGR,TANTGR,GAINGR,TSCVGR,NGRPS,GTABGR,NALOWD
      REAL      PI, TWOPI, PIBY2, RPDEG, RPHR, SIDR, DBC, BTHEOR
      PARAMETER (PROGNM='CAL')
      PARAMETER (VERSN='5.11 - 1993 Dec 4')
      PARAMETER (INC=5,OUTC=6,PR=3,PS=4,INDAT=8,OUT=9,INPARS=1,INTAB=2)
C            Maximum numbers of stations, baselines, and triangles:
      PARAMETER (MAXS=40)
      PARAMETER (MAXB=(MAXS-1)*MAXS/2)
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)
C            Various array sizes.
      PARAMETER (MAXL=4*MAXB+2*MAXT)
      PARAMETER (NPARS=(41+MAXS),NHPARS=47)
      PARAMETER (MAXTAB=8000)
      PARAMETER (MAXTBL=3*MAXTAB+1)
C            Group numbers.
      PARAMETER (NGRPS=6)
      PARAMETER (BASEGR=1)
      PARAMETER (TSYSGR=2)
      PARAMETER (TANTGR=3)
      PARAMETER (GAINGR=4)
      PARAMETER (TSCVGR=5)
      PARAMETER (GTABGR=6)
      PARAMETER (NALOWD=NGRPS*NPARS)
C            Mathematical constants.
      PARAMETER (PI=3.14159265359)
      PARAMETER (TWOPI=2.0*PI)
      PARAMETER (PIBY2=PI/2.0)
      PARAMETER (RPDEG=PI/180.0)
      PARAMETER (RPHR=PI/12.0)
      PARAMETER (SIDR=(PI/2592000.0)*1.0027379093)
C               (radians of ST per sec/60 of UT)
      PARAMETER (DBC=0.230258509299)
C               ( ln(10)/10 )
C            Theoretical b-factor [for noise calculation]
      PARAMETER (BTHEOR=2.5)
C-----------------------------------------------------------------------
C Variables
      INTEGER   UT, UTMIN, UTMAX, ITYPE(MAXS), TSYST(MAXTAB,MAXS)
      INTEGER   TSTYPE(MAXS)
      INTEGER   TANTT(MAXTAB,MAXS), NTSYS(MAXS), NTANT(MAXS)
      INTEGER   NGTABL(MAXS), NPOLY
      INTEGER   L, IER, INMODE, MODE, IB, I, NNEW, J, NVER, NDWPB
      INTEGER   NHIST, NS, NB, NT, IT, IS, IY, IZ, LENG, JS, NTBL
      INTEGER   STFLAG, NSC(MAXB), NNOIS, K, NREC, LAST, NPASS, IPASS
      INTEGER   IS1, IS2, IREC, ID, IDAY, IGO, NPT, LP, IGR, IP, NVP
      INTEGER   NBI(MAXB), NB1(MAXB), NB2(MAXB)
      INTEGER   NTI(MAXT)
      INTEGER   REC1(20), IDUM(8)
      LOGICAL   EQUAT,ALTAZ,EOF,DB,BAD,ORDER,BADSTN(MAXS),USED(MAXS)
      LOGICAL   UNBIAS,LONG,GCNRAO,PLOT
      LOGICAL   ALOWD(NGRPS,NPARS),TRYAGN, FLAG
      REAL      REC(MAXL),BFAC(MAXB),TSYSV(MAXTAB,MAXS)
      REAL      TANTV(MAXTAB,MAXS)
      REAL      GTABLZ(MAXTAB,MAXS), GTABLV(MAXTAB,MAXS)
      REAL      AMP,SC,SCERR,SCMAX(MAXB)
      REAL      SCTOT(MAXB),NOISQ,MPN1,MPN2,DN1,DN2
      REAL      BDEF, SRA, FAC, TCAL, HAM, AMPERR, AMPC, TFAC
      REAL      Q, GST, DELTAT
      REAL      TSYS(MAXS),TANT(MAXS),DPFU(MAXS),GC(7,MAXS)
      REAL      TSZ(MAXS),TC(6,MAXS),TANTFT(MAXS)
      REAL      GLAT(MAXS), GLON(MAXS), CC(MAXS), SS(MAXS), HA(MAXS)
      REAL      ZA(MAXS)
      REAL      FLUXES(10)
      REAL      XPLOT1, XPLOT2, YPLOT2
      REAL      XPLOT(MAXTAB), YPLOT(MAXTAB), EPLOT(MAXTAB)
      REAL      XV1, XV2, YV1, YV2, HTVP, RLAST
      DOUBLE PRECISION STX(MAXS),STY(MAXS),STZ(MAXS)
      DOUBLE PRECISION PARS(NPARS),VAL(NPARS),DEFVAL(NPARS),ENDMRK
      DOUBLE PRECISION TABLE(MAXTBL), HPARS(NHPARS), HVALS(NHPARS)
      DOUBLE PRECISION STPARM(9)
      DOUBLE PRECISION RA, DEC, RAP, DECP, GASTM, FREQ
      DOUBLE PRECISION TINTG, AVG, BW, FLUX
      CHARACTER*8   STNAME(MAXS)
      CHARACTER*16  SOURCE, FLXSRC(10)
      CHARACTER*8   T, SPARS(NPARS)
      CHARACTER*255 INDSN,OUTDSN,PARDSN,PRDSN,SCRDSN,TABDSN,PLTDSN
      CHARACTER*22  FIELD(MAXS)
      CHARACTER*80  ADDHIS(5), HISTRY, TEXT
      EQUIVALENCE (PARS,SPARS)
C-----------------------------------------------------------------------
C Functions
      INTEGER       GETIN, LEN1, VLBOPE, LOOKUP, PGBEG
      REAL          POLY, RTERP, STERP
      CHARACTER*8   TIMEC
C-----------------------------------------------------------------------
C Data statements
      DATA  PARS /'BASELINE','TSYS','TANT','GAIN','TSCURV','GTABLE',
     1            MAXS*'********',
     2            'Bfac',
     3            'FT',
     4            'TCAL',
     5            'EQUAT','ALTAZ',
     6            'DPFU',
     7            7*'POLY',
     8            'CALERR',
     9            'GCNRAO', 
     A            'TABLE', 9*' ',
     B            'DELTAT',
     C            'TSZ',
     D            6*'TSPOLY', 
     E            'TANTFT', 
     F            '????????'/
      DATA DEFVAL/6*-1D0,
     1            MAXS*-1D0,
     2            0D0,
     3            1D0,
     4            0D0,
     5            2*-1D0,
     6            0D0,
     7            1D0,6*0D0,
     8            'ignored',
     9            -1D0, 
     A            10*' ',
     B            0D0,
     C            0D0,
     D            1D0,5*0D0,
     E            1D0,
     F            32768D0/
      DATA      HPARS/10*'Fluxes',
     1            'Sources', ' ', 'Sources', ' ', 'Sources', ' ',
     2            'Sources', ' ', 'Sources', ' ', 'Sources', ' ',
     3            'Sources', ' ', 'Sources', ' ', 'Sources', ' ',
     4            'Sources', ' ',
     5            'History',9*' ',
     6            'Unbias',   'Long',
     7            'BDefault', 'BW',    'Tavg',
     8            'IFno',     'POLno' /
      DATA      HVALS/10*0D0,
     1            20*' ',
     5            10*' ',
     6            -1D0, -1D0,
     7            '***UNSET',  1.8D6,  2.0D0,
     8            1D0,         1D0 /
      DATA      ITYPE/MAXS*0/,TSTYPE/MAXS*0/
      DATA      NTSYS/MAXS*0/,NTANT/MAXS*0/
      DATA      ALOWD/NALOWD*.FALSE./
      DATA      ENDMRK/'/'/
C-----------------------------------------------------------------------
  390 FORMAT('Program CAL   Version ',A//
     1       'Input file: ',A/
     2       'Output file: ',A/)
  395 FORMAT('++WARNING++ Flux density not specified; S=1 assumed'/
     1  3X,'Calibration will not be affected if gain curves are used,'/
     2  3X,'but printed antenna temperatures will be wrong')
  421 FORMAT(//1X,A)
  430 FORMAT(//' Station coordinates:'//T46,'Geocentric',T66,
     1     'Geocentric'/T46,'latitude',T66,'longitude'/)
  440 FORMAT(T10,A8,T46,F9.5,T66,F10.5)
  441 FORMAT(T10,A8,T46,'*** not available ***')
  470 FORMAT(//' System temperature data:')
  480 FORMAT(/T10,A8,' *** Data not provided ***')
  490 FORMAT(/T10,A8, T25,'U.T.',T39,'T (K)'/)
  502 FORMAT(T20,I3,1X,A8,T35,F12.4)
  503 FORMAT(1X,I5,'>',I6,1X,A8,F7.1)
  501 FORMAT(/' ++WARNING++ Entries are not in increasing order for ',A)
  510 FORMAT(//' Antenna temperature data:')
  520 FORMAT(//' Gain curves:'//T30,'K/Jy',T60,
     1  'Relative gain polynomial coefficients'//T45,'x**0',T58,'x**1',
     2  T71,'x**2',T84,'x**3',T97,'x**4',T110,'x**5',T123,'x**6'/)
  521 FORMAT(/T60,'x is HA (hours) for "EQUAT" telescopes,'
     1       /T60,'     ZA (deg)   for "ALTAZ" telescopes.'/)
  530 FORMAT( T10,A8,T20,A8,T30,F8.3,T40,1P,7G13.4)
  531 FORMAT( T10,A8,T20,A16)
  542 FORMAT( T20,'Flux Scale Factor=',F8.4)
  550 FORMAT(' ++WARNING++ Negative or zero antenna temperature: ',
     1             A8,1X,A8,' ZA =',F6.1,' HA =',F6.1)
  560 FORMAT(//'Tsys curves:'//T30,'K at Zenith',T60,
     1    'Relative Tsys (off) polynomial coefficients'//T45,'x**0',
     2    T58,'x**1',T71,'x**2',T84,'x**3',T97,'x**4',T110,'x**5')
  581 FORMAT(//'Summary of calibration results:'
     1       //T5,'Baseline',T23,'Bfac',T34,'Points',T43,'Correl Flux',
     2         T63,'Visibility'/T43,'Mean   Max',T63,'Mean   Max'/)
  582 FORMAT(T5,A8,1X,A8,T23,F6.3,T35,I4,T40,2F8.2,T60,2F8.3)
  583 FORMAT(//'Estimates of rms noise on map:'//
     1         'The first estimate in each pair applies to a map ',
     2         'made with natural weighting;'/
     3         'the second applies to a map with natural weighting ',
     4         'and additional weighting of'/
     5         'each measurement by its inverse variance.')
  584 FORMAT(/ '(a) Derived from system temperatures, assuming',
     1         ' bandwidth = ',F9.3,' MHz,'/
     2         '    integration time per point = ',F8.1,' sec,',
     3         ' true b-factor = ',F4.2,':'/
     4         '    SQRT(sum of squares of theoretical noises',
     5         ' from each point) / N :',F8.3,' mJy'/
     6         '    1/SQRT(sum of inverse squares of theoretical',
     7         ' noises) :          ',F8.3,' mJy'//
     8         '(b) Derived from amplitude error estimates in input ',
     9         'file:'/
     9         '    SQRT(sum of amplitude errors squared) / N : ',
     A         '                    ',F8.3,' mJy'/
     B         '    1/SQRT(sum of inverse squares of amplitude',
     C         ' errors) :            ',F8.3,' mJy'//
     D         'Total number of data points : ',I6)
  590 FORMAT(///1X,A16,' Hour Angle, Zenith Angle, System temperature'
     1    ,', and Antenna temperature'//20X,4(8X,A8,12X))
  600 FORMAT(7X,'U.T.',2X,'G.S.T.',4('     HA    ZA   Tsys   Tant ')/)
  610 FORMAT(1X,I3,'/',A8,F6.2,4(2X,F6.2,F6.1,F6.1,F8.3))
  640 FORMAT(A,T20,A)
C-----------------------------------------------------------------------
C Start.
C
      CALL SCLOCK
      CALL PUTOUT(PROGNM//' applies calibration corrections to '//
     1       'Merge-format visibility files')
      CALL PUTOUT('(Version '//VERSN//')')
      CALL PUTOUT(' ')
C
C Open scratch file.
C
      CALL SCRNAM(PROGNM, SCRDSN, L)
      OPEN (UNIT=PS, FILE=SCRDSN(:L), STATUS='NEW', 
     1      FORM='UNFORMATTED', IOSTAT=IER)
      IF (IER.NE.0) CALL ERROR('Cannot open scratch file '//SCRDSN(:L))
C
C Open input file.
C
  710 IER = GETIN(INDSN, 'Input data file name:  ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(INDAT, INDSN(1:L), 'MERGE', 'OLD', INDSN)
      IF (IER.NE.1) GOTO 710
C
C Open output file.
C
  720 IER = GETIN(OUTDSN, 'Output data file name: ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(OUT, OUTDSN(1:L), 'MERGE', 'NEW', OUTDSN)
      IF (IER.NE.1) GOTO 720
C
C Open parameter file.
C
  730 IER = GETIN(PARDSN, 'Parameter file name:   ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(INPARS, PARDSN(1:L), 'TEXT', 'OLD', PARDSN)
      IF (IER.NE.1) GOTO 730
      INMODE = 0
C
C Open listing file.
C
  740 IER = GETIN(PRDSN, 'Listing file name:     ', L)
      IF (IER.NE.1) STOP
      IER = VLBOPE(PR, PRDSN(1:L), 'TEXT', 'NEW', PRDSN)
      IF (IER.NE.1) GOTO 740
C
C Get plot file name.
C
      IER = GETIN(PLTDSN, 'Plot device/type (blank for no plot): ', L)
      IF (IER.NE.1) THEN
          PLOT = .FALSE.
      ELSE
          PLOT = PLTDSN .NE. ' '
      END IF
C
C Read miscellaneous parameters.
C
      BAD=.FALSE.
      MODE = INMODE
      CALL KEYIN(HPARS,HVALS,NHPARS,ENDMRK,MODE,INPARS,OUTC)
      EOF = MODE.EQ.1
      DO 104 I=1,10
          FLUXES(I) = HVALS(I)
          WRITE(FLXSRC(I),'(2A8)') HVALS(10+2*I-1),HVALS(10+2*I)
  104 CONTINUE
      UNBIAS = HVALS(41).GE.0D0
      LONG  = HVALS(42).GE.0D0
      BDEF   = HVALS(43)
      DEFVAL(7+MAXS) = BDEF
      DO 10 IB=1,MAXB
          BFAC(IB) = BDEF
   10 CONTINUE
C
C Make the new history records.
C
      CALL MKHIST(PROGNM, VERSN, ADDHIS(1))
      ADDHIS(2) = '- Input file: '//INDSN
      ADDHIS(3) = '- Output file: '//OUTDSN
      ADDHIS(4) = '- Parameter file: '//PARDSN
      WRITE (ADDHIS(5), '(''- '',9A8,A6)') (HVALS(I),I=31,40)
      NNEW = 5
      IF (ADDHIS(5).EQ.' ') NNEW = 4
      DO 105 I=1,NNEW
          CALL PUTOUT(ADDHIS(I)(1:LEN1(ADDHIS(I))))
  105 CONTINUE
      CALL PUTOUT(' ')
C
C----- Read the file header --------------------------------------------
C
      READ (INDAT,END=19,ERR=19) (REC1(J),J=1,20)
      IF (REC1(1).EQ.1) THEN
          NVER = 1
          NDWPB = 4
      ELSE IF (REC1(1).EQ.2) THEN
          NVER = 2
          NDWPB = 7
      ELSE
          GOTO 19
      END IF
      NHIST = REC1(2)
      NS    = REC1(4)
      NB    = REC1(5)
      NT    = REC1(6)
      IF (NS.LT.2) GOTO 19
      IF (NS.GT.MAXS) CALL ERROR('Too many stations.')
      IF (REC1(8).NE.0) CALL PUTOUT(
     :   '++WARNING++ Data have already been calibrated.')
      IF (NB.GT.0) READ (INDAT,END=19,ERR=19) (NBI(IB),IB=1,NB)
      IF (NT.GT.0) READ (INDAT,END=19,ERR=19) (NTI(IT),IT=1,NT)
      DO 7 IB=1,NB
          NB1(IB) = NBI(IB)/100
          NB2(IB) = NBI(IB) - 100*NB1(IB)
    7 CONTINUE
C
C Rewrite the first three header recorders, modified to reflect
C that the data have been calibrated.
C
      REC1(2) = REC1(2)+NNEW
      REC1(3) = REC1(3)+NNEW
      REC1(8) = 1
      WRITE (OUT) (REC1(J),J=1,20)
      IF (NB.GT.0) WRITE (OUT) (NBI(IB),IB=1,NB)    
      IF (NT.GT.0) WRITE (OUT) (NTI(IT),IT=1,NT)
C
C Copy the history records and add the new ones.
C
      DO 14 I=1,NHIST
          READ (INDAT,END=19,ERR=19) HISTRY
          WRITE (OUT) HISTRY
   14 CONTINUE
      DO 15 I=1,NNEW
          WRITE (OUT) ADDHIS(I)
   15 CONTINUE
C
C Copy the remaining header records, changing only FLUX.
C
      READ (INDAT,END=19,ERR=19) SOURCE,RA,DEC,RAP,DECP,FLUX
      DO 146 I=1,10
          IF (SOURCE.EQ.FLXSRC(I)) THEN
              IF (FLUXES(I).NE.0D0) FLUX = FLUXES(I)
          END IF
  146 CONTINUE
      WRITE (OUT) SOURCE,RA,DEC,RAP,DECP,FLUX
      DO 16 IS=1,NS
          IF (NVER.EQ.1) THEN
              READ (INDAT,END=19,ERR=19) STNAME(IS), STPARM(1),
     1            STPARM(2), STPARM(3)
              DO 148 I=4,9
                  STPARM(I) = 0D0
  148         CONTINUE
              STFLAG = 0
          ELSE
              READ (INDAT,END=19,ERR=19) STNAME(IS), STPARM,
     1            STFLAG
          END IF
          STX(IS) = STPARM(1)
          STY(IS) = STPARM(2)
          STZ(IS) = STPARM(3)
          WRITE (OUT) STNAME(IS), STPARM, STFLAG
   16 CONTINUE
      READ (INDAT,END=19,ERR=19) IY, IZ, GASTM, FREQ, TINTG,
     1            AVG, BW, (IDUM(I),I=1,8)
C     -- Get integration time and bandwidth from parameters
C            if not specified in header
      IF (BW.EQ.0D0)    BW = HVALS(44)
      IF (TINTG.EQ.0D0) TINTG = HVALS(45)
      WRITE (OUT) IY, IZ, GASTM, FREQ, TINTG, AVG, BW, (IDUM(I),I=1,8)
      GOTO 20
   19 CALL ERROR('Something wrong with input MERGE file')
   20 CONTINUE
C
C-----------------------------------------------------------------------
C
      WRITE (PR,390) VERSN,
     1               INDSN(1:LEN1(INDSN)),OUTDSN(1:LEN1(OUTDSN))
      CALL KEYOUT(HPARS, HVALS, NHPARS, PR)
      WRITE (TEXT,'(''Processing source '',A,'', '',F8.3,''Jy'')')
     1       SOURCE, FLUX
      CALL PUTOUT(TEXT(:LEN1(TEXT)))
      WRITE (PR,'(/A)') TEXT(:LEN1(TEXT))
      IF (FLUX.LE.0D0) THEN
          WRITE (PR,395)
          FLUX = 1D0
      END IF
      LENG = 4*NB + 2*NT
C
C               Put station-names into parameter list; must be
C               upper case to avoid KEYIN minmatch.
C
      DO 22 IS=1,NS
          SPARS(IS+NGRPS) = STNAME(IS)
          CALL UPCASE(SPARS(IS+5))
   22 CONTINUE
C
      BW = ABS(BW)
C
C               Station constants. BADSTN flag is set for stations
C               which have invalid coordinates.
C
      DO 30 IS=1,NS
          BADSTN(IS) = (STX(IS)*STY(IS)*STZ(IS)).EQ.0D0
          IF (BADSTN(IS)) GOTO 30
          GLAT(IS) = DATAN2(STZ(IS),
     1                DSQRT(STX(IS)*STX(IS)+STY(IS)*STY(IS)))
          GLON(IS) = DATAN2(STY(IS),STX(IS))
          CC(IS) = COS(GLAT(IS))*DCOS(DECP)
          SS(IS) = SIN(GLAT(IS))*DSIN(DECP)
   30 CONTINUE
      SRA = RAP
C
C Determine group-parameter match
C
C   Allow all antennas for each group
      DO 33 IS=1,NS
          DO 33 IGR=1,NGRPS
              ALOWD(IGR,IS+6) = .TRUE.
   33 CONTINUE
C
C   Allow BASELINE group parameters
      ALOWD(BASEGR,BASEGR) = .TRUE.
C     -- Bfac
      ALOWD(BASEGR,7+MAXS) = .TRUE.
C
C   Allow TSYS group parameters
      ALOWD(TSYSGR,TSYSGR) = .TRUE.
C     -- FT?
      ALOWD(TSYSGR,8+MAXS) = .TRUE.
C     -- TCAL
      ALOWD(TSYSGR,9+MAXS) = .TRUE.
C     -- DELTAT
      ALOWD(TSYSGR,32+MAXS) = .TRUE.
C     -- TABLE
      DO 34 I=1,10
          ALOWD(TSYSGR,I+21+MAXS) = .TRUE.
   34 CONTINUE
C
C   Allow TANT group parameters
      ALOWD(TANTGR,TANTGR) = .TRUE.
C     -- FT?
      ALOWD(TANTGR,8+MAXS) = .TRUE.
C     -- DELTAT
      ALOWD(TANTGR,32+MAXS) = .TRUE.
C     -- TABLE
      DO 35 I=1,10
          ALOWD(TANTGR,I+21+MAXS) = .TRUE.
   35 CONTINUE
C
C   Allow GAIN group parameters
      ALOWD(GAINGR,GAINGR) = .TRUE.
C     -- EQUAT
      ALOWD(GAINGR,10+MAXS) = .TRUE.
C     -- ALTAZ
      ALOWD(GAINGR,11+MAXS) = .TRUE.
C     -- GCNRAO
      ALOWD(GAINGR,21+MAXS) = .TRUE.
C     -- DPFU
      ALOWD(GAINGR,12+MAXS) = .TRUE.
C     -- POLY
      DO 36 I=1,7
          ALOWD(GAINGR,I+12+MAXS) = .TRUE.
   36 CONTINUE
C
C   Allow TSCURV group parameters
      ALOWD(TSCVGR,TSCVGR)  = .TRUE.
C     -- EQUAT
      ALOWD(TSCVGR,10+MAXS) = .TRUE.
C     -- ALTAZ
      ALOWD(TSCVGR,11+MAXS) = .TRUE.
C     -- TSZ
      ALOWD(TSCVGR,33+MAXS) = .TRUE.
C     -- TANTFT
      ALOWD(TSCVGR,40+MAXS) = .TRUE.
C     -- TSPOLY
      DO 37 I=1,6
          ALOWD(TSCVGR,I+33+MAXS) = .TRUE.
   37 CONTINUE
C
C   Allow GTABLE group parameters
      ALOWD(GTABGR,GTABGR)  = .TRUE.
C     -- FT?
      ALOWD(GTABGR,8+MAXS) = .TRUE.
C     -- EQUAT
      ALOWD(GTABGR,10+MAXS) = .TRUE.
C     -- ALTAZ
      ALOWD(GTABGR,11+MAXS) = .TRUE.
C     -- TABLE
      DO 38 I=1,10
          ALOWD(GTABGR,I+21+MAXS) = .TRUE.
   38 CONTINUE
C
C               Read a control data group
C
   40 IF (EOF) GOTO 210
      DO 41 I=1,NPARS
          VAL(I) = DEFVAL(I)
   41 CONTINUE
      MODE = INMODE
      CALL KEYIN(PARS,VAL,NPARS,ENDMRK,MODE,INPARS,OUTC)
      EOF = MODE.EQ.1
C
C               Which sort of group ? (I)
C
      IF (VAL(NPARS).NE.DEFVAL(NPARS)) THEN
          CALL PUTOUT('++WARNING++ next group ignored')
          GOTO 40
      END IF
      DO 60 I=1,NGRPS
          IF (VAL(I).NE.-1) GOTO  70
   60 CONTINUE
      I = NGRPS+1
C
C               Which station ? (IS)
C
   70 DO 80 IS=1,NS
          IF (VAL(IS+NGRPS).NE.-1) GOTO 90
   80 CONTINUE
      I = NGRPS+1
C
C             Are specified parameters consistent with 
C               the group specified?
C
   90 CONTINUE
      TRYAGN = .FALSE.
      IF (I.LE.NGRPS) THEN
          DO 95 IP=I+1,NPARS
              IF ((VAL(IP) .NE. DEFVAL(IP)) .AND. 
     1            .NOT.ALOWD(I,IP)) THEN
                  WRITE(TEXT,'(A,A,A,A,1X,A,A)') 
     1                '++WARNING++ invalid parameter:  ',
     2                PARS(IP), ' for ',STNAME(IS), PARS(I),' group'
                  CALL PUTOUT(TEXT(:LEN1(TEXT)))
                  TRYAGN = .TRUE.
              END IF
   95     CONTINUE
          IF (TRYAGN) THEN
              CALL PUTOUT('Valid parameters are: ')
              DO 96 IP=I+1,NPARS
                  IF (ALOWD(I,IP) .AND. (PARS(IP).NE.PARS(IP-1))) THEN
                      WRITE(OUTC,'(5X,A)') PARS(IP)
                  END IF
   96         CONTINUE
              WRITE(OUTC,'(/)')
              GO TO 40
          END IF
      END IF
C
C               Process group information
C
      GOTO (120,150,170,190,205,208,100),I
C
C ---------- Default group: we get here if either no group keyword (BASELINE,
C            TSYS, TANT, GAIN, TSCURV, GTABLE) was specified or no station
C            name was specified.
C
  100 IF (EOF) GOTO 210
      CALL PUTOUT('++WARNING++ group or station keyword missing')
      GOTO 40
C
C ---------- Baseline group: we have found keyword BASELINE
C            and one station name; we need another. Baseline
C            parameters are: B (only).
C
  120 DO 130 JS=IS+1,NS
          IF (VAL(JS+NGRPS).NE.-1) GOTO 140
  130 CONTINUE
      CALL PUTOUT('++WARNING++ second station name missing '//
     1            'in BASELINE group')
      GOTO 40
  140 IB = LOOKUP(IS*100+JS,NBI,NB)
      IF (IB.EQ.0) IB = LOOKUP(JS*100+IS,NBI,NB)
      IF (IB.EQ.0) THEN
          CALL PUTOUT('++WARNING++ invalid baseline in BASELINE group')
          GOTO 40
      ELSE
          BFAC(IB) = VAL(7+MAXS)
      END IF
      CALL PUTOUT('Accepted BASELINE '//STNAME(IS)//' '//STNAME(JS))
      GOTO  40
C
C ---------- TSYS group: we have found keyword TSYS and one
C            station name. A TSYS table follows.  TSYS
C            parameters are: FT?, TCAL, DELTAT, TABLE.
C
  150 FAC  = VAL(8+MAXS)
      TCAL = VAL(9+MAXS)
      DELTAT = VAL(32+MAXS)
      DB   = FAC.LT.0.0 .OR. TCAL.NE.0D0
      FAC  = ABS(FAC)
      MODE = INMODE
      NTBL = MAXTBL
      WRITE (TABDSN, '(10A8)') (VAL(K),K=22+MAXS,31+MAXS)
      IF (TABDSN.EQ.' ') THEN
          CALL FFREAD(TABLE,NTBL,MODE,INPARS,OUTC)
          EOF  = MODE.EQ.1
      ELSE
          IER = VLBOPE(INTAB, TABDSN, 'TEXT', 'OLD', TABDSN)
          IF (IER.NE.1) CALL ERROR('in group TSYS '//STNAME(IS))
          CALL PUTOUT('Reading TSYS table '//TABDSN(1:LEN1(TABDSN)))
          CALL FFREAD(TABLE,NTBL,MODE,INTAB,OUTC)
          CLOSE (UNIT=INTAB)
      END IF
      IF (NTBL.LT.3)
     1    CALL ERROR('<3 entries in TSYS table for '//STNAME(IS))
      IF (NTBL.EQ.MAXTBL) 
     1    CALL ERROR('Too many entries in TSYS table for '//STNAME(IS))
      NTBL = NTBL/3
      L = 1
      DO 160 K=1,NTBL
          TSYST(K,IS) = ((TABLE(L)-1)*1440  + TABLE(L+1))*3600 - 
     1                  DELTAT*60
          Q = TABLE(L+2)
          IF (DB) Q = 1.0/(EXP(DBC*Q)-1.0)
          TSYSV(K,IS) = Q*FAC
          IF (TCAL.NE.0D0) TSYSV(K,IS) = TSYSV(K,IS)*TCAL
          L = L+3
  160 CONTINUE
      NTSYS(IS) = NTBL
      TSTYPE(IS) = 1
      WRITE (TEXT,'(''Accepted TSYS '',A,I5,'' entries'')') 
     :   STNAME(IS), NTSYS(IS)
      CALL PUTOUT(TEXT)
      GOTO 40
C
C ---------- TANT group: we have found keyword TANT and one
C            station name. A TANT table follows.  TANT
C            parameters are: FT?, DELTAT, TABLE.
C
  170 FAC  = VAL(8+MAXS)
      DB   = FAC.LT.0.0
      FAC  = ABS(FAC)
      DELTAT = VAL(32+MAXS)
      MODE = INMODE
      NTBL = MAXTBL
      WRITE (TABDSN, '(10A8)') (VAL(K),K=22+MAXS,31+MAXS)
      IF (TABDSN.EQ.' ') THEN
          CALL FFREAD(TABLE,NTBL,MODE,INPARS,OUTC)
          EOF  = MODE.EQ.1
      ELSE
          IER = VLBOPE(INTAB, TABDSN, 'TEXT', 'OLD', TABDSN)
          IF (IER.NE.1) CALL ERROR('in group TANT '//STNAME(IS))
          CALL PUTOUT('Reading TANT table '//TABDSN(1:LEN1(TABDSN)))
          CALL FFREAD(TABLE,NTBL,MODE,INTAB,OUTC)
          CLOSE (UNIT=INTAB)
      END IF
      IF (FLUX.EQ.0D0) THEN
          CALL PUTOUT(
     1      '++WARNING++ FLUX not specified: TANT tables ignored')
          GOTO 40
      END IF
      IF (NTBL.LT.3)
     1    CALL ERROR('<3 entries in TANT table for '//STNAME(IS))
      IF (NTBL.EQ.MAXTBL) 
     1    CALL ERROR('Too many entries in TANT table for '//STNAME(IS))
      NTBL = NTBL/3
      L = 1
      DO 180 K=1,NTBL
          TANTT(K,IS) = ((TABLE(L)-1)*1440  + TABLE(L+1))*3600 -
     1                  DELTAT*60
          Q = TABLE(L+2)
          IF (DB) Q = 1.0/(EXP(DBC*Q)-1.0)
          TANTV(K,IS) = Q*FAC
          L=L+3
  180 CONTINUE
      NTANT(IS) = NTBL
      ITYPE(IS) = 1
      WRITE (TEXT,'(''Accepted TANT '',A,I5,'' entries'')') 
     :   STNAME(IS), NTANT(IS)
      CALL PUTOUT(TEXT)
      GOTO 40
C
C ---------- GAIN group: we have found keyword GAIN and one
C            station name. GAIN parameters are: either EQUAT or
C            ALTAZ or GCNRAO, DPFU, and POLY.
C
  190 EQUAT = VAL(10+MAXS).NE.-1
      ALTAZ = VAL(11+MAXS).NE.-1
      GCNRAO = VAL(21+MAXS).NE.-1
C
      IF (EQUAT.AND.ALTAZ) CALL ERROR('Strange telescope mounting!.')
      IF (.NOT.(ALTAZ.OR.EQUAT.OR.GCNRAO)) CALL 
     1    ERROR('Specify ALTAZ or EQUAT or GCNRAO.')
C
      DPFU(IS) = VAL(12+MAXS)
      DO 200 K=1,7
          GC(K,IS) = VAL(12+MAXS+K)
  200 CONTINUE
      IF (ALTAZ) ITYPE(IS) = 2
      IF (EQUAT) ITYPE(IS) = 3
      IF (GCNRAO) ITYPE(IS) = 4
      CALL PUTOUT('Accepted GAIN '//STNAME(IS))
      GOTO 40
C
C ---------- TSCURV group: we have found keyword TSCURV and one
C            station name. TSCURV parameters are: either EQUAT or
C            ALTAZ, TSZ, TANTFT (flux-density fudge factor) and TSPOLY.
C
  205 TANTFT(IS) = VAL(40+MAXS)
      EQUAT = VAL(10+MAXS).NE.-1
      ALTAZ = VAL(11+MAXS).NE.-1
      IF (EQUAT.AND.ALTAZ) CALL ERROR('Strange telescope mounting!.')
      IF (.NOT.(ALTAZ.OR.EQUAT)) CALL 
     1    ERROR('Specify ALTAZ or EQUAT.')
C
      TSZ(IS) = VAL(33+MAXS)
      DO 207 K=1,6
          TC(K,IS) = VAL(33+MAXS+K)
  207 CONTINUE
      IF (ALTAZ) TSTYPE(IS) = 2
      IF (EQUAT) TSTYPE(IS) = 3
      CALL PUTOUT('Accepted TSCURV '//STNAME(IS))
      GOTO 40
C
C ---------- GTABLE group: we have found keyword GTABLE and one
C            station name. GTABLE parameters are: FT, EQUAT, ALTAZ.
C
  208 FAC  = VAL(8+MAXS)
      EQUAT = VAL(10+MAXS).NE.-1
      ALTAZ = VAL(11+MAXS).NE.-1
      IF (EQUAT.AND.ALTAZ) CALL ERROR('Strange telescope mounting!.')
      IF (.NOT.(ALTAZ.OR.EQUAT)) CALL 
     1    ERROR('Specify ALTAZ or EQUAT.')
      MODE = INMODE
      NTBL = MAXTBL
      WRITE (TABDSN, '(10A8)') (VAL(K),K=22+MAXS,31+MAXS)
      IF (TABDSN.EQ.' ') THEN
          CALL FFREAD(TABLE,NTBL,MODE,INPARS,OUTC)
          EOF  = MODE.EQ.1
      ELSE
          IER = VLBOPE(INTAB, TABDSN, 'TEXT', 'OLD', TABDSN)
          IF (IER.NE.1) CALL ERROR('in group GTABLE '//STNAME(IS))
          CALL PUTOUT('Reading GTABLE table '//TABDSN(1:LEN1(TABDSN)))
          CALL FFREAD(TABLE,NTBL,MODE,INTAB,OUTC)
          CLOSE (UNIT=INTAB)
      END IF
      IF (NTBL.LT.2)
     1    CALL ERROR('<2 entries in GTABLE table for '//STNAME(IS))
      IF (NTBL.EQ.MAXTBL) 
     1    CALL ERROR('Too many entries in GTABLE for '//STNAME(IS))
      NTBL = NTBL/2
      L = 1
      DO 209 K=1,NTBL
          GTABLZ(K,IS) = TABLE(L)
          GTABLV(K,IS) = TABLE(L+1)*FAC
          L=L+2
  209 CONTINUE
      NGTABL(IS) = NTBL
      IF (ALTAZ) ITYPE(IS) = 5
      IF (EQUAT) ITYPE(IS) = 6
      CALL PUTOUT('Accepted GTABLE '//STNAME(IS))
      GOTO 40
C
C ------------- (end case)
C
C               Summarize and check control data
C
  210 IF (UNBIAS) WRITE (PR,421) 'Unbiassing correction included'
      IF (.NOT.UNBIAS) WRITE (PR,421) 'Unbiassing correction OMITTED'
C
C               (1) Station coordinates.
C
      WRITE (PR,430)
      DO 220 IS=1,NS
          IF (BADSTN(IS)) THEN
              WRITE (PR,441) STNAME(IS)
          ELSE
              WRITE (PR,440) STNAME(IS), GLAT(IS)/RPDEG, GLON(IS)/RPDEG
          END IF
  220 CONTINUE
C
C               (2) TSYS data.
C
      WRITE (PR,470)
      NREC = 0
      DO 260 IS=1,NS
          IF (TSTYPE(IS).NE.1) GOTO 260
          NTBL = NTSYS(IS)
          IF (NTBL.LE.0 .AND. (.NOT.BADSTN(IS))) THEN
              WRITE (OUTC,480) 'TSYS',STNAME(IS)
              WRITE (PR,480) 'TSYS',STNAME(IS)
              BAD = .TRUE.
          ELSE
              LAST=0
              ORDER=.FALSE.
              DO 250 K=1,NTBL
                  IF (TSYST(K,IS).LT.LAST) THEN
                      ID = TSYST(K-1,IS)/5184000 + 1
                      T = TIMEC(MOD(TSYST(K-1,IS)/60,86400))
                      WRITE (PR,503) K-1, ID, T, TSYSV(K-1,IS)
                      ID = TSYST(K,IS)/5184000 + 1
                      T = TIMEC(MOD(TSYST(K,IS)/60,86400))
                      WRITE (PR,503) K,   ID, T, TSYSV(K,IS)
                      ORDER=.TRUE.
                  END IF
                  LAST=TSYST(K,IS)
  250         CONTINUE
              IF (ORDER) THEN
                  WRITE (OUTC,501) STNAME(IS)//' (TSYS)'
                  WRITE (PR,501) STNAME(IS)//' (TSYS)'
              END IF
              BAD = BAD.OR.ORDER
          END IF
  260 CONTINUE
      IF (.NOT.BAD .AND. LONG) THEN
          NPASS = NS/6
          IF (NS.GT.NPASS*6) NPASS=NPASS+1
          DO 265 IPASS=1,NPASS
              IS1 = (IPASS-1)*6 +1
              IS2 = MIN0(IS1+5,NS)
              NREC = NTSYS(IS1)
              DO 262 IS=IS1+1,IS2
                  NREC = MAX(NREC,NTSYS(IS))
  262         CONTINUE
              WRITE (PR,'(/1X,6(7X,A8,7X))') (STNAME(IS),IS=IS1,IS2)
              WRITE (PR,'(/1X,6(7X,A15))') ('U.T.      T (K)',
     1               IS=IS1,IS2)
              DO 264 IREC=1,NREC
                  DO 263 IS=IS1,IS2
                      FIELD(IS) = ' '
                      IF (IREC.LE.NTSYS(IS)) THEN
                          ID = TSYST(IREC,IS)/5184000 + 1
                          T = TIMEC(MOD(TSYST(IREC,IS)/60,86400))
                          IF (TSTYPE(IS).EQ.1) THEN
                              WRITE (FIELD(IS),'(I6,1X,A8,F7.1)') 
     1                            ID,T,TSYSV(IREC,IS)
                          ELSE
                              WRITE (FIELD(IS),'(I6,1X,A8,A7)') 
     1                            ID,T,'  ***  '
                          END IF
                      END IF
  263             CONTINUE
                  WRITE (PR,'(1X,6A22)') (FIELD(IS),IS=IS1,IS2)
  264         CONTINUE
  265     CONTINUE
      END IF
C
C               (3) TANT data
      WRITE (PR,510)
      DO 290 IS=1,NS
      IF (ITYPE(IS).GT.1) GOTO 290
      NTBL=NTANT(IS)
      IF (NTBL.LE.0 .AND. (.NOT.BADSTN(IS))) THEN
          WRITE (OUTC,480) 'TANT',STNAME(IS)
          WRITE (PR,480) 'TANT',STNAME(IS)
          BAD = .TRUE.
      ELSE
          WRITE (PR,490) STNAME(IS)
          LAST=0
      ORDER=.FALSE.
      DO 280 K=1,NTBL
          IF (TANTT(K,IS).LT.LAST) ORDER=.TRUE.
          LAST=TANTT(K,IS)
          ID=TANTT(K,IS)/5184000 + 1
          T = TIMEC(MOD(TANTT(K,IS)/60,86400))
  280       WRITE (PR,502) ID,T,TANTV(K,IS)
      IF (ORDER) THEN
          WRITE (OUTC,501) STNAME(IS)//' (TANT)'
          WRITE (PR,501) STNAME(IS)//' (TANT)'
      END IF
      BAD=BAD.OR.ORDER
      END IF
  290 CONTINUE
C
C               (3.5) GTABLE data
      DO 296 IS=1,NS
        IF (ITYPE(IS).NE.5 .AND. ITYPE(IS).NE.6) GOTO 296
        NTBL=NGTABL(IS)
        IF (NTBL.LE.0 .AND. (.NOT.BADSTN(IS))) THEN
            WRITE (OUTC,480) 'GTABLE',STNAME(IS)
            WRITE (PR,480) 'GTABLE',STNAME(IS)
            BAD = .TRUE.
        ELSE
            WRITE (PR,490) STNAME(IS)
            RLAST=-1E10
            ORDER=.FALSE.
            DO 294 K=1,NTBL
                IF (GTABLZ(K,IS).LT.RLAST) ORDER=.TRUE.
                RLAST=GTABLZ(K,IS)
                WRITE (PR,*) GTABLZ(K,IS), GTABLV(K,IS)
  294       CONTINUE
            IF (ORDER) THEN
                WRITE (OUTC,501) STNAME(IS)//' (GTABLE)'
                WRITE (PR,501) STNAME(IS)//' (GTABLE)'
            END IF
            BAD=BAD.OR.ORDER
        END IF
  296 CONTINUE
C
C               (4) Gain curves.
C
      WRITE (PR,520)
      DO 300 IS=1,NS
          CALL CCOEFF(GC(1,IS), 7, NPOLY)
          IF (ITYPE(IS).EQ.1) WRITE (PR,531) STNAME(IS), '(TANT table)' 
          IF (ITYPE(IS).EQ.2) WRITE (PR,530) STNAME(IS), '(ALTAZ)', 
     1       DPFU(IS), (GC(K,IS),K=1,NPOLY)
          IF (ITYPE(IS).EQ.3) WRITE (PR,530) STNAME(IS), '(EQUAT)', 
     1       DPFU(IS), (GC(K,IS),K=1,NPOLY)
          IF (ITYPE(IS).EQ.4) WRITE (PR,530) STNAME(IS), '(GCNRAO)', 
     1       DPFU(IS), (GC(K,IS),K=1,NPOLY)
          IF (ITYPE(IS).EQ.5) 
     1        WRITE (PR,531) STNAME(IS), '(ALTAZ gtable)'
          IF (ITYPE(IS).EQ.6) 
     1        WRITE (PR,531) STNAME(IS), '(EQUAT gtable)'
  300 CONTINUE
      WRITE (PR,521)
C
C               (5) Tsys curves.
C
      WRITE (PR,560)
      DO 301 IS=1,NS
          CALL CCOEFF(TC(1,IS), 6, NPOLY)
          IF (TSTYPE(IS).EQ.1) 
     1        WRITE (PR,531) STNAME(IS), '(TSYS table)' 
          IF (TSTYPE(IS).EQ.2) WRITE (PR,530) STNAME(IS), '(ALTAZ)',
     1       TSZ(IS), (TC(K,IS),K=1,NPOLY)
          IF (TSTYPE(IS).EQ.3) WRITE (PR,530) STNAME(IS), '(EQUAT)',
     1       TSZ(IS), (TC(K,IS),K=1,NPOLY)
          IF (TSTYPE(IS).EQ.2 .OR. TSTYPE(IS).EQ.3)
     1        WRITE(PR,542) TANTFT(IS)
  301 CONTINUE
      WRITE (PR,521)
C
      IF (BAD) CALL ERROR('VLBCAL abandoned because of errors noted.')
C
C Process data records
C
      CALL PUTOUT(' ')
      CALL PUTOUT('Applying calibration')
      DO 302 IB=1,NB
          SCTOT(IB) = 0.0
          SCMAX(IB) = 0.0
          NSC(IB) = 0
  302 CONTINUE
      NREC = 0
      NNOIS = 0
      MPN1 = 0.0
      MPN2 = 0.0
      DN1 = 0.0
      DN2 = 0.0
      UTMIN = 366*86400*60
      UTMAX = 0
C
C               (start loop)
C
  310 READ (INDAT,END=370) UT,(REC(J),J=1,LENG)
      IF (UT.LT.0) GOTO 370
      UTMIN = MIN(UTMIN,UT)
      UTMAX = MAX(UTMAX,UT)
      IDAY = UT/5184000 + 1
      T = TIMEC(MOD(UT/60,86400))
      GST = (UT-IZ)*SIDR + GASTM
C
C               Find system and antenna temperatures for each station
C
      DO 350 IS=1,NS
          USED(IS) = .FALSE.
          IF (BADSTN(IS)) GOTO 349
C         -- ZA and HA of antenna
          HAM = AMOD(GST-GLON(IS)-SRA,TWOPI)
          IF (HAM.GT. PI) HAM = HAM-TWOPI
          IF (HAM.LE.-PI) HAM = HAM+TWOPI
          ZA(IS) = ACOS(SS(IS)+CC(IS)*COS(HAM))/RPDEG
          HA(IS) = HAM/RPHR
C         -- Antenna temperature
          IGO = ITYPE(IS)
          GOTO (320,322,325,327,328,329),IGO
          CALL ERROR('Internal bug error in CAL')
C
C         -- Case (1): TANT table.
  320     IF (NTSYS(IS).LT.1) CALL ERROR('TANT table: '//STNAME(IS))
          TANT(IS) = RTERP(TANTT(1,IS),TANTV(1,IS),NTANT(IS),UT)
          GOTO 330
C
C         -- Case (2): ALTAZ gain curve.
  322     TANT(IS) = DPFU(IS)*FLUX*POLY(ZA(IS),GC(1,IS),7)
          GOTO 330
C
C         -- Case (3): EQUAT gain curve.
  325     TANT(IS) = DPFU(IS)*FLUX*POLY(HA(IS),GC(1,IS),7)
          GOTO 330
C
C         -- Case (4): GCNRAO gain curve.
  327     TANT(IS) = DPFU(IS)*FLUX*(
     1       GC(1,IS) +
     2       GC(2,IS)*DCOS(PIBY2-DECP) + 
     3       GC(3,IS)*DSIN(PIBY2-DECP)*COS(HAM) +
     4       GC(4,IS)*DSIN(PIBY2-DECP)*SIN(HAM) +
     5       GC(5,IS)*(1.5*DCOS(PIBY2-DECP)*DCOS(PIBY2-DECP)-0.5) +
     6       GC(6,IS)*3.0*DSIN(PIBY2-DECP)*DCOS(PIBY2-DECP)*COS(HAM) +
     7       GC(7,IS)*3.0*DSIN(PIBY2-DECP)*DCOS(PIBY2-DECP)*SIN(HAM) )
          GOTO 330
C
C         -- Case (5): GTABLE table (gain versus ZA).
  328     TANT(IS) = FLUX*STERP(GTABLZ(1,IS),GTABLV(1,IS),NGTABL(IS),
     :                          ZA(IS))
          GOTO 330
C
C         -- Case (6): GTABLE table (gain versus HA).
  329     TANT(IS) = FLUX*STERP(GTABLZ(1,IS),GTABLV(1,IS),NGTABL(IS),
     :                          HA(IS))
          GOTO 330
C
C         -- (end case for TANT)
C
C
  330     CONTINUE
          IGO = TSTYPE(IS)
          GOTO (340,342,345),IGO
C
C         -- Case (1): TSYS table.
  340     IF (NTSYS(IS).LT.1) CALL ERROR('TSYS table: '//STNAME(IS))
          TSYS(IS) = RTERP(TSYST(1,IS),TSYSV(1,IS),NTSYS(IS),UT)
          GOTO 350
C
C         -- Case (2): ALTAZ Tsys curve.
  342     TSYS(IS) = TANT(IS)*TANTFT(IS) + 
     1               TSZ(IS)*POLY(ZA(IS),TC(1,IS),6)
          GOTO 350
C
C         -- Case (3): EQUAT Tsys curve.
  345     TSYS(IS) = TANT(IS)*TANTFT(IS) + 
     1               TSZ(IS)*POLY(HA(IS),TC(1,IS),6)
          GOTO 350
C               (end case for TSYS)
C
C
C         -- invalid station?
  349     ZA(IS) = 0.0
          HA(IS) = 0.0
          TSYS(IS) = 0.0
          TANT(IS) = 0.0
  350 CONTINUE
C
C               Corrections for each baseline
C
      DO 360 IB=1,NB
          AMP    = REC(4*IB-3)
          AMPERR = ABS(REC(4*IB-2))
          FLAG   = REC(4*IB-2).LE.0.0
C
C               Correction for noise bias; flag data too weak to 
C               correct.
C
          IF (UNBIAS) THEN
              IF (AMP.LE.AMPERR) THEN
                  AMPC = 0.0
                  FLAG = .TRUE.
              ELSE
                  AMPC = SQRT(AMP*AMP - AMPERR*AMPERR)
              END IF
          END IF
C
C               Flag data with negative TANT.
C
          IS1 = NB1(IB)
          IS2 = NB2(IB)
          IF (AMPERR.GT.0.0) THEN
            USED(IS1) = .TRUE.
            USED(IS2) = .TRUE.
            IF (TANT(IS1).LE.0.0) THEN
              WRITE (OUTC,550) STNAME(IS1),T,ZA(IS1),HA(IS1)
              WRITE (PR,550) STNAME(IS1),T,ZA(IS1),HA(IS1)
              FLAG = .TRUE.
            END IF
            IF (TANT(IS2).LE.0.0) THEN
              WRITE (OUTC,550) STNAME(IS2),T,ZA(IS2),HA(IS2)
              WRITE (PR,550) STNAME(IS2),T,ZA(IS2),HA(IS2)
              FLAG = .TRUE.
            END IF
          END IF
C
          IF (TANT(IS1).LE.0.0 .OR. TANT(IS2).LE.0.0 .OR.
     :        TSYS(IS1).LE.0.0 .OR. TSYS(IS2).LE.0.0) THEN
              TFAC = 0.0
          ELSE
              TFAC = SQRT(TSYS(IS1)*TSYS(IS2)/TANT(IS1)/TANT(IS2))
          END IF
          SC    = AMP*(BFAC(IB)*TFAC*FLUX)
          SCERR = AMPERR*(BFAC(IB)*TFAC*FLUX)
          IF (FLAG) THEN
              SCERR = -1.0*SCERR
          ELSE
              SCMAX(IB) = MAX(SC,SCMAX(IB))
              SCTOT(IB) = SCTOT(IB) + SC
              NSC(IB) = NSC(IB) + 1
              NOISQ = (BTHEOR*TFAC*FLUX)**2/(2.0*BW*TINTG)
              MPN1 = MPN1 + NOISQ
              MPN2 = MPN2 + 1.0/NOISQ
              DN1  = DN1 + SCERR**2
              DN2  = DN2 + 1.0/(SCERR**2)
              NNOIS = NNOIS + 1
          END IF
          REC(4*IB-3) = SC
          REC(4*IB-2) = SCERR
  360 CONTINUE
C
C               Save results in scratch file
C
      GST=AMOD(GST/RPHR,24.0)
      DO 365 IS=1,NS
          IF (.NOT.USED(IS)) THEN
              TSYS(IS) = 0.0
              TANT(IS) = 0.0
          END IF
  365 CONTINUE
      WRITE (PS) UT,GST, (HA(IS),ZA(IS),TSYS(IS),TANT(IS),IS=1,NS)
      NREC=NREC+1
C
C               Rewrite data record
C
      WRITE (OUT) UT,(REC(J),J=1,LENG)
      GOTO 310
C
C               (end loop)
C
  370 UT=-1
      WRITE (OUT) UT,(UT,J=1,LENG)
C
C            Print summary of maximum and average visibilities.
C
      WRITE (PR,581)
      DO 371 IB=1,NB
          IF (NSC(IB).GT.0) THEN
            WRITE (PR,582) STNAME(NB1(IB)),STNAME(NB2(IB)),
     1                     BFAC(IB),NSC(IB),SCTOT(IB)/NSC(IB),SCMAX(IB),
     2                     SCTOT(IB)/NSC(IB)/FLUX,SCMAX(IB)/FLUX
          ELSE
            WRITE (PR,582) STNAME(NB1(IB)),STNAME(NB2(IB)),
     1                     BFAC(IB),NSC(IB)
          END IF
  371 CONTINUE
C
C            Print noise summary.
C
      IF (MPN1.GT.0.0 .AND. MPN2.GT.0.0 .AND.
     1            DN1.GT.0.0 .AND. DN2.GT.0.0) THEN
          MPN1 = SQRT(MPN1)/NNOIS*1000.
          MPN2 = SQRT(1.0/MPN2)*1000.
          DN1  = SQRT(DN1)/NNOIS*1000.
          DN2  = SQRT(1.0/DN2)*1000.
          WRITE (PR,583)
          WRITE (PR,584) BW/1E6,TINTG,BTHEOR,MPN1,MPN2,DN1,DN2,NNOIS
      END IF
C
C Print tables of Tsys and Tant (if option LONG was specified). 
C The tables are divided into pages with 4 antennas per page.
C
      IF (LONG) THEN
          CALL PUTOUT('Printing tables of Tsys and Tant')
          NPASS = NS/4
          IF (NS.GT.NPASS*4) NPASS=NPASS+1
          DO 375 IPASS=1,NPASS
              REWIND PS
              IS1 = (IPASS-1)*4 +1
              IS2 = MIN0(IS1+3,NS)
              WRITE (PR,590) SOURCE,(STNAME(IS),IS=IS1,IS2)
              WRITE (PR,600)
              DO 374 IREC=1,NREC
                  READ (PS) UT,GST,(HA(IS),ZA(IS),TSYS(IS),TANT(IS),
     1                      IS=1,NS)
                  IDAY = UT/5184000 + 1
                  T = TIMEC(MOD(UT/60,86400))
                  WRITE (PR,610) IDAY,T,GST,(HA(IS),ZA(IS),TSYS(IS),
     1                      TANT(IS),IS=IS1,IS2)
  374         CONTINUE
  375     CONTINUE
      END IF
C
C Plot Tsys in Jy versus time (if option PLOT was specified). 
C
      IF (PLOT) THEN
          CALL PUTOUT('Plotting graphs')
          IER = PGBEG(0, PLTDSN, 1, 1)
          IF (IER.NE.1) PLOT = .FALSE.
          CALL PGQINF('DEV/TYPE', PLTDSN, LP)
          CALL PUTOUT(PLTDSN(:LP))
          CALL PGSCH(0.6)
      END IF
      IF (PLOT) THEN
          XPLOT1 = REAL(UTMIN)/5184000.0 + 1.0 - 1./50.
          XPLOT2 = REAL(UTMAX)/5184000.0 + 1.0 + 1./50.
          DO 940 JS=1,NS
              REWIND PS
              YPLOT2 = 0.0
              NPT = 0
              DO 930 IREC=1,NREC
                  READ (PS) UT,GST,(HA(IS),ZA(IS),TSYS(IS),TANT(IS),
     1                      IS=1,NS)
                  IF (TANT(JS).GT.0.0) THEN
                      NPT = NPT+1
                      XPLOT(NPT) = REAL(UT)/5184000.0 + 1.0
                      YPLOT(NPT) = TSYS(JS)*FLUX/TANT(JS)
                      YPLOT2 = MAX(YPLOT2, YPLOT(NPT))
                      EPLOT(NPT) = 90.0 - ZA(JS)
                  END IF
  930         CONTINUE
              IF (YPLOT2.EQ.0.0) YPLOT2 = 100.0
              NVP = MOD(JS,5)
              IF  (NVP.EQ.0) NVP = 5
              IF (NVP.EQ.1) THEN
                  CALL PGPAGE
                  CALL PGVSTD
                  CALL PGQVP(0, XV1, XV2, YV1, YV2)
                  CALL PGSCI(3)
                  CALL PGMTXT('T', 1.0, 0.0, 0.0, 'T\dsys\u (Jy)')
                  CALL PGSCI(2)
                  CALL PGMTXT('T', 1.0, 1.0, 1.0, 'Z.A.')
                  CALL PGSCI(1)
              END IF
              HTVP = (YV2-YV1)/5.0
              CALL PGSVP(XV1, XV2, YV2-NVP*HTVP, YV2-(NVP-1)*HTVP)
              CALL PGSWIN(XPLOT1, XPLOT2, 0.0, YPLOT2*1.2)
              CALL PGSCI(1)
              CALL PGBOX('BCST', 0.0, 0, 'BNSTV', 0.0, 0)
              CALL PGMTXT('B', -2.0, 0.05, 0.0, STNAME(JS))
              IF (NVP.EQ.5 .OR. JS.EQ.NS) THEN
                  CALL PGBOX('N', 0.0, 0, ' ', 0.0, 0)
                  CALL PGLAB('UT (day of year)', ' ', ' ')
              END IF
              CALL PGSCI(3)
              CALL PGPT(NPT, XPLOT, YPLOT, 2)
              CALL PGSWIN(XPLOT1, XPLOT2, 0.0, 100.0)
              CALL PGSCI(1)
              CALL PGBOX(' ', 0.0, 0, 'CMSTV', 30.0, 3)
              CALL PGSCI(2)
              CALL PGSLS(2)
              CALL PGLINE(NPT, XPLOT, EPLOT)
              CALL PGSLS(1)
  940     CONTINUE
          CALL PGEND
      END IF
C
C Close and delete scratch file.
C
      CLOSE (UNIT=PS, STATUS='DELETE')
C
C End of job: generate execution summary and stop.
C
      WRITE (PR,'(//A/)') PROGNM//' execution summary:'
      WRITE (PR,640) PROGNM//' version:', VERSN
      CALL REPORT(PR)
      L = MIN(110,LEN1(INDSN))
      WRITE (PR,640) 'Input:', INDSN(1:L)
      L = MIN(110,LEN1(OUTDSN))
      WRITE (PR,640) 'Output:', OUTDSN(1:L)
      L = MIN(110,LEN1(PARDSN))
      WRITE (PR,640) 'Parameters:', PARDSN(1:L)
      L = MIN(110,LEN1(PRDSN))
      WRITE (PR,640) 'Listfile:', PRDSN(1:L)
      CALL REPCPU(PR)
C-----------------------------------------------------------------------
      END
C*POLY   -- evaluate a polynomial
C+
      REAL FUNCTION POLY(X, COEFF, N)
      INTEGER N
      REAL X, COEFF(N)
C
C Evaluate a polynomial:
C     POLY = c1 + c2*X + c3*X*X + ... cn*X**(n-1)
C
C Arguments:
C  X       (input)  : the argument at which the polynomial is to be
C                     evaluated.
C  COEFF   (input)  : array of coefficients c1..cn.
C  N       (input)  : number of coefficients; dimension of COEFF.
C
C Subroutines required: none.
C
C History:
C  1990 Aug 30  - revised (TJP).
C-----------------------------------------------------------------------
      INTEGER I
      REAL ARG
C
      IF (N.LT.1) THEN
          POLY = 0.0
      ELSE
          ARG  = 1.0
          POLY = COEFF(1)
          DO 10 I=2,N
              ARG  = ARG*X
              POLY = POLY + COEFF(I)*ARG
   10     CONTINUE
      END IF
C
      END

C*RTERP  -- linear interpolation in a table of values
C+
      REAL FUNCTION RTERP(IX, Y, N, ISEEK)
      INTEGER ISEEK, N, IX(N)
      REAL    Y(N)
C
C TABLE INTERPOLATION: LOCATE ISEEK IN IX
C AND RETURN CORRESPONDING VALUE OF Y
C
C T.J.PEARSON   1978 DECEMBER 15
C Revised 1990 Aug 30.
C-----------------------------------------------------------------------
      INTEGER I, IX1, IX2
      REAL Y1, Y2
C
      IF (N.LT.1) CALL ERROR('incorrect call to RTERP.')
C
      IF (ISEEK.LE.IX(1)) THEN
            RTERP = Y(1)
      ELSE IF (ISEEK.GE.IX(N)) THEN
            RTERP = Y(N)
      ELSE
          DO 10 I=2,N
          IF (ISEEK.GT.IX(I)) GOTO 10
              Y1 = Y(I-1)
              Y2 = Y(I)
              IX1 = IX(I-1)
              IX2 = IX(I)
              RTERP = Y1 + REAL(ISEEK-IX1)*(Y2-Y1)/REAL(IX2-IX1)
              RETURN
   10     CONTINUE
      END IF
C
      END

C*STERP  -- linear interpolation in a table of values
C+
      REAL FUNCTION STERP(X, Y, N, SEEK)
      INTEGER N
      REAL SEEK, X(N), Y(N)
C
C TABLE INTERPOLATION: LOCATE SEEK IN X
C AND RETURN CORRESPONDING VALUE OF Y
C
C T.J.PEARSON   1978 DECEMBER 15
C Revised 1990 Aug 30.
C-----------------------------------------------------------------------
      INTEGER I
      REAL X1, X2
      REAL Y1, Y2
C
      IF (N.LT.1) CALL ERROR('incorrect call to STERP.')
C
      IF (SEEK.LE.X(1)) THEN
            STERP = Y(1)
      ELSE IF (SEEK.GE.X(N)) THEN
            STERP = Y(N)
      ELSE
          DO 10 I=2,N
          IF (SEEK.GT.X(I)) GOTO 10
              Y1 = Y(I-1)
              Y2 = Y(I)
              X1 = X(I-1)
              X2 = X(I)
              STERP = Y1 + (SEEK-X1)*(Y2-Y1)/(X2-X1)
              RETURN
   10     CONTINUE
      END IF
C
      END

C*CCOEFF -- count number of non-zero polynomial coefficients
C+
      SUBROUTINE CCOEFF(A, NMAX, N)
      INTEGER NMAX, N
      REAL A(NMAX)
C-----------------------------------------------------------------------
      INTEGER I
      N = NMAX
      DO 10 I=NMAX,1,-1
          IF (A(I).NE.0.0) GOTO 20
          N = N-1
   10 CONTINUE
   20 RETURN
      END
