      PROGRAM FITSMG
C-----------------------------------------------------------------------
C Convert an AIPS-compatible UVFITS file into the Caltech Merge format.
C
C Control parameters:
C   INPUT:    the name of the input disk file (a FITS file with standard
C             2880-byte records).
C   OUTPUT:   the name for the output merge-format file.
C   LISTFILE: name for listing file (optional; blank for no listing).
C   FORMAT:   type of merge file to produce (1 or 2).
C   TAI-UTC:  time correction (default correct, usually).
C   NOHISTORY: discard history records.
C   WTFACTOR: (default 1.0)
C   WTPOWER:  (default 0)
C             AIPS weight = WTFACTOR/(err**WTPOWER)
C             amp error = (WTFACTOR/weight)**(1/WTPOWER)
C
C Author:     T.J. Pearson (TJP)
C Language:   Fortran-77 (with some extensions)
C
C Modifications:
C   Version 1.0:  1989 Aug 31 - new program.
C   Version 1.01: 1989 Oct 26 - allow up to 20 axes.
C   Version 1.02: 1990 Feb  6 - use SLA routine for TAI-UTC.
C   Version 1.03: 1990 Feb  8 - recognize special "CIT" keywords.
C   Version 1.04: 1990 Feb 22 - allow user to override TAI-UTC.
C   Version 1.05: 1990 Feb 23 - add NOHISTORY.
C   Version 1.06: 1990 Mar 17 - accept AIPS 15OCT89 antenna tables.
C   Version 1.07: 1990 May 22 - use RDFITS/RDFITC routines.
C   Version 1.08: 1990 Jun 13 - avoid a divide-by-zero error.
C   Version 1.09: 1990 Nov 13 - remove unused variables.
C   Version 1.10: 1990 Nov 29 - standard Fortran.
C   Version 1.11: 1991 Apr 23 - workaround for Sun Fortran bug.
C   Version 1.12: 1991 Apr 25 - increase to 40 stations, multichannel.
C   Version 1.13: 1991 May 15 - correct error in uvw coordinates.
C   Version 1.14: 1991 Jun 21 - correct error in precession.
C   Version 1.15: 1991 Aug 11 - decode weights.
C   Version 1.16: 1991 Nov  9 - accept IEEE float.
C   Version 1.17: 1992 Mar 23 - accept IF axis.
C   Version 1.18: 1993 Mar  2 - check for zero weight.
C   Version 1.19: 1994 Jan 20 - sort data into time order if necessary.
C   Version 1.20: 1994 Jan 31 - correctly decode negative weights;
C                               recognize FITS BINTABLEs.
C   Version 1.21: 1994 Apr 11 - abort if antenna table is incorrect.
C   Version 1.22: 1994 Aug  5 - clean up bad source/antenna names.
C   Version 1.23: 1995 Jun 15 - handle J2000 coordinates.
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) PROGNM, VERSN
      INTEGER    INC,OUTC,INDAT,OUTDAT,PR,SCR,NPARS
      PARAMETER  (PROGNM='FITSMERGE')
      PARAMETER  (VERSN='1.23 - 1995 Jun 15')
C     -- I/O unit numbers
      PARAMETER  (INC=5,OUTC=6,INDAT=1,OUTDAT=2,PR=3,SCR=4)
C     -- number of KEYIN parameters
      PARAMETER  (NPARS=35)
C
C Variables.
C
      CHARACTER  FITFIL*128, MRGFIL*128, LISFIL*128, SCRFIL*128
      CHARACTER  TEXT*80, HISTRY*80
      INTEGER    I, IER, J, L, MODE, GCOUNT, PCOUNT, NPIX, BITPIX
      INTEGER    LREC, NBITS, NREC, NBYTE, MGFMT, NHIST, NCHAN
      INTEGER    NWORDS(10), LOC(9)
      LOGICAL    NOHIST
      REAL       WTFAC, WTPOW
      DOUBLE PRECISION  DT
      DOUBLE PRECISION  ENDMRK, PARS(NPARS), VALS(NPARS)
C
      INTEGER LUNIT
      COMMON /UNFIT1/ LUNIT
C Dummy array: dynamic allocation is based on this array
      INTEGER P(2)
      COMMON /STORE/ P
C
C External functions.
C
      INTEGER    LEN1, VLBOPE, VALLOC
C
C Initialize KEYIN parameters.
C
      DATA       ENDMRK/'/'/
      DATA       PARS/ 'Input', 9*' ', 
     1                 'Output', 9*' ',
     2                 'Listfile', 9*' ',
     3                 'Format',
     4                 'TAI-UTC',
     5                 'NOHistor',
     6                 'WTFactor',
     7                 'WTPower'/
      DATA       VALS/ '***UNSET', 9*' ',
     1                 '***UNSET', 9*' ',
     2                 10*' ',
     3                 1D0,
     4                 0D0,
     5                 -1D0,
     6                 1D0,
     7                 0D0/
C
C Formats.
C
  610 FORMAT (A,T20,A)
  630 FORMAT ('Reading ',I6,' visibility samples,',
     1                   I4,' FITS data records:')
  640 FORMAT ('  (this requires ',I12,' bytes of memory = ',
     1                   F6.1,' Mbytes)')
C-----------------------------------------------------------------------
C
C Introduction.
C
      CALL SCLOCK
      CALL PUTOUT(PROGNM//' converts files from UVFITS to MERGE format')
      CALL PUTOUT('(Version '//VERSN//')')
C
C Control parameters.
C
   10 MODE = 0
      CALL PUTOUT(' ')
      CALL PUTOUT('Control parameters (INPUT, OUTPUT, LISTFILE;'//
     1            ' end with /):')
      CALL KEYIN(PARS,VALS,NPARS,ENDMRK,MODE,INC,OUTC)
      IF (MODE.NE.0) GOTO 80
      IF (VALS(31).NE.1D0 .AND. VALS(31).NE.2D0) THEN
          CALL PUTOUT('Parameter FORMAT must have value 1 or 2.')
          GOTO 10
      ELSE
          MGFMT = VALS(31)
      END IF
      WRITE (FITFIL,'(10A8)') (VALS(J),J= 1,10)
      WRITE (MRGFIL,'(10A8)') (VALS(J),J=11,20)
      WRITE (LISFIL,'(10A8)') (VALS(J),J=21,30)
      DT = VALS(32)
      NOHIST = VALS(33).NE.-1D0
      WTFAC = VALS(34)
      WTPOW = VALS(35)
C
C Open input FITS file.
C
      IER = VLBOPE(INDAT, FITFIL, 'FITS', 'OLD', FITFIL)
      IF (IER.NE.1) GOTO 10
C
C Open listing file.
C
      IF (LISFIL.EQ.' ') THEN
          LUNIT = 0
      ELSE
          IER = VLBOPE(PR, LISFIL, 'TEXT', 'NEW', LISFIL)
          IF (IER.NE.1) THEN
              CLOSE (UNIT=INDAT)
              GOTO 10
          END IF
          LUNIT = PR
          WRITE (LUNIT, '(2A)') 'Program ',PROGNM
      END IF
C
C Open scratch file for history.
C
      CALL SCRNAM(PROGNM, SCRFIL, L)
      IER = VLBOPE(SCR, SCRFIL(:L), 'TEXT', 'NEW', SCRFIL)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          IF (LUNIT.NE.0) CLOSE (UNIT=LUNIT, STATUS='DELETE')
          GOTO 10
      END IF
      NHIST = 0
C
C Open output Merge file.
C
      IER = VLBOPE(OUTDAT, MRGFIL, 'MERGE', 'NEW', MRGFIL)
      IF (IER.NE.1) THEN
          CLOSE (UNIT=INDAT)
          IF (LUNIT.NE.0) CLOSE (UNIT=LUNIT, STATUS='DELETE')
          CLOSE (UNIT=SCR, STATUS='DELETE')
          GOTO 10
      END IF
C
C Initialize the Merge header with default parameters; the argument
C is the Merge format version number.
C
      CALL MGINIT(MGFMT)
C
C Read FITS header.
C
      CALL LIST(' ')
      CALL LIST('Input file: '//FITFIL)
      CALL LIST(' ')
      CALL LIST('Reading FITS header:')
      CALL UVFHED(INDAT, SCR, IER, GCOUNT, PCOUNT, NPIX, BITPIX, NHIST, 
     1            NCHAN,WTFAC,WTPOW)
      IF (IER.NE.0) GOTO 80
      IF (NOHIST) THEN
          REWIND (UNIT=SCR)
          NHIST = 0
      END IF
C
C Report size of file.
C
      CALL LIST(' ')
      LREC = PCOUNT+NPIX
      NBITS = ABS(BITPIX)*GCOUNT*LREC
      NREC = (NBITS+23039)/23040
      WRITE (TEXT,630) GCOUNT, NREC
      CALL LIST(TEXT)
      NBYTE = 4*10*GCOUNT
      WRITE (TEXT,640) NBYTE, NBYTE/1048576.0
      CALL LIST(TEXT)
C
C Allocate memory for arrays.
C
      NWORDS(1) = GCOUNT
      NWORDS(2) = GCOUNT
      NWORDS(3) = GCOUNT
      NWORDS(4) = GCOUNT
      NWORDS(5) = GCOUNT*2
      NWORDS(6) = GCOUNT
      NWORDS(7) = GCOUNT
      NWORDS(8) = GCOUNT
      NWORDS(9) = GCOUNT
      DO 30 I=1,9
          LOC(I) = VALLOC(NWORDS(I), P)
   30 CONTINUE
C
C Read data.
C
      IF (BITPIX.EQ.32 .OR. BITPIX.EQ.16 .OR. BITPIX.EQ.-32) THEN
          CALL READIT(INDAT,BITPIX,GCOUNT,PCOUNT,NPIX,
     1                P(LOC(1)), P(LOC(2)), P(LOC(3)),
     2                P(LOC(4)), P(LOC(5)), P(LOC(6)),
     3                P(LOC(7)), P(LOC(8)))
      ELSE
          CALL ERROR('Invalid BITPIX')
      END IF
C
C Read extensions.
C
      CALL LIST(' ')
      CALL LIST('Reading FITS extension:')
  110 CALL FITSEX(INDAT, IER)
      IF (IER.EQ.0) GOTO 110
C
C End of file.
C
      CLOSE (UNIT=INDAT)
C
C Check data are in time order; correct for TAI-UTC.
C
      CALL CHKTIM(GCOUNT, P(LOC(5)), P(LOC(9)), DT)
C
C Check baselines.
C
      CALL CHKBAS(GCOUNT, P(LOC(4)))
C
C Add new history records.
C
      CALL MKHIST(PROGNM, VERSN, HISTRY)
      WRITE (SCR, '(A)') HISTRY(:LEN1(HISTRY))
      HISTRY = '- Input = '//FITFIL
      WRITE (SCR, '(A)') HISTRY(:LEN1(HISTRY))
      NHIST = NHIST+2
C
C Write Merge file header.
C
      REWIND (UNIT=SCR)
      CALL MGWRIT(OUTDAT, SCR, NHIST)
C
C Write Merge data.
C
      CALL MGOUT(OUTDAT,GCOUNT, WTFAC, WTPOW,
     1                P(LOC(1)), P(LOC(2)), P(LOC(3)),
     2                P(LOC(4)), P(LOC(5)), P(LOC(6)),
     3                P(LOC(7)), P(LOC(8)), P(LOC(9)))
C
      CLOSE (UNIT=SCR, STATUS='DELETE')
      CLOSE (UNIT=OUTDAT)
C
C End of job: print Execution Summary.
C
      CALL LIST(' ')
      IF (LUNIT.NE.0) THEN
          WRITE (LUNIT,610) PROGNM//' version:', VERSN
          WRITE (LUNIT,610) 'Input:   ', FITFIL(1:LEN1(FITFIL))
          WRITE (LUNIT,610) 'Output:  ', MRGFIL(1:LEN1(MRGFIL))
          WRITE (LUNIT,610) 'Listfile:', LISFIL(1:LEN1(LISFIL))
          CALL REPORT(LUNIT)
          CALL REPCPU(LUNIT)
          CLOSE (UNIT=LUNIT)
      END IF
C
C Error exit.
C
   80 CONTINUE
C-----------------------------------------------------------------------
      END

      SUBROUTINE UVFHED (INUNIT, SCR, STATUS, GCOUNT, PCOUNT, NPIX, 
     1      BITPIX, NHIST, NCHAN, WTFAC, WTPOW)
      INTEGER INUNIT, SCR, STATUS
      INTEGER GCOUNT, PCOUNT, NPIX, BITPIX, NHIST, NCHAN
      REAL WTFAC, WTPOW
C-----------------------------------------------------------------------
C UVFHED: read header from UVFITS file.
C
C     Parameters -   (">" input, "<" output )
C
C     (>) INUNIT  (Integer) Fortran unit number for the input file.
C                 The file must have been opened using a Fortran
C                 open statement, and UVFHED assumes the file is
C                 positioned at the start of the header record(s).
C     (>) SCR     (Integer) Fortran unit number for the scratch file.
C     (<) STATUS  (Integer) A status code for the read.
C                 0 => OK, 1 => error
C     (!) NHIST   Number of history records.
C     (<) NCHAN   Number of channels.
C     (!) WTFAC
C     (!) WTPOW
C-----------------------------------------------------------------------
      INTEGER MAXAX
      PARAMETER (MAXAX=20)
      DOUBLE PRECISION PI, RPDEG
      PARAMETER (PI=3.14159265359D0)
      PARAMETER (RPDEG=PI/180D0)
      LOGICAL BITP, SIMPLE, GROUPS, EXTEND
      DOUBLE PRECISION RVAL
      INTEGER I, K, N, ISTAT, TYPE, LCARD
      CHARACTER HEADER*2880, KEYW*8, CVAL*72, CARD*80, TEXT*80
      INTEGER LEN1
C-----------------------------------------------------------------------
C MPNDM       number of dimensions (1..20) (NAXIS)
C NAXIS(MAXAX)number of pixels in each dim (NAXISn)
C MPNIT       number of CLEAN iterations (NITER)
C
C MPOBJ       source name (OBJECT)
C MPTEL       telescope name, eg 'VLA', (TELESCOP)
C MPINS       instrument name, eg 'VLBI' (INSTRUME)
C MPOBS       observer name (OBSERVER)
C MPDOB       date of observation (DATE-OBS)
C MPDMP       date of creation of map (DATE-MAP)
C MPBUN       map units, eg 'Jy/beam' (BUNIT)
C CTYPE(MAXAX)coordinate, eg 'RA', 'Dec' (CTYPEn)
C MPTYP       type of map (--)
C MPCRT       creating program (--)
C
C CRVAL()    ref point value (CRVALn)
C CRPIX()    ref point pixel locn (CRPIXn)
C CDELT()    coord value increment (CDELTn)
C CROTA()    rotation angle (CROTAn)
C MPEPO       equinox of coordinates (EPOCH or EQUINOX)
C MPFRQ       frequency, MHz (--)
C MPBWD       bandwidth, MHz (--)
C-----------------------------------------------------------------------
      INCLUDE 'mrghed.inc'
      INTEGER MPNDM, NAXIS(MAXAX), MPNIT
C
      CHARACTER*16 MPOBJ, MPTEL, MPINS, MPOBS, MPDOB, MPDMP, MPBUN
      CHARACTER*16 CTYPE(MAXAX), MPTYP, MPCRT, DATE
C
      DOUBLE PRECISION CRVAL(MAXAX), CRPIX(MAXAX), CDELT(MAXAX)
      DOUBLE PRECISION CROTA(MAXAX)
      DOUBLE PRECISION MPEPO, MPFRQ, MPBWD
C-----------------------------------------------------------------------
      INTEGER MAXP
      PARAMETER (MAXP=32)
      CHARACTER*16 PTYPE(MAXP)
      DOUBLE PRECISION STOKES, FREQ, RA, DEC
      DOUBLE PRECISION BSCALE, BZERO, WTSCAL
      DOUBLE PRECISION PSCAL(MAXP), PZERO(MAXP)
      INTEGER PTUU, PTVV, PTWW, PTBASE, PTDAT1, PTDAT2
      COMMON /FITS1/ BSCALE, BZERO, PSCAL, PZERO,
     1               PTUU, PTVV, PTWW, PTBASE, PTDAT1, PTDAT2, WTSCAL
C-----------------------------------------------------------------------
C
C            Initial values
C
      STATUS = 1
      MPNDM = 0
      MPNIT = 0
      MPOBJ = ' '
      MPTEL = ' '
      MPINS = ' '
      MPOBS = ' '
      MPDOB = ' '
      DATE  = ' '
      MPDMP = ' '
      MPBUN = ' '
      MPTYP = 'MAP'
      MPCRT = ' '
      MPEPO = 0D0
      MPFRQ = 0D0
      MPBWD = 0D0
      DO 5 I=1,MAXAX
          NAXIS(I) = 0
          CTYPE(I) = 'PIXEL'
          CRVAL(I) = 0D0
          CRPIX(I) = 0D0
          CDELT(I) = 1D0
          CROTA(I) = 0D0
    5 CONTINUE
      BSCALE = 1.
      BZERO = 0.
      WTSCAL = 1.0
      BITP = .FALSE.
      SIMPLE = .FALSE.
      GROUPS = .FALSE.
      EXTEND = .FALSE.
      MPNDM = -1
      PCOUNT = -1
      GCOUNT = -1
      DO 6 I=1,MAXP
          PTYPE(I) = ' '
          PZERO(I) = 0D0
          PSCAL(I) = 1D0
    6 CONTINUE
C
C Read in the next header record.
C
   10 CALL RDFITC(INUNIT, ISTAT, HEADER)
      IF (ISTAT.NE.0) THEN
          CALL LIST ('File has wrong format for a FITS file')
          STATUS = 1
          RETURN
      END IF
C
C Decode the header records.
C
      DO 20 I=0,35
          CARD = HEADER(80*I+1:80*I+80)
          LCARD = LEN1(CARD)
          CALL FITKEY(CARD, KEYW, TYPE, RVAL, CVAL)
          IF (TYPE.EQ.6) THEN
C             -- History card
              WRITE (SCR,'(A)') CARD(9:MAX(9,LCARD))
              NHIST = NHIST+1
C             -- Look for special AIPS WTSCAL
              IF (CARD(9:19).EQ.'AIPS WTSCAL') THEN
                  CALL LIST(CARD)
                  K = INDEX(CARD,'=')
                  READ (CARD(K+1:),*) WTSCAL
                  WRITE (TEXT,'(''WTSCAL = '',E20.10)') WTSCAL
                  CALL LIST(TEXT)
              END IF
          ELSE IF (KEYW.EQ.'END') THEN
              GOTO 320
          ELSE IF (KEYW.EQ.'SIMPLE') THEN
              IF (CVAL.NE.'T') GOTO 560
              SIMPLE = .TRUE.
          ELSE IF (KEYW.EQ.'EXTEND') THEN
              IF (CVAL.EQ.'T') THEN
                  EXTEND = .TRUE.
              ELSE IF (CVAL.EQ.'F') THEN
                  EXTEND = .FALSE.
              ELSE
                  GOTO 560
              END IF
          ELSE IF (KEYW.EQ.'BLOCKED') THEN
              IF (CVAL.NE.'T' .AND. CVAL.NE.'F') GOTO 560
          ELSE IF (KEYW.EQ.'BITPIX') THEN
              BITPIX = RVAL
              IF ((BITPIX.NE.8).AND.(BITPIX.NE.16).AND.(BITPIX.NE.32)
     1                .AND.(BITPIX.NE.-32)) GOTO 560
              BITP = .TRUE.
          ELSE IF (KEYW.EQ.'NAXIS') THEN
              MPNDM = RVAL
              IF (MPNDM.LT.0 .OR. MPNDM.GT.MAXAX) GOTO 560
          ELSE IF (KEYW(1:6).EQ.'NAXIS1') THEN
              IF (RVAL.NE.0.0) GOTO 560
              NAXIS(1) = 0
          ELSE IF (KEYW(1:5).EQ.'NAXIS') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
              NAXIS(N) = RVAL
          ELSE IF (KEYW.EQ.'GROUPS') THEN
              IF (CVAL.NE.'T') GOTO 560
              GROUPS = .TRUE.
          ELSE IF (KEYW.EQ.'GCOUNT') THEN
              GCOUNT = RVAL
              CALL LIST(CARD)
          ELSE IF (KEYW.EQ.'PCOUNT') THEN
              PCOUNT = RVAL
              IF (PCOUNT.GT.MAXP) GOTO 560
          ELSE IF (KEYW(1:5).EQ.'CRVAL') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
              CRVAL(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'CRPIX') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
              CRPIX(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'CDELT') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
              CDELT(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'CTYPE') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
              CTYPE(N) = CVAL
          ELSE IF (KEYW(1:5).EQ.'CROTA') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.MPNDM) GOTO 560
              CROTA(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'PTYPE') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.PCOUNT) GOTO 560
              PTYPE(N) = CVAL
          ELSE IF (KEYW(1:5).EQ.'PZERO') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.PCOUNT) GOTO 560
              PZERO(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'PSCAL') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.PCOUNT) GOTO 560
              PSCAL(N) = RVAL
          ELSE IF (KEYW.EQ.'BSCALE') THEN
            BSCALE = RVAL
          ELSE IF (KEYW.EQ.'BZERO') THEN
            BZERO = RVAL
          ELSE IF (KEYW.EQ.'OBJECT') THEN
              MGSRC = CVAL
              CALL ASCCNV(MGSRC)
              CALL LIST(CARD)
          ELSE IF (KEYW.EQ.'TELESCOP') THEN
              MPTEL = CVAL
          ELSE IF (KEYW.EQ.'INSTRUME') THEN
              MPINS = CVAL
          ELSE IF (KEYW.EQ.'OBSERVER') THEN
              MPOBS = CVAL
          ELSE IF (KEYW.EQ.'DATE') THEN
              DATE = CVAL
          ELSE IF (KEYW.EQ.'DATE-OBS') THEN
              MPDOB = CVAL
              CALL LIST(CARD)
          ELSE IF (KEYW.EQ.'DATE-MAP') THEN
              MPDMP = CVAL
          ELSE IF (KEYW.EQ.'BUNIT') THEN
              MPBUN = CVAL
              IF (MPBUN.EQ.'JY') THEN
                  MGTYPE = 1
              END IF
          ELSE IF (KEYW.EQ.'EPOCH') THEN
              MGEQUI = RVAL
          ELSE IF (KEYW.EQ.'EQUINOX') THEN
              MGEQUI= RVAL
          ELSE IF (KEYW.EQ.'ORIGIN') THEN
              MPCRT = CVAL
          ELSE IF (KEYW.EQ.'CITFLUX') THEN
              MGFLUX = RVAL
          ELSE IF (KEYW.EQ.'CITTINTG') THEN
              MGAVG1 = RVAL
          ELSE IF (KEYW.EQ.'CITAVG') THEN
              MGAVG2 = RVAL
          ELSE IF (KEYW.EQ.'CITWTPOW') THEN
              WTPOW = RVAL
          ELSE IF (KEYW.EQ.'CITWTFAC') THEN
              WTFAC = RVAL
          ELSE IF (KEYW.EQ.'WTSCAL') THEN
              WTSCAL = RVAL
          ELSE IF (KEYW.EQ.'VELREF' .OR.
     1             KEYW.EQ.'ALTRPIX' .OR.
     2             KEYW.EQ.'OBSRA' .OR.
     3             KEYW.EQ.'OBSDEC' .OR.
     4             KEYW.EQ.' ') THEN
              CONTINUE
          ELSE
              CALL LIST(CARD)
          END IF
   20 CONTINUE
      GOTO 10
C
C Check for required parameters.
C
  320 CONTINUE
      IF (.NOT.SIMPLE) THEN
          KEYW = 'SIMPLE'
          GOTO 570
      END IF
      IF (.NOT.GROUPS) THEN
          KEYW = 'GROUPS'
          GOTO 570
      END IF
      IF (.NOT.BITP) THEN
          KEYW = 'BITPIX'
          GOTO 570
      END IF
      IF (MPNDM.LT.0) THEN
          KEYW = 'NAXIS'
          GOTO 570
      END IF
      IF (PCOUNT.LT.0) THEN
          KEYW = 'PCOUNT'
          GOTO 570
      END IF
      IF (GCOUNT.LT.0) THEN
          KEYW = 'GCOUNT'
          GOTO 570
      END IF
C
C Check axes (axis 1 doesn't exist in GROUPS format). Axes used by AIPS
C are:
C       COMPLEX: real part, imaginary part, and (optional) weight
C       STOKES:  Stokes parameter
C       FREQ:    frequency in Hz
C       IF:
C       RA, DEC: celestial coordinates in degrees
C
      NCHAN = 1
      DO 400 I=2,MPNDM
          IF (CTYPE(I).EQ.'COMPLEX') THEN
              WRITE (TEXT,
     1               '(''Axis'',I2,'' = COMPLEX, '',I2,'' pixels'')')
     2               I, NAXIS(I)
              CALL LIST(TEXT)
              IF (CROTA(I).NE.0) GOTO 590
          ELSE IF (CTYPE(I).EQ.'STOKES') THEN
              STOKES = CRVAL(I) + CDELT(I)*(CRPIX(I)-1.0)
              MGSTOK = STOKES
              WRITE (TEXT, '(''Axis'',I2,'' = STOKES = '',F10.4)')
     1               I, STOKES
              CALL LIST(TEXT)
              IF (NAXIS(I).NE.1) THEN
                  WRITE (TEXT, '(10X,I3,'' pixels'')') NAXIS(I)
                  CALL LIST(TEXT)
              END IF
              NCHAN = NCHAN*NAXIS(I)
              IF (CROTA(I).NE.0) GOTO 590
          ELSE IF (CTYPE(I).EQ.'FREQ') THEN
              FREQ = CRVAL(I) + CDELT(I)*(CRPIX(I)-1.0)
              MGFREQ = FREQ
              MGBW   = CDELT(I)
              WRITE (TEXT,
     1               '(''Axis'',I2,'' = FREQ = '',-6PF10.4,'' MHz'')') 
     2               I, FREQ
              CALL LIST(TEXT)
              IF (NAXIS(I).NE.1) THEN
                  WRITE (TEXT, '(10X,I3,'' pixels'')') NAXIS(I)
                  CALL LIST(TEXT)
              END IF
              NCHAN = NCHAN*NAXIS(I)
              IF (CROTA(I).NE.0) GOTO 590
          ELSE IF (CTYPE(I).EQ.'IF') THEN
              WRITE (TEXT,
     1               '(''Axis'',I2,'' = IF'')') I
              CALL LIST(TEXT)
              IF (NAXIS(I).NE.1) THEN
                  WRITE (TEXT, '(10X,I3,'' pixels'')') NAXIS(I)
                  CALL LIST(TEXT)
              END IF
              NCHAN = NCHAN*NAXIS(I)
              IF (CROTA(I).NE.0) GOTO 590
          ELSE IF (CTYPE(I).EQ.'RA') THEN
              RA = CRVAL(I) + CDELT(I)*(CRPIX(I)-1.0)
              MGRA50 = RA*RPDEG
              WRITE (TEXT, '(''Axis'',I2,'' = RA = '')') I
              CALL PANGLE('H', MGRA50, TEXT(14:31), 4)
              CALL LIST(TEXT)
              IF (NAXIS(I).NE.1) GOTO 580
              IF (CROTA(I).NE.0) GOTO 590
          ELSE IF (CTYPE(I).EQ.'DEC') THEN
              DEC = CRVAL(I) + CDELT(I)*(CRPIX(I)-1.0)
              MGDC50 = DEC*RPDEG
              WRITE (TEXT, '(''Axis'',I2,'' = DEC = '')') I
              CALL PANGLE('D', MGDC50, TEXT(15:31), 3)
              CALL LIST(TEXT)
              IF (NAXIS(I).NE.1) GOTO 580
              IF (CROTA(I).NE.0) GOTO 590
          ELSE IF (NAXIS(I).EQ.1 .AND. CROTA(I).EQ.0) THEN
              WRITE (TEXT, '(''Axis'',I2,'' = '',A,'' ignored'')')
     1             I, CTYPE(I)
              CALL LIST(TEXT)
          ELSE
              WRITE(TEXT,'(''Axis '',I2,'' = '',A,I6)')
     1              I, CTYPE(I), NAXIS(I)
              CALL LIST(TEXT)
              CALL LIST('++ERROR++ Unrecognized axis type')
              STATUS = 1
              RETURN
          END IF
  400 CONTINUE
C
C Check parameters. Parameters used by AIPS are:
C       UU, VV, WW : spatial frequency coordinates, in seconds
C       BASELINE: antenna1*256 + antenna2 + (0.01)*(array-1)
C       DATE: TAI Julian date, in days
C       SOURCE: source number
C
      PTUU = 0
      PTVV = 0
      PTWW = 0
      PTBASE = 0
      PTDAT1 = 0
      PTDAT2 = 0
      TEXT = 'Parameters:'
      DO 410 I=1,PCOUNT
          TEXT(LEN1(TEXT)+2:) = PTYPE(I)
          IF (PTYPE(I).EQ.'UU') THEN
              PTUU = I
          ELSE IF (PTYPE(I).EQ.'VV') THEN
              PTVV = I
          ELSE IF (PTYPE(I).EQ.'WW') THEN
              PTWW = I
          ELSE IF (PTYPE(I).EQ.'BASELINE') THEN
              PTBASE = I
          ELSE IF (PTYPE(I).EQ.'DATE') THEN
              IF (PTDAT1.EQ.0) THEN
                  PTDAT1 = I
C                 -- convert Julian Date to Modified JD
                  PZERO(I) = PZERO(I)-2400000.5D0
              ELSE
                  PTDAT2 = I
              END IF
          END IF
  410 CONTINUE
      CALL LIST(TEXT)
C
C Compute size of each group in bytes.
C
      NPIX = 1
      DO 420 I=2,MPNDM
          NPIX = NPIX*NAXIS(I)
  420 CONTINUE
C
C All done: return.  STATUS=0 implies success, STATUS=1 implies failure.
C
      STATUS = 0
      RETURN
C
C Error returns.
C
  560 CALL LIST('++ERROR++ Invalid parameter: '//KEYW//'='//CVAL)
      STATUS = 1
      RETURN
  570 CALL LIST('++ERROR++ Required header parameter missing: '//KEYW)
      STATUS = 1
      RETURN
  580 CALL LIST('++ERROR++ Axis is not degenerate')
      STATUS = 1
      RETURN
  590 CALL LIST('++ERROR++ Rotated axes not allowed')
      STATUS = 1
      RETURN
C
      END

      SUBROUTINE FITSEX (INUNIT, STATUS)
      INTEGER INUNIT, STATUS
C-----------------------------------------------------------------------
C FITSEX: read FITS extension file.
C
C     Parameters -   (">" input, "<" output )
C
C     (>) INUNIT  (Integer) Fortran unit number for the input file.
C                 The file must have been opened using a Fortran
C                 open statement, and UVFHED assumes the file is
C                 positioned at the start of the header record(s).
C     (<) STATUS  (Integer) A status code for the read.
C                 0 => OK, 1 => error
C-----------------------------------------------------------------------
C     -- maximum number of keywords in extension header
      INTEGER MXTBKW
      PARAMETER (MXTBKW=200)
C
      INTEGER I, N, GCOUNT, PCOUNT, NPIX, NBITS, NRECS, NTBKW, EXTVER
      INTEGER BITPIX, NAXES, NAXIS(7), LCARD, TYPE, ISTAT
      DOUBLE PRECISION RVAL
      CHARACTER HEADER*2880, KEYW*8, CVAL*72, CARD*80
      CHARACTER*80 KEYWRD(MXTBKW)
      CHARACTER*8  XTENSI, EXTNAM
      LOGICAL TABLE, TABLE3
      INTEGER LEN1
C-----------------------------------------------------------------------
C
C Read in the next record.
C
      NTBKW  = 0
      XTENSI = ' '
      EXTNAM = ' '
      EXTVER = 1
      NAXES  = 0
      BITPIX = 0
      GCOUNT = 1
      PCOUNT = 0
   10 CALL RDFITC(INUNIT, ISTAT, HEADER)
      IF (ISTAT.NE.0) THEN
          STATUS = 1
          RETURN
      END IF
C
C Decode the header records. Records that aren't recognized are
C stored for later use.
C
      DO 20 I=0,35
          CARD = HEADER(80*I+1:80*I+80)
          LCARD = LEN1(CARD)
          CALL FITKEY(CARD, KEYW, TYPE, RVAL, CVAL)
          IF (TYPE.EQ.6 .OR. KEYW.EQ.'AUTHOR' .OR.
     1                       KEYW.EQ.'REFERENC') THEN
              CALL LIST(CARD)
          ELSE IF (KEYW.EQ.'XTENSION') THEN
              XTENSI = CVAL
          ELSE IF (KEYW.EQ.'EXTNAME') THEN
              EXTNAM = CVAL
          ELSE IF (KEYW.EQ.'EXTVER') THEN
              EXTVER = INT(RVAL)
          ELSE IF (KEYW.EQ.'END') THEN
              GOTO 320
          ELSE IF (KEYW.EQ.'BITPIX') THEN
              BITPIX = RVAL
          ELSE IF (KEYW.EQ.'NAXIS') THEN
              NAXES = RVAL
              IF (NAXES.LT.0 .OR. NAXES.GT.7) GOTO 560
          ELSE IF (KEYW(1:5).EQ.'NAXIS') THEN
              N = ICHAR(KEYW(6:6)) - ICHAR('0')
              IF (N.LT.1 .OR. N.GT.NAXES) GOTO 560
              NAXIS(N) = RVAL
          ELSE IF (KEYW.EQ.'GCOUNT') THEN
              GCOUNT = RVAL
          ELSE IF (KEYW.EQ.'PCOUNT') THEN
              PCOUNT = RVAL
          ELSE
              NTBKW = NTBKW+1
              IF (NTBKW.LE.MXTBKW) KEYWRD(NTBKW) = CARD
          END IF
   20 CONTINUE
      GOTO 10
C
  320 CONTINUE
      IF (NTBKW.GT.MXTBKW) CALL LIST(
     1    '++WARNING++ Too many header records in FITS extension file')
C
C Compute the number of data records.
C
      NPIX = 1
      DO 330 I=1,NAXES
          NPIX = NPIX*NAXIS(I)
  330 CONTINUE
      NBITS = ABS(BITPIX)*GCOUNT*(PCOUNT+NPIX)
      NRECS = (NBITS+23039)/23040
C
      TABLE = XTENSI.EQ.'TABLE' .AND. BITPIX.EQ.8 .AND.
     1        NAXES.EQ.2 .AND. PCOUNT.EQ.0 .AND. GCOUNT.EQ.1 .AND.
     2        NAXIS(1).LE.1024 .AND. EXTVER.EQ.1
      TABLE3= (XTENSI.EQ.'A3DTABLE' .OR. XTENSI.EQ.'BINTABLE') .AND. 
     1        BITPIX.EQ.8 .AND. NAXES.EQ.2 .AND. PCOUNT.EQ.0 .AND. 
     2        GCOUNT.EQ.1 .AND. NAXIS(1).LE.1024 .AND. EXTVER.EQ.1
C
C Is the file a standard FITS table of a recognisable type?
C
      IF (TABLE .AND. EXTNAM.EQ.'AIPS AN') THEN
          CALL ANTAB(INUNIT, NAXIS(1), NAXIS(2), KEYWRD, NTBKW)
C
C Or an AIPS 3D-table?
C
      ELSE IF (TABLE3 .AND. EXTNAM.EQ.'AIPS AN') THEN
          CALL ANTAB3(INUNIT, NAXIS(1), NAXIS(2), KEYWRD, NTBKW)
C
C If not, skip the data.
C
      ELSE
          CALL LIST('Skipping FITS extension file: '//XTENSI//
     1                ' '//EXTNAM)
          DO 340 I=1,NRECS
              CALL RDFITC(INUNIT, ISTAT, HEADER)
  340     CONTINUE
      END IF
C
C All done: return.  STATUS=0 implies success, STATUS=1 implies failure.
C
      STATUS = 0
      RETURN
C
C Error returns.
C
  560 CALL LIST('++ERROR++ Invalid parameter: '//KEYW//'='//CVAL)
      STATUS = 1
      RETURN
C
      END

      SUBROUTINE READIT(INDAT, BITPIX, GCOUNT, PCOUNT, NPIX,
     1                  UU, VV, WW, BASE, DATE, VISR, VISI, VISWT)
      INTEGER INDAT, BITPIX, GCOUNT, PCOUNT, NPIX
      REAL    UU(GCOUNT), VV(GCOUNT), WW(GCOUNT)
      INTEGER BASE(GCOUNT)
      DOUBLE PRECISION DATE(GCOUNT)
      REAL   VISR(GCOUNT), VISI(GCOUNT), VISWT(GCOUNT)
C-----------------------------------------------------------------------
C Read 16-bit or 32-bit integer data or 32-bit floating data,
C
C Input:
C     INDAT: Fortran unit number
C     BITPIX: bits per pixel (must be 16 or 32 or -32)
C     GCOUNT: number of visiblity samples (groups)
C     PCOUNT: number of random parameters
C     NPIX: number of pixels per sample
C Common input:
C     BSCALE, BZERO: scaling parameters for pixels
C     PSCAL, PZERO: scaling parameters for random parameters
C     PTUU, PTVV, PTWW, PTBASE, PTDAT1, PTDAT2: indices of
C         random parameters
C Output:
C     UU, VV, WW: baseline coordinates
C     BASE: baseline label
C     DATE: Julian date
C     VISR, VISI: complex visibility
C     VISWT: visibility weight
C-----------------------------------------------------------------------
      INTEGER NW16, NW32, MAXP, MAXPIX
      PARAMETER (NW16=2880/2, NW32=2880/4)
      PARAMETER (MAXP=32)
      PARAMETER (MAXPIX=3*128)
      INTEGER*2 REC16(NW16)
      INTEGER   REC32(NW32)
      REAL      RECR4(NW32)
      EQUIVALENCE (REC16, REC32, RECR4)
      INTEGER I, J, K, LREC, IER, NBITS, NREC
      CHARACTER*80 TEXT
      DOUBLE PRECISION PARM(0:MAXP), PIX(MAXPIX)
      DOUBLE PRECISION PMAX(MAXP), PMIN(MAXP)
      DOUBLE PRECISION BSCALE, BZERO, WTSCAL
      DOUBLE PRECISION PSCAL(MAXP), PZERO(MAXP)
      INTEGER PTUU, PTVV, PTWW, PTBASE, PTDAT1, PTDAT2
      COMMON /FITS1/ BSCALE, BZERO, PSCAL, PZERO,
     1               PTUU, PTVV, PTWW, PTBASE, PTDAT1, PTDAT2, WTSCAL
C-----------------------------------------------------------------------
      LREC = PCOUNT+NPIX
      NBITS = ABS(BITPIX)*GCOUNT*LREC
      NREC = (NBITS+23039)/23040
      IF (PCOUNT.GT.MAXP) CALL ERROR('Too many random parameters.')
      IF (NPIX.GT.MAXPIX) CALL ERROR('Too many pixels per sample.')
C
      DO 5 J=1,PCOUNT
          PMAX(J) = -1E38
          PMIN(J) =  1E38
    5 CONTINUE
C
      PARM(0) = 0D0
      IF (BITPIX.EQ.16) THEN
          CALL LIST('16-bit integer data')
          K = NW16
          DO 20 I=1,GCOUNT
              DO 10 J=1,PCOUNT
                  IF (K.GE.NW16) THEN
                      CALL RDFITS(INDAT, IER, REC16)
                      CALL FITS_BSWAP(REC16, NW16)
                      K = 1
                  ELSE
                      K = K+1
                  END IF
                  PARM(J) = REC16(K)*PSCAL(J) + PZERO(J)
                  PMAX(J) = MAX(PMAX(J),PARM(J))
                  PMIN(J) = MIN(PMIN(J),PARM(J))
   10         CONTINUE
              DO 15 J=1,NPIX
                  IF (K.GE.NW16) THEN
                      CALL RDFITS(INDAT, IER, REC16)
                      CALL FITS_BSWAP(REC16, NW16)
                      K = 1
                  ELSE
                      K = K+1
                  END IF
                  PIX(J) = REC16(K)*BSCALE + BZERO
   15         CONTINUE
C             -- sort out the parameters
              UU(I) = PARM(PTUU)
              VV(I) = PARM(PTVV)
              WW(I) = PARM(PTWW)
              BASE(I) = NINT(PARM(PTBASE))
              DATE(I) = PARM(PTDAT1) + PARM(PTDAT2)
              VISR(I) = PIX(1)
              VISI(I) = PIX(2)
              VISWT(I) = PIX(3)*WTSCAL
   20     CONTINUE
      ELSE IF (BITPIX.EQ.32) THEN
          CALL LIST('32-bit integer data')
          K = NW32
          DO 120 I=1,GCOUNT
              DO 110 J=1,PCOUNT
                  IF (K.GE.NW32) THEN
                      CALL RDFITS(INDAT, IER, REC32)
                      CALL FITS_WBSWAP(REC32, NW32)
                      K = 1
                  ELSE
                      K = K+1
                  END IF
                  PARM(J) = REC32(K)*PSCAL(J) + PZERO(J)
                  PMAX(J) = MAX(PMAX(J),PARM(J))
                  PMIN(J) = MIN(PMIN(J),PARM(J))
  110         CONTINUE
              DO 115 J=1,NPIX
                  IF (K.GE.NW32) THEN
                      CALL RDFITS(INDAT, IER, REC32)
                      CALL FITS_WBSWAP(REC32, NW32)
                      K = 1
                  ELSE
                      K = K+1
                  END IF
                  PIX(J) = REC32(K)*BSCALE + BZERO
  115         CONTINUE
C             -- sort out the parameters
              UU(I) = PARM(PTUU)
              VV(I) = PARM(PTVV)
              WW(I) = PARM(PTWW)
              BASE(I) = NINT(PARM(PTBASE))
              DATE(I) = PARM(PTDAT1) + PARM(PTDAT2)
              VISR(I) = PIX(1)
              VISI(I) = PIX(2)
              VISWT(I) = PIX(3)*WTSCAL
  120     CONTINUE
      ELSE IF (BITPIX.EQ.-32) THEN
          CALL LIST('32-bit IEEE floating-point data')
          K = NW32
          DO 140 I=1,GCOUNT
              DO 130 J=1,PCOUNT
                  IF (K.GE.NW32) THEN
                      CALL RDFITS(INDAT, IER, RECR4)
                      CALL ZR32RL(NW32, 1, RECR4, RECR4)
                      K = 1
                  ELSE
                      K = K+1
                  END IF
                  PARM(J) = RECR4(K)*PSCAL(J) + PZERO(J)
                  PMAX(J) = MAX(PMAX(J),PARM(J))
                  PMIN(J) = MIN(PMIN(J),PARM(J))
  130         CONTINUE
              DO 135 J=1,NPIX
                  IF (K.GE.NW32) THEN
                      CALL RDFITS(INDAT, IER, RECR4)
                      CALL ZR32RL(NW32, 1, RECR4, RECR4)
                      K = 1
                  ELSE
                      K = K+1
                  END IF
                  PIX(J) = RECR4(K)*BSCALE + BZERO
  135         CONTINUE
C             -- sort out the parameters
              UU(I) = PARM(PTUU)
              VV(I) = PARM(PTVV)
              WW(I) = PARM(PTWW)
              BASE(I) = NINT(PARM(PTBASE))
              DATE(I) = PARM(PTDAT1) + PARM(PTDAT2)
              VISR(I) = PIX(1)
              VISI(I) = PIX(2)
              VISWT(I) = PIX(3)*WTSCAL
  140     CONTINUE
      END IF
      IF (IER.NE.0) CALL ERROR('Error reading FITS data.')
C
      DO 410 J=1,PCOUNT
          WRITE(TEXT,'(''Parameter '',I2,'': range = '',2G16.8)')
     1            J, PMIN(J), PMAX(J)
          CALL LIST(TEXT)
  410 CONTINUE
C
      END

      SUBROUTINE CHKTIM(GCOUNT, DATE, IDX, DT)
      INTEGER GCOUNT
      DOUBLE PRECISION DATE(GCOUNT)
      INTEGER IDX(GCOUNT)
      DOUBLE PRECISION DT
C-----------------------------------------------------------------------
C Check that dates in DATE are in increasing order. Compute reference
C date and GAST.
C-----------------------------------------------------------------------
      INCLUDE 'mrghed.inc'
      INTEGER I, IY, IM, ID, J, SECS
      LOGICAL ORDER
      DOUBLE PRECISION FD, DT1, DT2
      DOUBLE PRECISION SLA_DAT, SLA_GMST, SLA_EQEQX
      CHARACTER*80 TEXT
      CHARACTER*8  TIMEC
      CHARACTER*3  MONTH(12)
      DATA MONTH/'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug',
     1           'Sep','Oct','Nov','Dec'/
C-----------------------------------------------------------------------
  600 FORMAT('Time range (TAI modified Julian date):',
     1       F12.3,' to ',F12.3)
  610 FORMAT(2X,A5,' UTC: ',I4,1X,A3,1X,I2.2,1X,A8)
  620 FORMAT('  Converting from TAI to UTC: TAI-UTC =',F6.2,' s')
C-----------------------------------------------------------------------
C
      ORDER = .TRUE.
      IDX(1) = 1
      DO 10 I=2,GCOUNT
          IF (DATE(I).LT.DATE(I-1)) ORDER = .FALSE.
          IDX(I) = I
   10 CONTINUE
      CALL LIST(' ')
      IF (ORDER) THEN
          CALL LIST('Data are in increasing time order.')
      ELSE
          CALL LIST('Sorting data into increasing time order.')
          CALL IDSORT(DATE, GCOUNT, IDX)
      END IF
C
C Show time range in TAI Modified Julian Date.
C
      WRITE (TEXT, 600) DATE(1), DATE(GCOUNT)
      CALL LIST(TEXT)
C
C Convert from TAI to UTC.
C WARNING: the input to SLA_DAT should be UTC, not TAI!
C
      IF (DT .EQ. 0D0) THEN
          DT1 = SLA_DAT(DATE(IDX(1)))
          DT2 = SLA_DAT(DATE(IDX(GCOUNT)))
      ELSE
          CALL LIST('User-supplied value of TAI-UTC overrides default:')
          DT1 = DT
          DT2 = DT
      END IF
      IF (DT1.NE.DT2) THEN
          CALL LIST('A leap second occurred during the observations.')
          CALL LIST('Unable to proceed!')
          CALL ERROR('Leap second error')
      ELSE
          WRITE (TEXT, 620) DT1
          CALL LIST(TEXT)
          DT1 = DT1/86400D0
          DO 20 I=1,GCOUNT
              DATE(I) = DATE(I) - DT1
   20     CONTINUE
      END IF
      CALL SLA_DJCL(DATE(IDX(1)), IY, IM, ID, FD, J)
      IF (J.EQ.-1) THEN
          TEXT = 'Start: invalid date!'
      ELSE
          SECS = NINT(86400*FD)
          WRITE (TEXT,610) 'Start', IY, MONTH(IM), ID, TIMEC(SECS)
      END IF
      CALL LIST(TEXT)
      CALL SLA_DJCL(DATE(IDX(GCOUNT)), IY, IM, ID, FD, J)
      IF (J.EQ.-1) THEN
          TEXT = '  End: invalid date!'
      ELSE
          SECS = NINT(86400*FD)
          WRITE (TEXT,610) '  End', IY, MONTH(IM), ID, TIMEC(SECS)
      END IF
      CALL LIST(TEXT)
C
C Calculate reference date for merge file (0 UTC on first day).
C
      MGEPO = INT(DATE(IDX(1)))
      MGGAST = SLA_GMST(MGEPO) + SLA_EQEQX(MGEPO)
C***CAUTION*** We need to convert UTC to UT1 as well here!
C
      END

      SUBROUTINE CHKBAS(GCOUNT, BASE)
      INTEGER GCOUNT
      INTEGER BASE(GCOUNT)
C-----------------------------------------------------------------------
C Find what stations and baselines have data.
C-----------------------------------------------------------------------
      INCLUDE 'mrghed.inc'
      INTEGER I, S1, S2, BAD, NS, NB
      INTEGER COUNT(MGMAXS,MGMAXS), OBASE(MGMAXB)
      CHARACTER*132 TEXT
C-----------------------------------------------------------------------
  600 FORMAT('Number of antennas used  =',I4)
  605 FORMAT('Number of baselines used =',I4)
  610 FORMAT('++WARNING++ ',I10,
     1 ' visibilities could not be assigned to baselines!')
C-----------------------------------------------------------------------
      NS = 0
      DO 2 S2=1,MGMAXS
          DO 1 S1=1,MGMAXS
              COUNT(S1,S2) = 0
    1     CONTINUE
    2 CONTINUE
      DO 10 I=1,GCOUNT
          S1 = BASE(I)/256
          S2 = MOD(BASE(I),256)
          IF (S1.LT.1 .OR. S1.GT.MGMAXS .OR.
     1        S2.LT.1 .OR. S2.GT.MGMAXS) THEN
              BAD = BAD+1
          ELSE
              COUNT(S1,S2) = COUNT(S1,S2)+1
              NS = MAX(NS, S1, S2)
          END IF
   10 CONTINUE
C
C Make a list of the baselines that will be included in the output file.
C
      NB = 0
      DO 50 S2=2,NS
          DO 40 S1=1,S2-1
              IF (COUNT(S1,S2).GT.0) THEN
                  NB = NB+1
                  OBASE(NB) = 256*S1 + S2
                  MGBIDX(NB) = 100*S1 + S2
              END IF
   40     CONTINUE
   50 CONTINUE
C
C Parameters for MERGE header.
C
      MGNS = NS
      MGNB = NB
C
C Write a summary of data points by antennas.
C
      CALL LIST(' ')
      WRITE (TEXT, 600) NS
      CALL LIST(TEXT)
      WRITE (TEXT, 605) NB
      CALL LIST(TEXT)
      WRITE (TEXT, 610) BAD
      IF (BAD.GT.0) CALL LIST(TEXT)
      CALL LIST('Number of samples on each baseline:')
      WRITE (TEXT, '(6X,25(I4,1X))') (S2,S2=1,MIN(25,NS))
      CALL LIST(TEXT)
      DO 30 S1=1,NS
          WRITE (TEXT, '(2X,I2,'': '',25I5.0)')
     1        S1, (COUNT(S1,S2),S2=1,MIN(25,NS))
          CALL LIST(TEXT)
   30 CONTINUE
C     
      END

      SUBROUTINE MGINIT(FMT)
      INTEGER FMT
C-----------------------------------------------------------------------
C Initialize the header for the output file. The argument FMT specifies
C the format (1 or 2) of the file.
C-----------------------------------------------------------------------
      INCLUDE 'mrghed.inc'
      INTEGER I, J
C
      MGVERS = FMT
      MGNHIS = 0
      MGNS   = 0
      MGNB   = 0
      MGNT   = 0
      MGTYPE = 0
      MGSTOK = 0
      DO 20 I=1,MGMAXS
          MGSTYP(I) = 1
          MGSTN(I) = CHAR(I-1+ICHAR('A'))
          DO 10 J=1,9
              MGXYZ(J,I) = 0D0
   10     CONTINUE
   20 CONTINUE
      MGSRC = ' '
      MGRA50 = 0D0
      MGDC50 = 0D0
      MGARA  = 0D0
      MGADEC = 0D0
      MGFLUX = 0D0
      MGEPO  = 0D0
      MGGAST = 0D0
      MGFREQ = 0D0
      MGBW   = 0D0
      MGAVG1 = 0D0
      MGAVG2 = 0D0
      MGEQUI = 0D0
C
      END

      SUBROUTINE MGWRIT(U, SCR, NHIST)
      INTEGER U, SCR, NHIST
C-----------------------------------------------------------------------
C Write MERGE header. 
C Input:
C  U      Fortran unit number for output file.
C  SCR    Fortran unit number from which to read history.
C  NHIST  Number of history records in SCR file.
C-----------------------------------------------------------------------
      INCLUDE 'mrghed.inc'
      INTEGER I, J, YEAR, IUT, REC1(20), IM, ID, NY, ND, IER
      DOUBLE PRECISION DUMMY, FD, R2000, D2000, DR, DD
      CHARACTER*80 HISTRY
C
C First record.
C
      REC1(1) = MGVERS
      REC1(2) = NHIST
      REC1(3) = NHIST + MGNS + 2
      REC1(4) = MGNS
      REC1(5) = MGNB
      REC1(6) = MGNT
      IF (MGVERS.EQ.1) THEN
          REC1(7) = 1 + 4*MGNB + 2*MGNT
      ELSE IF (MGVERS.EQ.1) THEN
          REC1(7) = 1 + 7*MGNB + 2*MGNT
      END IF
      REC1(8) = MGTYPE
      REC1(9) = MGSTOK
      DO 10 I=10,20
          REC1(I) = 0
   10 CONTINUE
      IF (MGNB.GT.0) REC1(3) = REC1(3)+1
      IF (MGNT.GT.0) REC1(3) = REC1(3)+1
C======
C Workaround for bug in Sun Fortran 1.3, 1.4. Remove when bug is fixed.
      WRITE (U) REC1
      REWIND (UNIT=U)
C======
      WRITE (U) REC1
C
C Index records.
C
      IF (MGNB.GT.0) WRITE (U) (MGBIDX(I),I=1,MGNB)
      IF (MGNT.GT.0) WRITE (U) (MGTIDX(I),I=1,MGNT)
C
C History records.
C
      DO 15 I=1,NHIST
          READ (SCR,'(A)') HISTRY
          WRITE (U) HISTRY
   15 CONTINUE
C
C Source parameters; compute apparent place if necessary.
C
      IF (MGRA50.EQ.0D0 .OR. MGDC50.EQ.0D0) THEN
         CALL LIST('++WARNING++ Source coordinates missing')
         MGRA50 = 0D0
         MGDC50 = 0D0
         MGARA  = 0D0
         MGADEC = 0D0
         MGEQUI = 0D0
      ELSE IF (MGEQUI.EQ.2000D0) THEN
         CALL LIST('Calculating B1950.0 coordinates from input J2000.0')
         R2000 = MGRA50
         D2000 = MGDC50
C        -- Convert J2000 to B1950
         CALL SLA_FK54Z(R2000, D2000, 1950D0, MGRA50, MGDC50, DR, DD)
C        -- Convert J2000 to geocentric apparent place          
         CALL SLA_MAP(R2000, D2000, 0D0, 0D0, 0D0, 0D0,
     :                 2000D0, MGEPO, MGARA, MGADEC)
      ELSE IF (MGEQUI.EQ.1950D0 .OR. MGEQUI.EQ.0D0) THEN
         CALL LIST('Using B1950.0 coordinates from input')
C        -- Convert B1950 FK4 to J2000 FK5, assuming observed at 1975
         CALL SLA_FK45Z(MGRA50, MGDC50, 1975D0, R2000, D2000)
C        -- Convert J2000 to geocentric apparent place          
         CALL SLA_MAP(R2000, D2000, 0D0, 0D0, 0D0, 0D0,
     :                 2000D0, MGEPO, MGARA, MGADEC)
      ELSE
         CALL LIST('++WARNING++ Equinox of input coordinates is not'//
     :             ' B1950 or J2000')
         MGRA50 = 0D0
         MGDC50 = 0D0
         MGARA  = 0D0
         MGADEC = 0D0
         MGEQUI = 0D0
      END IF
      DUMMY = 0D0
      WRITE (U) MGSRC, MGRA50, MGDC50, MGARA, MGADEC, MGFLUX,
     1          DUMMY, DUMMY, DUMMY
C
C Station parameters.
C
      DO 20 I=1,MGNS
          WRITE (U) MGSTN(I), (MGXYZ(J,I),J=1,9), MGSTYP(I)
   20 CONTINUE
C
C Additional parameters.
C
      CALL SLA_DJCL(MGEPO, YEAR, IM, ID, FD, IER)
      IF (IER.EQ.-1 .OR. ABS(FD).GT.1E-10) 
     1    CALL ERROR('Invalid reference epoch.')
      CALL SLA_CALYD(YEAR, IM, ID, NY, ND, IER)
      IUT = (ND-1)*86400*60
      WRITE (U) YEAR, IUT, MGGAST, MGFREQ, MGAVG1, MGAVG2, MGBW,
     1          DUMMY, DUMMY, DUMMY, DUMMY
C
      END

      SUBROUTINE MGOUT (U, GCOUNT, WTFAC, WTPOW,
     1                  UU, VV, WW, BASE, DATE, VISR, VISI, VISWT,
     2                  IDX)
      INTEGER U
      INTEGER GCOUNT
      REAL    WTFAC, WTPOW
      REAL    UU(GCOUNT), VV(GCOUNT), WW(GCOUNT)
      INTEGER BASE(GCOUNT)
      DOUBLE PRECISION DATE(GCOUNT)
      REAL   VISR(GCOUNT), VISI(GCOUNT), VISWT(GCOUNT)
      INTEGER IDX(GCOUNT)
C
C U: Fortran unit for output.
C GCOUNT: number of visibility samples.
C-----------------------------------------------------------------------
      REAL PI, DEGREE
      PARAMETER (PI=3.14159265359D0)
      PARAMETER (DEGREE = 180D0/PI)
      INCLUDE 'mrghed.inc'
      INTEGER I, II, K, KK, IUT, NREC, NP, IS1, IS2, IY, IM, ID, IER
      REAL AMP(MGMAXB),AMPERR(MGMAXB),PHS(MGMAXB),PHSERR(MGMAXB)
      REAL UC(MGMAXB),VC(MGMAXB),WC(MGMAXB)
      LOGICAL FOUND(MGMAXB)
      CHARACTER*80 TXT
      DOUBLE PRECISION TIME, DATEP, REF, FD
C
C External and intrinsic functions.
C
      EXTERNAL LOOKUP
      INTEGER LOOKUP
      INTRINSIC SQRT, ATAN2, NINT
C-----------------------------------------------------------------------
C
C Convert dates from days (MJD) to seconds relative to start of year.
C
      CALL SLA_DJCL(MGEPO, IY, IM, ID, FD, IER)
      CALL SLA_CLDJ(IY, 1, 1, REF, IER)
      DO 10 I=1,GCOUNT
          DATE(I) = (DATE(I)-REF)*86400D0
   10 CONTINUE
C
      I = 0
      NREC = 0
      DATEP = DATE(IDX(1))
      DO 20 K=1,MGNB
          FOUND(K) = .FALSE.
          AMP(K) = 0.0
          AMPERR(K) = 0.0 
          PHS(K) = 0.0 
          PHSERR(K) = 0.0 
          UC(K) = 0.0 
          VC(K) = 0.0 
          WC(K) = 0.0
   20 CONTINUE
      TIME = 0D0
      NP = 0
C
C Group the data into records and output. Start a new record if
C (a) we already have some data for this baseline, or
C (b) more than 1 sec has elapsed.
C
      DO 100 II=1,GCOUNT
          I = IDX(II)
C         -- Which baseline?
          IS1 = BASE(I)/256
          IS2 = BASE(I) - 256*IS1
          K = LOOKUP(100*IS1+IS2, MGBIDX, MGNB)
          IF (K.EQ.0) GOTO 100
          IF (FOUND(K) .OR. (DATE(I)-DATEP .GT. 1D0)) THEN
C             -- Write the data record.
              IUT = NINT(60D0*TIME/DBLE(NP))
              CALL PUTREC(U, MGVERS, IUT, MGNB, AMP, AMPERR,
     1                    PHS, PHSERR, UC, VC, WC)
              NREC = NREC+1
              DATEP = DATE(I)
              DO 50 KK=1,MGNB
                  FOUND(KK) = .FALSE.
                  AMP(KK) = 0.0
                  AMPERR(KK) = 0.0 
                  PHS(KK) = 0.0
                  PHSERR(KK) = 0.0 
                  UC(KK) = 0.0
                  VC(KK) = 0.0
                  WC(KK) = 0.0
   50         CONTINUE
              TIME = 0D0
              NP = 0
          END IF
          FOUND(K) = .TRUE.
          AMP(K) = SQRT(VISR(I)**2 + VISI(I)**2)
C         -- convert AIPS weights to error bars
          IF (WTPOW.EQ.0.0) THEN
              IF (VISWT(I).EQ.0.0) THEN
                  AMPERR(K) = 0.0
              ELSE
                  AMPERR(K) = SIGN(WTFAC,VISWT(I))
              END IF
          ELSE IF(VISWT(I).EQ.0.0) THEN
C             -- zero weights indicate "unrecoverable" data
              AMPERR(K) = 0.0
          ELSE IF (VISWT(I).LT.0.0) THEN
C             -- negative weights go to negative error bars
              AMPERR(K) = -1.0*(WTFAC/ABS(VISWT(I)))**(1.0/WTPOW)
          ELSE
C             -- positive weights are OK
              AMPERR(K) = (WTFAC/VISWT(I))**(1.0/WTPOW)
          END IF
          IF (AMP(K).GT.0.0) THEN
              PHS(K) = ATAN2(VISI(I), VISR(I)) * DEGREE
              PHSERR(K) = MIN(180.0, DEGREE*AMPERR(K)/AMP(K))
          ELSE
              PHS(K) = 0.0
              PHSERR(K) = 180.0
          END IF
C         -- convert from seconds to wavelengths
          UC(K) = UU(I)*MGFREQ
          VC(K) = VV(I)*MGFREQ
          WC(K) = WW(I)*MGFREQ
          TIME = TIME+DATE(I)
          NP = NP+1
  100 CONTINUE
C
C There's always another record to write out.
C
      IUT = NINT(60D0*TIME/DBLE(NP))
      CALL PUTREC(U, MGVERS, IUT, MGNB, AMP, AMPERR,
     1            PHS, PHSERR, UC, VC, WC)
      NREC = NREC+1
C
C Write terminating record (optional).
C
      IUT = -1
      DO 120 K=1,MGNB
          AMP(K) = 0.0
          AMPERR(K) = 0.0 
          PHS(K) = 0.0 
          PHSERR(K) = 0.0 
          UC(K) = 0.0 
          VC(K) = 0.0 
          WC(K) = 0.0
  120 CONTINUE
      CALL PUTREC(U, MGVERS, IUT, MGNB, AMP, AMPERR,
     1            PHS, PHSERR, UC, VC, WC)
C
C Report number of records written.
C
      WRITE (TXT,'(''Number of MERGE data records written ='',I6)') NREC
      CALL LIST(TXT)
      END

      SUBROUTINE PUTREC(U, MGVERS, IUT, MGNB, AMP, AMPERR, PHS, PHSERR,
     1                  UC, VC, WC)
      INTEGER U, MGVERS, IUT, MGNB
      REAL AMP(MGNB), AMPERR(MGNB), PHS(MGNB), PHSERR(MGNB), UC(MGNB),
     1     VC(MGNB), WC(MGNB)
C
C Write one data record in merge file; all parameters are input.
C
C U: Fortran unit
C MGVERS: format version (1 or 2).
C IUT: time (UTC).
C AMP: amplitudes (by baseline).
C AMPERR: amplitude errors (by baseline).
C PHS: phases (by baseline).
C PHSERR: phase errors (by baseline).
C UC, VC, WC: spatial frequencies (by baseline).
C-----------------------------------------------------------------------
      INTEGER J
C
      IF (MGVERS.EQ.2) THEN
          WRITE (U) IUT, (AMP(J),AMPERR(J),PHS(J),PHSERR(J),UC(J),
     1                    VC(J),WC(J), J=1,MGNB)
      ELSE
          WRITE (U) IUT, (AMP(J),AMPERR(J),PHS(J),PHSERR(J), J=1,MGNB)
      END IF
C
      END

      SUBROUTINE ANTAB (INUNIT, WIDTH, NREC, CARDS, NTBKW)
      INTEGER INUNIT, WIDTH, NREC, NTBKW
      CHARACTER*80 CARDS(NTBKW)
C-----------------------------------------------------------------------
C Process a FITS extension file of type TABLE with EXTNAM='AIPS AN'.
C This is an AIPS antenna table. 
C The header has previously been read, and unprocessed header cards are
C stored in array CARDS. Restriction: the width of a table entry must 
C be 1024 characters or less, and the number of fields must be 20 or 
C less. The table is required to have columns labelled 'ANT NO.',
C 'STATION', 'LX', 'LY', and 'LZ'. The units of LX, LY, and LZ must
C be 'METERS'.
C
C Parameters:
C  INUNIT (input) : Fortran unit number
C  WIDTH  (input) : number of characters in each entry
C  NREC   (input) : number of entries
C  CARDS  (input) : unprocessed header records
C  NTBKW  (input) : number of unprocessed header records
C-----------------------------------------------------------------------
      INTEGER MAXF
      PARAMETER (MAXF=20)
      INTEGER TBCOL(MAXF)
      CHARACTER*16 TFORM(MAXF), TTYPE(MAXF), TUNIT(MAXF)
      DOUBLE PRECISION TSCAL(MAXF), TZERO(MAXF), RVAL, X, Y, Z
      CHARACTER*1024 ENTRY
      CHARACTER*2880 HEADER
      CHARACTER KEYW*8, CVAL*72
      INTEGER I, ISTAT, TYPE, N, TFLDS, AN, NIN1, NIN2, NOUT1, NOUT2
      INTEGER NBYTES
      INTEGER T(5)
      CHARACTER*16 NAME
      CHARACTER*80 TEXT
      INCLUDE 'mrghed.inc'
C
      CALL LIST('Processing AIPS antenna table (TABLE format)')
C
C Initialize the header fields.
C
      DO 10 I=1,MAXF
          TBCOL(I) = 0
          TFORM(I) = ' '
          TTYPE(I) = ' '
          TUNIT(I) = ' '
          TSCAL(I) = 1D0
          TZERO(I) = 0D0
   10 CONTINUE
C
C Extract parameters from the header.
C
      DO 20 I=1,NTBKW
          CALL FITKEY(CARDS(I), KEYW, TYPE, RVAL, CVAL)
          IF (KEYW.EQ.'TFIELDS') THEN
              TFLDS = INT(RVAL)
          ELSE IF (KEYW(1:5).EQ.'TBCOL') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TBCOL(N) = INT(RVAL)
          ELSE IF (KEYW(1:5).EQ.'TFORM') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TFORM(N)(1:1) = '('
              TFORM(N)(2:15) = CVAL(1:14)
              TFORM(N)(16:16) = ')'
          ELSE IF (KEYW(1:5).EQ.'TTYPE') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TTYPE(N) = CVAL
          ELSE IF (KEYW(1:5).EQ.'TUNIT') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TUNIT(N) = CVAL
          ELSE IF (KEYW(1:5).EQ.'TSCAL') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TSCAL(N) = RVAL
          ELSE IF (KEYW(1:5).EQ.'TZERO') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TZERO(N) = RVAL
          ELSE
              CALL LIST(CARDS(I))
          END IF
   20 CONTINUE
C
C Set up to read table entries.
C
      DO 25 I=1,5
          T(I) = 0
   25 CONTINUE
      DO 30 I=1,MAXF
          IF (TTYPE(I).EQ.'ANT NO.') THEN
              T(1) = I
          ELSE IF (TTYPE(I).EQ.'STATION') THEN
              T(2) = I
          ELSE IF (TTYPE(I).EQ.'LX' .AND.
     :             (TUNIT(I).EQ.'METERS' .OR. TUNIT(I).EQ. ' ')) THEN
              T(3) = I
          ELSE IF (TTYPE(I).EQ.'LY' .AND.
     :             (TUNIT(I).EQ.'METERS' .OR. TUNIT(I).EQ. ' ')) THEN
              T(4) = I
          ELSE IF (TTYPE(I).EQ.'LZ' .AND.
     :             (TUNIT(I).EQ.'METERS' .OR. TUNIT(I).EQ. ' ')) THEN
              T(5) = I
          END IF
   30 CONTINUE
      IF (T(1).EQ.0 .OR. T(2).EQ.0 .OR. T(3).EQ.0 .OR.
     :    T(4).EQ.0 .OR. T(5).EQ.0) THEN
          CALL LIST('++ WARNING++ Antenna table is not decodable')
          DO 321 I=1,TFLDS
              WRITE (TEXT,'(2X,I2,1X,A,1X,A)')
     1            I, TTYPE(I), TUNIT(I)
              CALL LIST(TEXT)
  321     CONTINUE
          NRECS = (WIDTH*NREC+2879)/2880
          DO 340 I=1,NRECS
              CALL RDFITC(INUNIT, ISTAT, HEADER)
  340     CONTINUE
          RETURN
      END IF
C
C Read and decode the table entries.
C
      NIN1 = 2880
      NIN2 = 0
      DO 40 I=1,NREC
C         Read an entry, which is not necessarily commensurate with
C         a FITS record.
          NOUT1 = 0
          NOUT2 = 0
C          -- do while
   32     IF (NOUT2.LT.WIDTH) THEN
               IF (NIN1.GE.NIN2) THEN
                   CALL RDFITC(INUNIT, ISTAT, HEADER)
                   NIN1 = 0
                   NIN2 = 2880
               END IF
               NBYTES = MIN(WIDTH-NOUT1, NIN2-NIN1)
               NOUT2 = NOUT1+NBYTES
               ENTRY(NOUT1+1:NOUT2) = HEADER(NIN1+1:NIN1+NBYTES)
               NOUT1 = NOUT2
               NIN1 = NIN1+NBYTES
               GOTO 32
          END IF
          READ (ENTRY(TBCOL(T(1)):WIDTH), FMT=TFORM(T(1))) AN
          READ (ENTRY(TBCOL(T(2)):WIDTH), FMT=TFORM(T(2))) NAME
          READ (ENTRY(TBCOL(T(3)):WIDTH), FMT=TFORM(T(3))) X
          READ (ENTRY(TBCOL(T(4)):WIDTH), FMT=TFORM(T(4))) Y
          READ (ENTRY(TBCOL(T(5)):WIDTH), FMT=TFORM(T(5))) Z
          X = X*TSCAL(T(3)) + TZERO(T(3))
          Y = Y*TSCAL(T(4)) + TZERO(T(4))
          Z = Z*TSCAL(T(4)) + TZERO(T(4))
          IF (AN.GT.0 .AND. AN.LE.MGMAXS) THEN
              CALL ASCCNV(NAME)
              MGSTN(AN) = NAME
              MGXYZ(1,AN) = X
              MGXYZ(2,AN) = Y
              MGXYZ(3,AN) = Z
          END IF
          WRITE (TEXT, '(I4,1X,A8,3(1X,F20.4))') AN,NAME,X,Y,Z
          CALL LIST(TEXT)
   40 CONTINUE
C
      END

      SUBROUTINE ANTAB3 (INUNIT, WIDTH, NREC, CARDS, NTBKW)
      INTEGER INUNIT, WIDTH, NREC, NTBKW
      CHARACTER*80 CARDS(NTBKW)
C-----------------------------------------------------------------------
C Process a FITS extension file of type A3DTABLE or BINTABLE with 
C EXTNAM='AIPS AN'.
C This is an AIPS antenna table. 
C The header has previously been read, and unprocessed header cards are
C stored in array CARDS. Restriction: the width of a table entry must 
C be 1024 bytes or less, and the number of fields must be 20 or 
C less. The table is required to have columns labelled 'NOSTA' (1I or 
C 1J), 'ANNAME' (nA), and 'STABXYZ' (3D) (units='METERS'). Other columns
C are ignored.
C
C Parameters:
C  INUNIT (input) : Fortran unit number
C  WIDTH  (input) : number of characters in each entry
C  NREC   (input) : number of entries
C  CARDS (input) : unprocessed header records
C  NTBKW  (input) : number of unprocessed header records
C-----------------------------------------------------------------------
      INTEGER MAXF
      PARAMETER (MAXF=20)
      INTEGER NIGN
      PARAMETER (NIGN=13)
      INTEGER TBCOL(MAXF), TBCNT(MAXF)
      CHARACTER*16 TFORM(MAXF), TTYPE(MAXF), TUNIT(MAXF)
      CHARACTER*1  TBFMT(MAXF)
      DOUBLE PRECISION RVAL, XYZ(3)
      CHARACTER*1024 ENTRY
      CHARACTER*2880 HEADER
      CHARACTER*8    KIGN(NIGN)
      CHARACTER KEYW*8, CVAL*72
      INTEGER I, ISTAT, TYPE, N, TFLDS, AN, NIN1, NIN2, NOUT1, NOUT2
      INTEGER NBYTES, K, C, BYTES, NRECS, NUMORB
      INTEGER T(5)
      CHARACTER*16 NAME
      CHARACTER*80 TEXT
      INCLUDE 'mrghed.inc'
C
      DATA KIGN /'ARRAYX', 'ARRAYY', 'ARRAYZ', 'GSTIA0', 'DEGPDY',
     1           'FREQ',   'RDATE',  'POLARX', 'POLARY', 'UT1UTC',
     2           'IATUTC', 'ARRNAM', 'NOPCAL' /
C
      CALL LIST('Processing AIPS antenna table (BINTABLE format)')
C
C Initialize the header fields.
C
      DO 10 I=1,MAXF
          TBCOL(I) = 0
          TBCNT(I) = 0
          TBFMT(I) = ' '
          TFORM(I) = ' '
          TTYPE(I) = ' '
          TUNIT(I) = ' '
   10 CONTINUE
C
C Extract parameters from the header.
C
      DO 20 I=1,NTBKW
          CALL FITKEY(CARDS(I), KEYW, TYPE, RVAL, CVAL)
          IF (KEYW.EQ.'TFIELDS') THEN
              TFLDS = INT(RVAL)
          ELSE IF (KEYW(1:5).EQ.'TFORM') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TFORM(N) = CVAL
          ELSE IF (KEYW(1:5).EQ.'TTYPE') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TTYPE(N) = CVAL
          ELSE IF (KEYW(1:5).EQ.'TUNIT') THEN
              READ (KEYW(6:8), FMT='(BN,I3)', ERR=20) N
              IF (N.LT.1 .OR. N.GT.MAXF) GOTO 20
              TUNIT(N) = CVAL
          ELSE IF (KEYW.EQ.'NUMORB') THEN
              NUMORB = INT(RVAL)
          ELSE
C             -- ignore other known keywords; else display card
              DO 15 K=1,NIGN
                  IF (KEYW.EQ.KIGN(K)) GOTO 20
   15         CONTINUE
              CALL LIST(CARDS(I))
          END IF
   20 CONTINUE
C
C Compute the widths and locations of each field by parsing the TFORM
C parameters.
C
      C = 0
      DO 22 I=1,TFLDS
          N = 0
          K = 1
C         -- do while
   21     IF ('0' .LE. TFORM(I)(K:K) .AND. TFORM(I)(K:K) .LE. '9') THEN
              N = N*10 + (ICHAR(TFORM(I)(K:K)) - ICHAR('0'))
              K = K+1
              GOTO 21
          END IF
          TBFMT(I) = TFORM(I)(K:K)
          TBCNT(I) = N
          TBCOL(I) = C+1
          BYTES = 1
          IF (TBFMT(I).EQ.'L') BYTES=1
          IF (TBFMT(I).EQ.'I') BYTES=2
          IF (TBFMT(I).EQ.'J') BYTES=4
          IF (TBFMT(I).EQ.'A') BYTES=1
          IF (TBFMT(I).EQ.'E') BYTES=4
          IF (TBFMT(I).EQ.'D') BYTES=8
          C = C + BYTES*N
   22 CONTINUE
C
C Set up to read table entries.
C
      DO 25 I=1,3
          T(I) = 0
   25 CONTINUE
      DO 30 I=1,TFLDS
C         -- fix error in AIPS 15OCT89 which pads with nulls
          TTYPE(I)(9:) = ' '
          IF (TTYPE(I).EQ.'NOSTA' .AND. TBFMT(I).EQ.'I') THEN
              T(1) = I
          ELSE IF (TTYPE(I).EQ.'NOSTA' .AND. TBFMT(I).EQ.'J') THEN
              T(1) = I
          ELSE IF (TTYPE(I).EQ.'ANNAME' .AND. TBFMT(I).EQ.'A') THEN
              T(2) = I
          ELSE IF (TTYPE(I).EQ.'STABXYZ' .AND. 
     :             (TUNIT(I).EQ.'METERS'  .OR. TUNIT(I).EQ.' ')
     :             .AND. TBFMT(I).EQ.'D' .AND. TBCNT(I).EQ.3) THEN
              T(3) = I
          END IF
   30 CONTINUE
      IF (T(1).EQ.0 .OR. T(2).EQ.0 .OR. T(3).EQ.0 .OR.NUMORB.NE.0) THEN
          CALL LIST('++ WARNING++ Antenna table is not decodable')
          DO 321 I=1,TFLDS
              WRITE (TEXT,'(2X,I2,1X,A,1X,A,I3,1X,A)')
     1            I, TTYPE(I), TBFMT(I), TBCNT(I), TUNIT(I)
              CALL LIST(TEXT)
  321     CONTINUE
          NRECS = (WIDTH*NREC+2879)/2880
          DO 340 I=1,NRECS
              CALL RDFITC(INUNIT, ISTAT, HEADER)
  340     CONTINUE
          RETURN
      END IF
C
C Read and decode the table entries.
C
      NIN1 = 2880
      NIN2 = 0
      DO 40 I=1,NREC
C         Read an entry, which is not necessarily commensurate with
C         a FITS record.
          NOUT1 = 0
          NOUT2 = 0
C          -- do while
   32     IF (NOUT2.LT.WIDTH) THEN
               IF (NIN1.GE.NIN2) THEN
                   CALL RDFITC(INUNIT, ISTAT, HEADER)
                   NIN1 = 0
                   NIN2 = 2880
               END IF
               NBYTES = MIN(WIDTH-NOUT1, NIN2-NIN1)
               NOUT2 = NOUT1+NBYTES
               ENTRY(NOUT1+1:NOUT2) = HEADER(NIN1+1:NIN1+NBYTES)
               NOUT1 = NOUT2
               NIN1 = NIN1+NBYTES
               GOTO 32
          END IF
          IF (TBFMT(T(1)).EQ.'I') THEN
              CALL ZI16(ENTRY(TBCOL(T(1)):), 1, AN)
          ELSE
              CALL ZI32(ENTRY(TBCOL(T(1)):), 1, AN)
          END IF
          NAME = ENTRY(TBCOL(T(2)):TBCOL(T(2)-1+TBCNT(T(2))))
          CALL ZR64(ENTRY(TBCOL(T(3)):), 3, XYZ)
          IF (AN.GT.0 .AND. AN.LE.MGMAXS) THEN
              CALL ASCCNV(NAME)
              MGSTN(AN) = NAME
              MGXYZ(1,AN) = XYZ(1)
              MGXYZ(2,AN) = XYZ(2)
              MGXYZ(3,AN) = XYZ(3)
              WRITE (TEXT, '(I4,1X,A8,3(1X,F20.4))') AN,NAME,XYZ
              CALL LIST(TEXT)
          END IF
   40 CONTINUE
C
      END

      SUBROUTINE LIST(TEXT)
      CHARACTER*(*) TEXT
C-----------------------------------------------------------------------
C Write a line in the listing file.
C-----------------------------------------------------------------------
      INTEGER LUNIT
      COMMON /UNFIT1/ LUNIT
      INTEGER LEN1, L
      L = MAX(1,LEN1(TEXT))
      CALL PUTOUT(TEXT(1:L))
      IF (LUNIT.NE.0) WRITE (LUNIT, '(A)') TEXT(1:L)
      END

      SUBROUTINE ZI16(S, N, OUT)
      CHARACTER*(*) S
      INTEGER N
      INTEGER OUT(N)
C-----------------------------------------------------------------------
C Extract N IEEE 16 bit integers from supplied character string, and
C return in array OUT.
C-----------------------------------------------------------------------
      INTEGER I
      INTEGER*2 WORD
      DO 10 I=1,N
          READ (S(2*I-1:), '(A2)') WORD
          CALL FITS_BSWAP(WORD, 1)
          OUT(I) = WORD
   10 CONTINUE
      END

      SUBROUTINE ZI32(S, N, OUT)
      CHARACTER*(*) S
      INTEGER N
      INTEGER OUT(N)
C-----------------------------------------------------------------------
C Extract N IEEE 32 bit integers from supplied character string, and
C return in array OUT.
C-----------------------------------------------------------------------
      INTEGER I
      INTEGER WORD
      DO 10 I=1,N
          READ (S(2*I-1:), '(A4)') WORD
          CALL FITS_WBSWAP(WORD, 1)
          OUT(I) = WORD
   10 CONTINUE
      END

      SUBROUTINE ZR64(S, N, OUT)
      CHARACTER*(*) S
      INTEGER N
      DOUBLE PRECISION OUT(N)
C-----------------------------------------------------------------------
C Extract N IEEE 64 bit floats from supplied character string, and
C return in array OUT. Uses the AIPS routine ZR64RL.
C-----------------------------------------------------------------------
      INTEGER I
      DOUBLE PRECISION WORD
      DO 10 I=1,N
          READ (S(8*I-7:), '(A8)') WORD
          CALL ZR64RL(1, 1, WORD, OUT(I))
   10 CONTINUE
      END

C*IDSORT -- indexed sort of double array into increasing order
C+
      SUBROUTINE IDSORT(A,N,IND)
      INTEGER N
      DOUBLE PRECISION A(N)
      INTEGER IND(N)
C
C Indexed sort of an array A into increasing order. Shell sort.
C
C Array IND(N) is sorted so that IND(I) is the index of array A(N).
C-----------------------------------------------------------------------
      INTEGER IGAP, IMAX, I, J, JPLUSG, IEX
C
      IGAP=N
    5 IF(IGAP.LE.1) RETURN
      IGAP=IGAP/2
      IMAX=N-IGAP
   10     IEX=0
          DO 20 I=1,IMAX
          J=IND(I)
          JPLUSG=IND(I+IGAP)
          IF(A(J).GT.A(JPLUSG)) GOTO 40
          GOTO 20
   40     IND(I)=JPLUSG
          IND(I+IGAP)=J
          IEX=IEX+1
   20   CONTINUE
      IF(IEX.GT.0) GOTO 10
      GOTO 5
      END

C*ASCCNV -- clean up character string
C+
      SUBROUTINE ASCCNV(STR)
      CHARACTER*(*) STR
C
C Replace unprintable characters with space; treat a null character as
C a terminator.
C-----------------------------------------------------------------------
      INTEGER I, K
      LOGICAL END
      END = .FALSE.
      DO 10 I=1,LEN(STR)
         K = ICHAR(STR(I:I))
         IF (K.EQ.0) END = .TRUE.
         IF (K.LE.31 .OR. K.GE.127 .OR. END) THEN
             STR(I:I) = ' '
         END IF
 10   CONTINUE
      END
