      PROGRAM VISPLOT
C-----------------------------------------------------------------------
C VISPLOT produces plots of visibility data (amplitude, phase, closure
C phase, and/or closure amplitude) versus time, with the option of
C plotting model curves. The data are read from a MERGE-format file
C (formats 1 and 2 are accepted). Any graphics device supported by
C PGPLOT can be used for the plot (e.g., Tek, VT125, Grinnell, ARGS 
C terminals; Printronix, Versatec, LaserWriter printers). 
C
C Parameters:
C  INPUT     input file name
C  PLOTFILE  graphics device spec
C  MODFILE   model file name
C  TIMEA     time base for time axis on amplitude plots:
C              'IHA'  'GST'  'UT'  or 'UTM' (UT mod 24 hr)
C  TIMEC     time base for time axis on closure phase plots:
C              'GST'  'UT'  or  'UTM' 
C  MAXWID    max width of plot in inches (<8) [8]
C  MINWID    min width of plot in inches [4]
C  NTICKS    number of ticks on vertical scale [5]
C  HTPLOT    height of each plot in inches [3.0]
C  HTSCAL    height of vertical scale on each plot [2.5]
C  HTTICK    length of short ticks [0.07]
C  HTLET     height of letters and numbers [0.10]
C  AMP       plot amplitudes
C  PHASE     plot phases
C  CLOSE     plot closure phases
C  CLOSEAMP  plot closure amplitudes:
C              if closeamp is specified, the program will 
C              ask if all closure amplitudes are desired 
C              (there may be a large number); if
C              not, it will ask for specific sets of baselines.
C  BAS       a list of up to 45 baselines for amplitudes
C              and phases [default: all]
C  TRIA      a list of up to 120 closure phases [default: all]
C              Note:  if none of AMP, CLOSE, PHASE, or CLOSEAMP 
C              are specified, AMP and CLOSE will be assumed
C  ASCALE    Value to use in place of data to set scale.
C  PSCALE    Phase scale will go from +PSCALE to -PSCALE
C  CSCALE    Closure phase scale will go from +CSCALE to -CSCALE
C  NOMOD24   Annotate UT time axis in straight hours, not modulo 24
C  UTMIN     Minimum UT accepted (default = minumum UT in data)
C  UTMAX     Maximum UT accepted (default = maximum UT in data)
C  NOERR     Suppress error bars
C
C Limitations:
C   Maximum number of stations is 20.
C
C Author:
C   R. Craig Walker
C   Modifications by T. J. Pearson, D. L. Meier, S. C. Unwin, et al.
C
C Language:
C   VAX-11 Fortran.
C
C History:
C   Version 1.0:              - new program (RCW).
C   Version 1.1:  1981 Nov 12 - phase plotting added and moved to 15
C                               stations (RCW).
C   Version 1.2:  1982 Apr  6 - misc bugs fixed (RCW).
C   Version 1.3:  1982 Jun 14 - bug in SYSPLT fixed (TJP).
C   Version 1.4:  1982 Jun 18 - Subroutines MODAP and MODCAL changed to
C                               make model visibility calculation using
C                               lookup tables.  Also, definitions of 
C                               the major axes of model types 2-5 
C                               redefined (see $ VHELP INTRO).
C                               Use colors for Grinnell plotting, add
C                               GRTERM in some places (SCU).
C   Version 1.5:  1982 Oct  2 - Fix bug in PHSAMP and MODCAL (type for
C                               CTYPE). Change colors for Grinnell.
C                               Increase NCP to 10000. Add LINEWIDTH
C                               parameter (default thickness=1) (SCU).
C   Version 1.6:  1982 Dec 15 - Fix bug in CLOSEAMP scales. Allow 8 
C                               character station names. Prevent 
C                               attempt to look at 2 station closure 
C                               phases. Add ASCALE, PSCALE, CSCALE 
C                               (RCW).
C   Version 1.7:  1983 Feb 11 - Add UTMIN, UTMAX, NOMOD24; add day of 
C                               month to plot label (DLM).
C   Version 1.8:  1983 Mar    - On an interactive device, reduce number
C                               of queries for <CR> to one per page 
C                               plotted; allow for erasure of text 
C                               written over plot (DLM).
C   Version 1.9:  1983 Apr  1 - Fix bug which draws boxes when no 
C                               closure amplitude exists (DLM).
C   Version 2.0:  1983 Aug  3 - Extend to include version 2 files.
C                               Include W in version 2 files (DLM).
C   Version 2.1:  1985 May 14 - Alter error bars on closure amplitude
C                               plots to reflect the nonlinearity of
C                               the LOG display (RCW).
C   Version 2.2:  1986 Aug 28 - Increase from 15 to 20 stations (TJP).
C   Version 3.0:  1986 Nov 17 - Change to use parameter input for file
C                               names (INPUT, PLOTFILE, MODFILE); also
C                               update the header comments to include
C                               retrospective version numbers (TJP).
C   Version 3.1:  1986 Dec  3 - Fix program to plot vertical scale 
C                               labels correctly when all baselines or
C                               triangles are not present. Introduce 
C                               new parameters PHTSCAL, COLOR, NOPLTMS,
C                               NOTEXT, ONEPAGE and SETWIDTH
C	                        (from A.K. Tzioumis 1986 April 28).
C   Version 3.2:  1986 Dec 23 - fix bug in NOTEXT option (TJP).
C   Version 3.3:  1987 Aug 19 - add error message if input file not 
C                               found (TJP).
C   Version 3.4:  1988 Apr  3 - changes for Convex-UNIX (TJP).
C   Version 3.5:  1990 Feb 22 - add NOERR parameter (TJP).
C   Version 3.6:  1992 Feb  5 - add PLOTDEL and SNR parameters (DLM).
C-----------------------------------------------------------------------
C
C Constants.
C
      CHARACTER*(*) VERSN
C     -- program version and date
      PARAMETER (VERSN='3.6 - 1992 Feb 5')
      PARAMETER (MAXS=20)	         ! maximum number of stations
      PARAMETER (MAXB=MAXS*(MAXS-1)/2)   ! maximum number of baselines
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)   ! maximum number of clos phase
      PARAMETER (MAXWPB=7)		 ! maximum words per baseline
      PARAMETER (MAXL=MAXWPB*MAXB+2*MAXT)!max length of data records.
      PARAMETER (MAXMOD=500)             ! max number of model points.
      PARAMETER (MAXVAL=237)             ! max number of KEYIN values.
      PARAMETER (MOSTB=45)               ! max specifiable baselines
      PARAMETER (MOSTC=120)              ! max specifiable clos phs.
      PARAMETER (INDAT=8)                ! data unit number
      PARAMETER (INMOD=10)               ! model unit number
C
C
      REAL*4  LEFT,INCHJY(MAXB),INCHDG,INHRB,INHRC,MINWID,MAXWID,LEFTC,
     +        INCHUN,LEFTA,INHRA
      LOGICAL   SKIPH,MODEL,FOOL,DOAMP,DOPHASE,DOCLOSE,DOCLAMP,
     +        NEWPAGE,ORBITERS
      LOGICAL   END,COEFF,MAX24
      LOGICAL USELOOK		! Lookup switch
      REAL    LOOK(4096)	! Lookup table
      DATA     END/.FALSE./,COEFF/.TRUE./,MAX24/.TRUE./,FOOL/.FALSE./,
     +   NEWPAGE/.FALSE./
C
C     VALUES FOR PLOT CONTROL
C
      CHARACTER*128  FILE, SCRDSN
      CHARACTER*255  WDATA,WMODEL,WWARN
      CHARACTER*17   REVTIM
      INTEGER PLTUNIT,TYPE,GROPEN,STATYP, VLBOPE
      INTEGER IWIDTH, COLOR(3), NPLTMS(2)
      LOGICAL NOMOD24, NOTEXT, ONEPAGE, SETWIDTH, NOERR, PLOTDEL,
     +        SNR
      DATA    NOTEXT/.FALSE./, ONEPAGE/.FALSE./
      DATA  TYPE/3/,PLTUNIT/11/,ICON/5184000/
C
      REAL*8 GASTM,FREQ,ENDMARK,XI,TINTG,AVG,BW   ! XI used in lookup
      REAL*8 STNAME,STX,STY,STZ,AEIOWM
      REAL*8 SOURCE(2),RA,DEC,FLUX,XDUM(130)
      COMMON/ORDER/  NS,NB,NC,NB1(MAXB),NB2(MAXB),
     +               NC1(MAXT),NC2(MAXT),NC3(MAXT),NVER,NDWPB
      COMMON /FLAGS/ NBP,NCP,MB1(MAXB),MB2(MAXB),
     +               MC1(MAXT),MC2(MAXT),MC3(MAXT)   
      COMMON /BLIMS/ BTBMAX(MAXB),BTBMIN(MAXB),
     +               CTCMAX(MAXT),CTCMIN(MAXT)
      COMMON  /LIMS/ BTMAX,CTMAX,BTMIN,CTMIN,UTMAX,UTMIN,UTBMAX(MAXB),
     +               UTBMIN(MAXB),UTCMAX(MAXT),UTCMIN(MAXT),
     +               AMPMAX(MAXB),ATMAX,ATMIN
      COMMON /VTICKS/ VALS(15),NDIG
      COMMON /PARMS/ STNAME(MAXS),STX(MAXS),STY(MAXS),STZ(MAXS),
     +               SOURCE,RA,DEC,GASTM,FREQ,FLUX,TINTG,AVG,BW,
     +               AEIOWM(6,MAXS),STATYP(MAXS),ORBITERS
      COMMON /TICKS/ WHERE(20),IHR(20),NTT,NOMOD24
      COMMON /TIMSCAL/ UTMIN0,UTMAX0
      COMMON /DATA/  AMP(MAXMOD,3),PHASE(MAXMOD,3),
     +               CLOS(MAXMOD),T(MAXMOD),IUT(MAXMOD)
      COMMON /T/     TAKE(MAXL)
      COMMON /LOOKUP/ USELOOK,LOOK	! Lookup switch,table
      DATA SIDR/1.00273791/,HRAD/3.819718634/,ENDMARK/'/       '/
C
      REAL*8          NAMES(MAXVAL),VALUES(MAXVAL)
      EQUIVALENCE     (TIMEA,VALUES(1)) , (TIMEC,VALUES(2))
      DATA NAMES/'TIMEA','TIMEC','MAXWID','MINWID','NTICKS','HTPLOT',
     +           'HTSCAL','HTTICK','HTLET','NP??????','AMP?','PHASE',
     +           'MODEL','PPRINT','NOLOOK??','UT','UTM','IHA','GST',
     +           'CLOSE','LINE????','CLOSEAMP',
     +           'ASCALE','PSCALE','CSCALE','NOMOD24',2*'UTMIN',
     +           2*'UTMAX', 3*'COLOR?',2*'NOPLTM?',45*'BAS??',
     +           120*'TRIA????',
     +           'INput', 9*' ',
     +           'PLotfile', 9*' ',
     +           'MODfile', 9*' ',
     +           'NOTEXT', 'PHTSCAL', 'ONEPAGE', 'SETWID??',
     +           'NOErrors', 'PLOTDEL', 'SNR'/
      DATA VALUES/'IHA',  'GST',  15.D0,    9.D0,    5.D0,    15.D0,
     +            15.D0,   0.07D0,  0.10D0, 5.D0,  -1.0D0,-1.0D0,
     +            -1.D0,  -1.D0,   -1.D0,-1.D0,-1.D0,-1.D0,-1.D0,
     +            -1.D0,   1.D0,   -1.D0,
     +             0.D0,   0.D0,    0.D0, -1.D0,-9999.,0.D0,
     +             9999.,0.D0, 5D0, 3D0, 1D0, 2*0D0, 45*0.D0, 120*0.D0,
     +            '***UNSET', 9*' ',
     +            '?', 9*' ',
     +            10*' ',
     +            -1D0, 0D0, -1D0, 0D0,
     +            -1D0, -1D0, -1D0 /
C
      DATA  HIHA/'IHA'/,HGST/'GST'/,HUT/'UT'/,HUTMOD/'UTM'/
      CHARACTER  SIHA*25, SGST*25, SUT*25, SCF*25,
     +           SCC*25,  SCS*25, SMF*15,  SFR*25, SDL*25, SPH*23
      CHARACTER  STIME*25, SCOEFF*25, DASH*1, SCLPH*24, SCLCA*24
      CHARACTER*60  STLAB
      DATA   SIHA /'INTERFEROMETER HOUR ANGLE'/,
     +       SGST /' GREENWICH SIDEREAL TIME '/,
     +       SUT  /'     UNIVERSAL TIME      '/,
     +       SCF  /'  CORRELATED FLUX (Jy)   '/,
     +       SCC  /' CORRELATION COEFFICIENT '/,
     +       SCS  /'  SIGNAL-TO-NOISE RATIO  '/,
     +       SFR  /'    FRINGE RATE (Hz)     '/,
     +       SMF  /'MODEL FLUX (Jy)'/,
     +       SDL  /'RESIDUAL DELAY (MICROSEC)'/,
     +       SPH  /'     PHASE (DEGREES)     '/
      DATA   DASH /'-'/
      DATA   SCLPH /'CLOSURE PHASE  (DEGREES)'/
      DATA   SCLCA /'LOG( CLOSURE AMPLITUDE )'/
      CHARACTER*6 FMTT
      DATA FMTT/'(F6.2)'/
      CHARACTER*128  PLOTIN,PLOTMOD
      CHARACTER*32   PLTYPE
      LOGICAL INTER
C-----------------------------------------------------------------------
 1006 FORMAT (' VISPLOT plots visibility data'/
     1        ' (Version ',A,')'/)
 1005 FORMAT(' Parameters - end with / - ',
     +  ' Commonly used variables are: '/
     +  '   INPUT,PLOTFILE,MODFILE,NP,AMP,PHASE,CLOSE,',
     +  'CLOSEAMP,UT,UTM,IHA,GST')
C-----------------------------------------------------------------------
C
C Introduction.
C
      WRITE (6,1006) VERSN
C
C Control parameters.
C
      MODE = 0
    1 IF (MODE.NE.0) CALL ERROR('Bad input parameters in VISPLOT.')
      WRITE (6,1005)
      MODE = 0
      CALL KEYIN (NAMES,VALUES,MAXVAL,ENDMARK,MODE,5,6)
C
C Input file.
C
      WRITE (PLOTIN,'(10A8)') (VALUES(J),J=201,210)
      IER = VLBOPE(INDAT, PLOTIN, 'MERGE', 'OLD', PLOTIN)
      IF (IER.NE.1) GOTO 1
      CALL FRDT(PLOTIN, REVTIM)
      WDATA(1:17)  = REVTIM
      WDATA(18:31) = ' Data file =  '
      WDATA(32:)   = PLOTIN
C
C Plot device.
C
      WRITE (FILE, '(10A8)') (VALUES(J),J=211,220)
      L = LEN1(FILE)
      IER = GROPEN(0,PLTUNIT,FILE,IDEVICE)
      IF (IER.NE.1) THEN
          WRITE (6,'(2A)') ' ++ERROR++ Cannot open PLOTFILE=',FILE(:L)
          GOTO 1
      END IF
      CALL GRQTYP(PLTYPE,INTER)
      IF (INTER) THEN
          TYPE = 3
          NOTEXT = .TRUE.
      ELSE
          TYPE = 1
          NOTEXT = (VALUES(231).GE.0.0)
      END IF
C
C Model file (optional).
C
      WRITE (PLOTMOD, '(10A8)') (VALUES(J),J=221,230)
      L = LEN1(PLOTMOD)
      MODEL = PLOTMOD.NE.' '
      IF (MODEL) THEN
          IER = VLBOPE(INMOD, PLOTMOD, 'MODEL', 'OLD', PLOTMOD)
          IF (IER.NE.1) GOTO 1
          CALL FRDT(PLOTMOD, REVTIM)
          WMODEL(1:17)  = REVTIM
          WMODEL(18:31) = ' Model file = '
          WMODEL(32:)   = PLOTMOD
          CALL SCRNAM('VISPLOT', SCRDSN, L)
          OPEN (UNIT=12, NAME=SCRDSN(:L),
     +     STATUS='SCRATCH', FORM='UNFORMATTED')
      END IF
C
C Other parameters.
C
      MAXWID  = VALUES(3)
      MINWID  = VALUES(4)
      NTICKS  = VALUES(5)
      HTPLOT  = VALUES(6)
      HTSCAL  = VALUES(7)
      HTTICK  = VALUES(8)
      HTLET   = VALUES(9)
      NPAGE   = VALUES(10)
      ASCALE  = VALUES(23)
      PSCALE  = VALUES(24)
      CSCALE  = VALUES(25)
      COLOR(1) = VALUES(31)
      COLOR(2) = VALUES(32)
      COLOR(3) = VALUES(33)
      NPLTMS(1) = VALUES(34)
      NPLTMS(2) = VALUES(35)
      PHTSCAL = VALUES(232)
      ONEPAGE  = (VALUES(233).GE.0.0)
      IF (VALUES(234).GE.0.0) THEN
          SETW = VALUES(39)
          SETWIDTH = .TRUE.
      ELSE
          SETWIDTH = .FALSE.
      END IF
      NOERR = (VALUES(235).NE.-1D0)
      PLOTDEL = (VALUES(236).NE.-1D0)
      SNR   = (VALUES(237).NE.-1D0)
C
C     UTMIN0 AND UTMAX0 ARE SET BELOW AFTER A CALL TO HEADX20
C
C       			DETERMINE WHAT TO DO
      DOAMP   = VALUES(11) .EQ. 0.D0
      DOPHASE = VALUES(12) .EQ. 0.D0
      DOCLOSE = VALUES(20) .EQ. 0.D0
      IWIDTH  = VALUES(21)
      IF (IWIDTH.LE.0)  IWIDTH = 1	! Set default line thickness
      IF (IWIDTH.GT.21) IWIDTH = 21
      DOCLAMP = VALUES(22) .EQ. 0.D0
      IF (.NOT.DOAMP .AND. .NOT.DOPHASE .AND. .NOT.DOCLOSE .AND.
     +           .NOT.DOCLAMP)  THEN
           DOAMP = .TRUE.
           DOCLOSE = .TRUE.
      END IF
      NOMOD24 = VALUES(26) .EQ. 0.D0
C
C                                DETERMINE TIME AXIS AND PLOT TYPE
      USELOOK = VALUES(15).EQ.-1.D0	! Optional suppression of lookup table
      IF(VALUES(16).EQ.0.D0) THEN
         TIMEA=HUT
         TIMEC=HUT
      ELSE IF(VALUES(17).EQ.0.D0) THEN
         TIMEA=HUTMOD
         TIMEC=HUTMOD
      ELSE IF(VALUES(18).EQ.0.D0) THEN
         TIMEA=HIHA
         TIMEC=HGST
      ELSE IF(VALUES(19).EQ.0.D0) THEN
         TIMEA=HGST
         TIMEC=HGST
      END IF
      IF (TIMEA.NE.HUT) NOMOD24 = .FALSE.  ! NOMOD24 only affects UT plots
C
C     NOTE IF TIMEA AND TIMEC NOT SET ABOVE - INPUT VALUES OR DEFAULT
C     WILL BE USED.
C
C Calculate lookup table if needed.
C
      IF (MODEL .AND. USELOOK) THEN
	  DO 99 I=1,4096
	     XI = DFLOTJ(I-1)*1.533980788D-3
   99	     LOOK(I) = DSIN(XI)
      END IF
C
      MODPTS = VALUES(13)
      IF(MODPTS.LE.10) MODPTS=100
      IF(MODPTS.GT.MAXMOD) MODPTS=MAXMOD
C
C Find data and read header.
C
      SKIPH = .FALSE.
      IPROUT = 0
      CALL HEADX20(SKIPH,INDAT,IZERO,IDAY,IYEAR,IDU,0,0,IUNITS,XDUM,
     +     IFILE,IPROUT)
      IF(IFILE.EQ.1) CALL ERROR('No header or data')
      IF (VALUES(28).NE.0.D0) THEN
        UTMIN0 = VALUES(27) + ((VALUES(28)-1.D0)*ICON-IZERO)/(60.*3600.)
      ELSE
        UTMIN0 = VALUES(27)
      END IF
      IF (VALUES(30).NE.0.D0) THEN
        UTMAX0 = VALUES(29) + ((VALUES(30)-1.D0)*ICON-IZERO)/(60.*3600.)
      ELSE
        UTMAX0   = VALUES(29)
      END IF
      IF ((TIMEA.EQ.HIHA).AND.ORBITERS) THEN
        WRITE(6,'(47H0 +++  Dataset includes orbiting VLBI stations.,/,
     1    57H       TIMEA = IHA is not valid.  Setting TIMEA = TIMEC =,
     2    A)') TIMEC
        TIMEA = TIMEC
      END IF
C
C                       Do not do closure phases if there are none.
      IF(NC.EQ.0) DOCLOSE = .FALSE.
C
C     DECODE BASELINE AND CLOSURE PHASE REQUESTS   
C
      NBP = 0
      DO 8 I = 1,MAXB
         IB = I + 35
         M = VALUES(IB) + 0.05
         IF(M.EQ.0) GO TO 10
         NBP = NBP + 1
         CALL BASERQ(M,MB1(I),MB2(I),FLAG)
         IF(FLAG.EQ.1.0) STOP
    8    CONTINUE
   10 CONTINUE
      IF (NBP.EQ.0 .AND. (DOAMP.OR.DOPHASE)) THEN
         NBP = NB
         DO 13 I = 1,NB
            MB1(I) = NB1(I)
   13       MB2(I) = NB2(I)
      END IF
C
      NCP = 0
      DO 11 I = 1,MAXT
         IC = I + 80
         M = VALUES(IC) + 0.05
         IF(M.EQ.0) GO TO 12
         NCP = NCP + 1
         IF(M.LT.1000) THEN
            MC1(I) = M/100
            MC2(I) = MOD(M,100)/10
            MC3(I) = MOD(M,10)
         ELSE IF(M.LT.10000) THEN
            MC1(I) = M/1000
            MC2(I) = MOD(M,1000)/100
            MC3(I) = MOD(M,100)
         ELSE
            MC1(I) = M/10000
            MC2(I) = MOD(M,10000)/100
            MC3(I) = MOD(M,100)
         END IF
         IF(MC3(I).LE.MC2(I) .OR. MC2(I).LE.MC1(I)) THEN
             CALL ERROR('Close specifications must have station'//
     1           ' numbers in ascending order.')
         END IF
   11    CONTINUE
   12 CONTINUE
      IF(NCP.EQ.0 .AND. DOCLOSE) THEN
         NCP = NC
         DO 14 I = 1,NC
            MC1(I) = NC1(I)
            MC2(I) = NC2(I)
   14       MC3(I) = NC3(I)
      END IF
C
C
C
C     CHECK HEADER DATA
C
      IF(NS.LE.MAXS.AND.NB.LE.MAXB.AND.NC.LE.MAXT) GO TO 15
         WRITE(6,1015) NS,NB,NC
 1015 FORMAT('0  Too much data.  Program is set up for no more than ',
     + '20 stations.',/,'0  Data records found for ',I5,' stations, ',
     + I5,' baselines and ',I5,' closure phases')
        CALL ERROR('Too many stations')
   15 CONTINUE
      DO 16 I = 1,NS
         IF(STX(I).EQ.0.D0.AND.STY(I).EQ.0.D0.AND.STATYP(I).LE.1) 
     1      GO TO 17
         IF(AEIOWM(1,I).EQ.0.D0.AND.STATYP(I).EQ.2) GO TO 17
   16 CONTINUE
      GO TO 18
   17 WRITE(6,1017) I,STNAME(I)
 1017 FORMAT(//'0  Coordinates for station ',I5,'(',A8,')',
     +' not provided -- set TIMEA = TIMEC = ''UT''',
     +    '  and don''t plot model'//)
      TIMEA = HUT
      TIMEC = HUT
      MODEL = .FALSE.
   18 CONTINUE
C
      IF(IUNITS.EQ.2 .OR. IUNITS.EQ.3) THEN
         IF(MODEL) WRITE (6,1018)
 1018    FORMAT('  Cannot model fringe rates or delays')
         MODEL = .FALSE.
      END IF
C
C
      IF(IUNITS.EQ.1) COEFF = .FALSE.
C
C     GET LIMITS
C
      CALL TIMING(TIMEA,TIMEC)
      CALL GETLIM(INDAT,IZERO,DOCLAMP,PLOTDEL,SNR)
      IDAY = IDAY + INT(UTMIN/24.0)
C     RETURN TO START OF DATA (GETLIM READS THE FILE)
      SKIPH = .TRUE.
      REWIND INDAT
      CALL HEADX20(SKIPH,INDAT,IDU,IDU,IDU,IDU,IDU,IDU,IDU,XDUM,IDU,IDU)
C
C     OBTAIN MONTH FOR HEADER  --  TJP SUBROUTINE
C
      IONE = 1
      CALL TDATE(IYEAR,IONE,IDAY,JD,MONTH,DAY)
C
C
C
C     INITIATE PLOT ROUTINES
C
      CALL GRSIZE(IDEVICE,XSZDEF,YSZDEF,XSZMAX,YSZMAX,XPERIN,YPERIN)
      YZSDEF = YSZDEF/NPAGE + 8.0*HTLET*YPERIN*(NPAGE-1)/NPAGE
      XINMAX = XSZDEF/XPERIN
      YINMAX = YZSDEF/YPERIN
      CALL GRTRAN(IDEVICE,0.0,0.0, XPERIN, YPERIN)
      CALL GRCHSZ(IDEVICE,CHARWD,CHARHT,CHARXS,CHARYS)
      HTLAT = HTLET*CHARWD/CHARHT
      CALL GRSETC(IDEVICE,HTLAT*XPERIN)
      CALL GRSCI(COLOR(1))	! Draw box and pips etc 
      CALL GRSLW(IWIDTH)	! Set line thickness (default=1)
C
C     CHECK SIZE PARAMETERS
C
      IF (MAXWID.GT.XINMAX-28.*HTLET) MAXWID = XINMAX-28.*HTLET
      IF (MINWID.GT.MAXWID*.7)  MINWID = MINWID*.7
      IF (SETWIDTH .AND. SETW .EQ. 0.0) SETW = MAXWID
      IF (SETW.GT.XINMAX-28.*HTLET) SETW = XINMAX-28.*HTLET
      IF (HTPLOT.GT.YINMAX-8.0*HTLET) HTPLOT = YINMAX-8.0*HTLET
      IF (PHTSCAL .LE. 0.0) THEN
        IF (HTSCAL.GT.HTPLOT.AND.NPAGE.LT.3) HTSCAL = HTPLOT - 0.5 
        IF (HTSCAL.GT.HTPLOT.AND.NPAGE.GE.3) HTSCAL = HTPLOT - 0.5/NPAGE
      ELSE
        IF (PHTSCAL.GE.1.0) PHTSCAL=0.95
         HTSCAL=HTPLOT * PHTSCAL
      END IF
C
C     SET PLOT BORDERS - BOTTOM
C
      NWORDS = NDWPB*NB + 2*NC
      BOT = 4.0 * HTLET
      YAD = BOT + HTPLOT/2.0
      XCENTER = XINMAX/2.0
      YCENTER = BOT + HTPLOT/2.0
C
C
C
C   =================================================================== 
C 
C                AMPLITUDES DATA 
C 
C   ===================================================================       
      IF(.NOT. DOAMP) GO TO 220 
C 
C 
      INCLUDE 'visplot1.inc' 
C 
C 
  220  CONTINUE 
C 
C 
C   ===================================================================
C
C                PHASE DATA 
C 
C   ===================================================================
      IF(.NOT. DOPHASE) GO TO 720 
C 
C 
      INCLUDE 'visplot2.inc'
C 
C 
  720  CONTINUE 
C 
C  ============================================================
C
C      CLOSURE PHASES
C
C  =============================================================
      IF(.NOT. DOCLOSE) GO TO 241
C
C
      INCLUDE 'visplot3.inc'
C
C
  241 CONTINUE
C
C  ============================================================
C
C      CLOSURE AMPLITUDES
C
C  =============================================================
      IF(.NOT.DOCLAMP) GO TO 541
C
C  
      INCLUDE 'visplot4.inc'
C
C Normal termination.
C
  541 CONTINUE
      CALL GRCLOS(IDEVICE)
      STOP
C-----------------------------------------------------------------------
      END

      BLOCK DATA
C
      PARAMETER  (MAXS=20)	         ! maximum number of stations
      PARAMETER  (MAXB=MAXS*(MAXS-1)/2)  ! maximum number of baselines
      PARAMETER  (MAXT=MAXB*(MAXS-2)/3)  ! maximum number of clos phase
      PARAMETER  (ISUM=MAXB+MAXT)        ! number for DATA statements
C
      COMMON /LIMS/  BTMAX,CTMAX,BTMIN,CTMIN,UTMAX,UTMIN,UTBMAX(MAXB),
     +               UTBMIN(MAXB),UTCMAX(MAXT),UTCMIN(MAXT),
     +               AMPMAX(MAXB),ATMAX,ATMIN
      COMMON /BLIMS/ BTBMAX(MAXB),BTBMIN(MAXB),CTCMAX(MAXT),
     *               CTCMIN(MAXT)
C
      DATA   ATMAX,BTMAX,CTMAX,UTMAX / 4*-9999. /
      DATA   UTBMAX,UTCMAX / ISUM * -9999./,
     +       AMPMAX /MAXB*0.0/,
     +       ATMIN,BTMIN,CTMIN,UTMIN / 4*9999. /
      DATA   UTBMIN,UTCMIN /ISUM*9999./
      DATA   BTBMAX,CTCMAX /ISUM*-9999./
      DATA   BTBMIN,CTCMIN /ISUM*9999./
      END

      SUBROUTINE ATICKS(RIGHT,LEFT,BASE,HTTICK,HTLET,HTPLOT,WR,
     +                  CAMIN,CAMAX,ID)
C
C      LABELS CLOSURE AMPLITUDE PLOTS
C
      REAL*4 LEFT,INCR
      REAL*4 RAT(7)
C
      INCR = (CAMAX-CAMIN)/6.0
      DO 5 I = 1,7
    5    RAT(I) = CAMIN + INCR*(I-1)
C
      DO 10 I = 1,7
         Y = BASE + (I-1)*HTPLOT/6.0
         CALL GRMOVA(RIGHT,Y)
   10    CALL GRLINA(RIGHT-HTTICK*WR,Y)
C
      DO 20 I = 1,7
         Y = BASE + (I-1)*HTPLOT/6.0
         CALL GRMOVA(LEFT,Y)
         CALL GRLINA(LEFT+HTTICK*WR,Y)
         X = LEFT - 5.0 * HTLET * WR
         CALL PRTNUM(X,Y-HTLET*.5,HTLET,RAT(I),'(F4.1)',0.,ID)
   20    CONTINUE
C
      RETURN
      END

      SUBROUTINE BORDER(RIGHT,LEFT,NPLOTS,HTPLOT,BOT,HTTICK,HTLET,WR,ID)
C
C     THIS SUBROUTINE PLOTS BORDERS AND TIME AXIS TICKS FOR NPLOTS PLOTS
C     THE LOCATION OF THE TICKS IS CONTROLED BY THE ARRAYS IN THE
C     COMMON BLOCK /TICKS/ GENERATED BY TSCALE
C
C     CALL TWICE - ONCE FOR BASELINES AND ONCE FOR CLOSURE PHASES
C
C     FOR VAX VERSION OF PLOT, WILL DO BORDER FOR ONE PLOT ONLY
      REAL*4 LEFT
      LOGICAL NOMOD24
      COMMON /TICKS/ WHERE(20),IHR(20),NTT,NOMOD24
C
C      DRAW SIDES
      CALL GRMOVA(LEFT,BOT)
      CALL GRLINA(LEFT,BOT+HTPLOT)
      CALL GRMOVA(RIGHT,BOT+HTPLOT)
      CALL GRLINA(RIGHT,BOT)
C
C
C     DRAW TIME AXIS AND TICKS
      N = 2
      DO 10 I = 1,N
         Y = (I-1)*HTPLOT + BOT
         CALL GRMOVA(RIGHT,Y)
         CALL GRLINA(LEFT,Y)
C
C        DRAW TICKS
         DO 8 J = 1,NTT
            HTT = HTTICK
            IF(MOD(IHR(J),3).EQ.0 .AND. NTT.GT.5) HTT = 2*HTT
            YU = Y + HTT
            IF(I.EQ.N) YU = Y
            YL = Y - HTT
            IF(I.EQ.1) YL = Y
            CALL GRMOVA(WHERE(J),YU)
            CALL GRLINA(WHERE(J),YL)
    8       CONTINUE
   10    CONTINUE
C
C     WRITE NUMBERS BELOW BOTTOM AXIS
C
      Y = BOT - 1.5*HTLET
      DO 20 I = 1,NTT
         IF(WHERE(2)-WHERE(1).LT.0.9 .AND. MOD(IHR(I),3).NE.0) GO TO 20
         X = WHERE(I) - 2.0*HTLET*WR
         IF(NPLOTS.EQ.0) GO TO 20
         INUM = IHR(I)
         XNUM = INUM
         IF (.NOT.NOMOD24) XNUM = MOD(INUM,24)
         IF(ABS(XNUM).GE.10.0) X = WHERE(I) - HTLET*1.6
         CALL PRTNUM(X,Y,HTLET,XNUM,'(I3)',0.,ID)
   20    CONTINUE
      RETURN
      END

      SUBROUTINE GETLIM(INDAT,IZERO,DOCLAMP,PLOTDEL,SNR)
C
C     READS DATA AND FINDS RANGES
C      
      PARAMETER (MAXS = 20)        !  STATIONS
      PARAMETER (MAXB = MAXS*(MAXS-1)/2)  !  BASELINES  
      PARAMETER (MAXT = MAXB*(MAXS-2)/3)  !  TRIANGLES
      PARAMETER (MAXWPB = 7)              !  WORDS PER BASELINE
      PARAMETER (MAXL = MAXWPB*MAXB+2*MAXT)     !  DATA RECORDS
C
C 
      REAL*4         AMP(MAXB),ERA(MAXB),PHASE(MAXB),ERP(MAXB),
     +               CLOSE(MAXT),ERC(MAXT),EBAR
      LOGICAL        DOCLAMP,PLOTDEL,SNR
      DATA           HRAD/3.819718634/,UTAMAX/-9999./,UTAMIN/9999./
      COMMON /BLIMS/ BTBMAX(MAXB),BTBMIN(MAXB),CTCMAX(MAXT),
     +               CTCMIN(MAXT)
      COMMON /LIMS/  BTMAX,CTMAX,BTMIN,CTMIN,UTMAX,UTMIN,UTBMAX(MAXB),
     +               UTBMIN(MAXB),UTCMAX(MAXT),UTCMIN(MAXT),
     +               AMPMAX(MAXB),ATMAX,ATMIN
      COMMON /ORDER/ NS,NB,NC,NB1(MAXB),NB2(MAXB),
     +               NC1(MAXT),NC2(MAXT),NC3(MAXT),NVER,NDWPB
      COMMON /FLAGS/ NBP,NCP,MB1(MAXB),MB2(MAXB),MC1(MAXT),
     +               MC2(MAXT),MC3(MAXT)
      COMMON /T/     TAKE(MAXL)
      COMMON /TIMSCAL/ UTMIN0,UTMAX0
      N = NDWPB*NB + 2*NC
      NPTS = 0
    1 CONTINUE
      READ(INDAT,END=1000) IT,(TAKE(I),I=1,N)
      IF(IT.EQ.-1) GO TO 1000
      UT = (IT-IZERO)/(60.*3600.)
      IF (UT.LT.UTMIN0) GO TO 1
      IF (UT.GT.UTMAX0) GO TO 1
      NPTS = NPTS + 1
C
C     USE ERROR ESTIMATES TO SENSE DATA.
C      BASELINES:
C
      IF(NBP.EQ.0) GO TO 25
      DO 20 I = 1,NB
         DO 10 M = 1,NBP
         IF(NB1(I).EQ.MB1(M) .AND. NB2(I).EQ.MB2(M))   THEN
         K = NDWPB*(I-1)
         AMP(I) = TAKE(K+1)
         ERA(I) = TAKE(K+2)
         PHASE(I) = TAKE(K+3)
         ERP(I) = TAKE(K+4)
         IF(ERA(I).EQ.0.0) GO TO 9
	 IF (SNR) THEN
	    EBAR = ABS(ERA(I))
	    AMP(I) = AMP(I)/EBAR
	    ERA(I) = ERA(I)/EBAR
	 END IF
         IF((.NOT.PLOTDEL).AND.(ERA(I).LT.0.0)) GO TO 9
            IF(UT.GT.UTBMAX(M)) UTBMAX(M) = UT
C>>>            XAMP=ABS(AMP(I)) + ABS(ERA(I))
            XAMP=ABS(AMP(I))
            IF(XAMP .GT. AMPMAX(M)) AMPMAX(M)=XAMP
            IF(UT.LT.UTBMIN(M)) UTBMIN(M) = UT
            CALL ATIME(UT,TF,I,1)
            IF(TF.GT.BTMAX) BTMAX = TF
            IF(TF.LT.BTMIN) BTMIN = TF
            IF(TF.GT.BTBMAX(M))  BTBMAX(M) = TF
            IF(TF.LT.BTBMIN(M))  BTBMIN(M) = TF
    9    CONTINUE
         GO TO 20
         END IF
   10    CONTINUE
   20    CONTINUE
C
C      CLOSURE PHASES
C
   25 IF(NCP.EQ.0 .OR. NC.EQ.0) GO TO 45
      DO 40 I = 1,NC
         DO 30 M = 1,NCP
         IF(NC1(I).EQ.MC1(M) .AND. NC2(I).EQ.MC2(M) .AND.
     +      NC3(I).EQ.MC3(M))   THEN
         K = NDWPB*NB + 2*(I-1)
         CLOSE(I) = TAKE(K+1)
         ERC(I) = TAKE(K+2)
         IF(ERC(I).EQ.0.0) GO TO 29
         IF((.NOT.PLOTDEL).AND.(ERC(I).LT.0.0)) GO TO 29
            IF(UT.GT.UTCMAX(M)) UTCMAX(M) = UT
            IF(UT.LT.UTCMIN(M)) UTCMIN(M) = UT
            CALL CCTIME(UT,CTF)
            IF(CTMAX.LT.CTF) CTMAX = CTF
            IF(CTMIN.GT.CTF) CTMIN = CTF
            IF(CTF.GT.CTCMAX(M))  CTCMAX(M) = CTF
            IF(CTF.LT.CTCMIN(M))  CTCMIN(M) = CTF
   29    CONTINUE
         GO TO 40
         END IF
   30    CONTINUE
   40    CONTINUE
   45 CONTINUE
C
C     CLOSURE AMPLITUDES
C
      IF(DOCLAMP) THEN
         NAMPS=0
         DO 55 I = 1,NB
            K=NDWPB*(I-1)
            AMP(I) = TAKE(K+1)
            ERA(I) = TAKE(K+2)
            IF (PLOTDEL) THEN
               IF(AMP(I).GT.0.0 .AND. ERA(I).NE.0.0 )  THEN
                  NAMPS=NAMPS+1
                  IF (UT.LT.UTBMIN(I)) UTBMIN(I) = UT
               END IF
            ELSE
               IF(AMP(I).GT.0.0 .AND. ERA(I).GT.0.0 )  THEN
                  NAMPS=NAMPS+1
                  IF (UT.LT.UTBMIN(I)) UTBMIN(I) = UT
               END IF
            END IF
   55       CONTINUE
         IF(NAMPS.GE.4) THEN
             IF(UT.GT.UTAMAX) UTAMAX = UT
             IF(UT.LT.UTAMIN) UTAMIN = UT
            CALL CCTIME(UT,CAT)
            IF(ATMAX.LT.CAT) ATMAX=CAT
            IF(ATMIN.GT.CAT) ATMIN=CAT
         END IF
       END IF
C
       GO TO 1
C
C      END OF INPUT
C
C
C
 1000 CONTINUE
      IF(NPTS.EQ.0) THEN
         CALL ERROR('0  No data')
      END IF
C
      DO 60 I = 1,NBP
            IF(UTBMAX(I).GT.UTMAX) UTMAX = UTBMAX(I)
            IF(UTBMIN(I).LT.UTMIN) UTMIN = UTBMIN(I)
   60    CONTINUE
C
C
      CMAX = -1.E20
      CMIN = 1.E20
      DO 80 I = 1,NCP
            IF(UTCMAX(I).GT.CMAX) CMAX = UTCMAX(I)
            IF(UTCMIN(I).LT.CMIN) CMIN = UTCMIN(I)
   80    CONTINUE
      IF(CMAX.GT.UTMAX) UTMAX = CMAX
      IF(CMIN.LT.UTMIN) UTMIN = CMIN
      IF(UTAMAX.GT.UTMAX) UTMAX = UTAMAX
      IF(UTAMIN.LT.UTMIN) UTMIN = UTAMIN
C
C
      RETURN
      END

      SUBROUTINE CTICKS(RIGHT,LEFT,BASE,HTTICK,HTLET,HTPLOT,
     +                    WR,SCALE,ID)
C
C      LABELS CLOSURE PHASE PLOTS
C
      REAL*4 LEFT
      REAL*4 DEG(5)
C
      DO 10 I = 1,5
         IF(SCALE.NE.0.0) THEN
            DEG(I) = -1.0*SCALE + (I-1)*SCALE/2.0
         ELSE
            DEG(I) = -180. + (I-1)*90.
         END IF
         Y = BASE + (I-1)*HTPLOT/4.0
         CALL GRMOVA(RIGHT,Y)
   10    CALL GRLINA(RIGHT-HTTICK*WR,Y)
C
      DO 20 I = 1,5
         Y = BASE + (I-1)*HTPLOT/4.0
         CALL GRMOVA(LEFT,Y)
         CALL GRLINA(LEFT+HTTICK*WR,Y)
         X = LEFT - 5.0 * HTLET * WR
         CALL PRTNUM(X,Y-HTLET*.5,HTLET,DEG(I),'(I4)',0.,ID)
   20    CONTINUE
C
      RETURN
      END

      SUBROUTINE FTICKS(RIGHT,LEFT,BASE,HTLET,HTTICK,HTSCAL,
     +              NTICKS,WR,ID)
C
C     THIS ROUTINE DRAWS AND LABELS TICKS ON THE INTENSITY AXIS OF
C     OF PLOTS OF BASELINE DATA
C
      CHARACTER*6 FMT(5),FMTT,FMINT
      DATA FMT /'(F6.1)','(F6.2)','(F6.3)','(F6.4)','(F6.5)'/
      REAL*4 LEFT
      DATA FMINT/'(I6)'/
      COMMON  /VTICKS/ VALS(15),NDIG
      ISIGN = HTSCAL/ABS(HTSCAL)
C
      DO 10 I = 0,NTICKS
         Y = BASE + I*HTSCAL/NTICKS
         CALL GRMOVA(RIGHT,Y)
         CALL GRLINA(RIGHT-HTTICK*WR,Y)
   10    CONTINUE
C
C     ESTABLISH FORMAT
      IF(NDIG.GT.5.OR.NDIG.LT.-4) THEN
         CALL ERROR('Number of digits for flux out of range')
      END IF
      IF(NDIG.GT.0) FMTT = FMT(NDIG)
      IF(NDIG.LE.0) FMTT = FMINT
C
      X = LEFT - 7.0*HTLET*WR
      Y = BASE 
      CALL GRMOVA(LEFT+HTTICK*WR,Y)
      CALL GRLINA(LEFT,Y)
      Y = Y - HTLET/2.0
      CALL PRTNUM(X,Y,HTLET,0.0,FMTT,0.,ID)
      DO 20 I = 1,NTICKS
         Y = BASE + I*HTSCAL/NTICKS
         CALL GRMOVA(LEFT+HTTICK*WR,Y)
         CALL GRLINA(LEFT,Y)
         Y = Y - HTLET/2.0
         CALL PRTNUM(X,Y,HTLET,ISIGN*VALS(I),FMTT,0.,ID)
   20    CONTINUE
      RETURN
      END

      SUBROUTINE HLAB(X,Y,HTLET,SOURCE,MONTH,IDAY,IYEAR,FLUX,FREQ,ID)
C
C     PUTS HEADER ON PLOTS
C
      CHARACTER  C1*40,C2*40
      REAL*8 SOURCE(2),FLUX,FREQ

      IF (FREQ.GT.1E9) THEN
          WRITE(C1,'(2A8,5X,F6.2,1X,3HGHz)') SOURCE,FREQ/1E9
      ELSE
          WRITE(C1,'(2A8,5X,F5.1,1X,3HMHz)') SOURCE,FREQ/1E6
      END IF
      IF (FLUX.LE.0.0) THEN
         WRITE(C2,'(A3,1X,I2,2H, ,I4)') MONTH,IDAY,IYEAR
      ELSE IF (FLUX.LT.10.0) THEN
         WRITE(C2,102) MONTH,IDAY,IYEAR,FLUX
      ELSE
         WRITE(C2,101) MONTH,IDAY,IYEAR,FLUX
      END IF
  101 FORMAT(A3,1X,I2,2H, ,I4,6X,'TOTAL FLUX = ',F5.1,' Jy')
  102 FORMAT(A3,1X,I2,2H, ,I4,6X,'TOTAL FLUX = ',F5.2,' Jy')
      CALL GRCHAR(ID,.FALSE.,0.0,.FALSE.,X,Y+3*HTLET,C1)
      CALL GRCHAR(ID,.FALSE.,0.0,.FALSE.,X,Y+.5*HTLET,C2)
      RETURN
      END

      SUBROUTINE MODCAL(INMOD,IST1,IST2,IST3,MAX24,MODPTS)
C
C     THIS ROUTINE READS MODEL CARDS AND CALCULATES AMPLITUDES AND
C     CLOSURE PHASES FOR A RANGE OF UT.  MODEL CALCULATED FOR MODPTS
C     POINTS.  MODEL PARAMETERS ARE READ ON UNIT 'MOD'.  WILL STOP AT
C     END OF DATA SET OR AT A BLANK CARD
C
C	Modified to use subroutine PHSAMP instead of MODAP - use
C	lookup table for sin/cos. Replace /MODPAR/ with /MODEL/; add
C	/LOOKUP/ (lookup switch and table).   S.C. Unwin  1982 June 10
C	Fix bug in type for CTYPE (now I*4) - also in PHSAMP.
C	Increase NCP to 10000.                S.C. Unwin  1982 Sept 18
C
      PARAMETER  (MS=20, MB=MS*(MS-1)/2, MC=MS*(MS-1)*(MS-2)/6)
      PARAMETER  (MAXMOD=500)
      PARAMETER  (NTI=MAXMOD,NCP=10000)
C
      LOGICAL  MAX24,FIRST,USELOOK
      INTEGER*4 ID1(MB),ID2(MB),STATYP
      REAL*4  BXY(3),BZ(3),ROT(2,3)
      REAL*4  ADSK(NTI),PDSK(NTI)
      REAL*8 STNAME,STX,STY,STZ,AEIOWM,ORB(6,2),RHST(6,2)
      REAL*8 GASTM,FREQ,TINTG,AVG,BW,DTSEC
      REAL*8 SOURCE(2),RAP,DECP,FLUX
      LOGICAL ORBITERS
      COMMON /ORDER/NS,NB,NC,NB1(MB),NB2(MB),NC1(MC),NC2(MC),NC3(MC),
     1    NVER,NDWPB
      COMMON /LIMS/  BTMAX,CTMAX,BTMIN,CTMIN,UTMAX,UTMIN,UTBMAX(MB),
     +               UTBMIN(MB),UTCMAX(MC),UTCMIN(MC),AMPMAX(MB),
     +               ATMAX,ATMIN
      COMMON /DATA/  AMP(NTI,3),PHASE(NTI,3),CLOS(NTI),T(NTI),IT(NTI)
      COMMON /PARMS/ STNAME(MS),STX(MS),STY(MS),STZ(MS),SOURCE,RAP,
     +               DECP,GASTM,FREQ,FLUX,TINTG,AVG,BW,AEIOWM(6,MS),
     +               STATYP(MS),ORBITERS
C
      REAL*4 AMOD(NCP),XMOD(NCP),YMOD(NCP),AXMOD(NCP),RATMOD(NCP),
     1       PHMOD(NCP),LOOK(4096)
      INTEGER CTYPE(NCP)
      COMMON /MODEL/ NMOD,AMOD,XMOD,YMOD,AXMOD,RATMOD,PHMOD,CTYPE
      COMMON /LOOKUP/ USELOOK,LOOK
C
      INTEGER*4  NBA(3),NBB(3),NBL(3)
      DATA  HIHA/'IHA'/,HGST/'GST'/
      DATA NDSK /0/, FIRST/.TRUE./
      NBAS = 1
      NBA(1) = IST1
      NBB(1) = IST2
      IF(IST3.NE.0) THEN
         NBAS = 3
         NBA(2) = IST2
         NBA(3) = IST1
         NBB(2) = IST3
         NBB(3) = IST3
      END IF
      DO 2 I = 1,NB
         DO 1 J = 1,NBAS
            IF(NB1(I).EQ.NBA(J).AND.NB2(I).EQ.NBB(J)) NBL(J) = I
    1       CONTINUE
    2    CONTINUE
      RADMS = 6.283185/(3600.*1000.*360.)
      C = 2.997925E8
      WAVE = C / (FREQ)
      HRAD = 12./3.1415926
      DRAD = 180./3.1415926
      COSRA  = DCOS(RAP)
      SINRA  = DSIN(RAP)
      COSDEC = DCOS(DECP)
      SINDEC = DSIN(DECP)
      ROT(1,1) = -SINRA
      ROT(1,2) = COSRA
      ROT(2,1) = -SINDEC*COSRA
      ROT(2,2) = -SINDEC*SINRA
      ROT(2,3) = COSDEC
C
C     PREPARE TIMING ROUTINE TO DELIVER IHA'S
      CALL TIMING(HIHA ,HGST )
C
C     GET BASELINE PARAMETERS
      DO 10 I = 1,NBAS
         IS1 = NBB(I)
         IS2 = NBA(I)
         IF ((STATYP(IS1).EQ.0) .AND. (STATYP(IS2).EQ.0)) THEN
            BX = (STX(IS1) - STX(IS2)) / WAVE
            BY = (STY(IS1) - STY(IS2)) / WAVE
            BZ(I) = (STZ(IS1) - STZ(IS2)) / WAVE
            BXY(I) = SQRT(BX*BX + BY*BY)
         ELSE
            BZ(I) = 0.
            BXY(I) = 0.
         END IF
   10 CONTINUE
C
C     SET TIME INCREMENT
      TOTIME = UTMAX - UTMIN
      IF(MAX24.AND.TOTIME.GT.24.0) TOTIME = 24.2
      TINCR = TOTIME / (MODPTS-1)
      DO 15 I = 1,MODPTS
         UT = UTMIN + (I-1)*TINCR
   15    T(I) = UT
C
C     SKIP NEXT PORTION IF NOT FIRST CALL TO SUBROUTINE
      IF(.NOT.FIRST) GO TO 31
      FIRST = .FALSE.
C
C     READ MODEL
C
      I = 1
      REWIND INMOD
   20 CONTINUE
C
      CALL GTMODL(INMOD,CTYPE(I),AMOD(I),R,TH,AXMOD(I),RATMOD(I),
     +            PHMOD(I), *30, 0)
      IF(AMOD(I).EQ.0.0) GO TO 30
      THE = TH
      TH = TH/DRAD
      X = R*SIN(TH)
      Y = R * COS(TH)
      XMOD(I) = X * RADMS*4096.0	! unit = radians*4096
      YMOD(I) = Y * RADMS*4096.0	! for PHSAMP
      IF( (ABS(AXMOD(I)) .GT. 0.001) .OR.
     1	 (CTYPE(I) .GE. 1) ) USELOOK = .FALSE. ! No lookup for Gaussians etc.
      AXMOD(I)=AXMOD(I)*RADMS
      PHMOD(I)=PHMOD(I)/DRAD		       ! PHSAMP uses P.A. in radians
      I = I + 1
      IF(I.LE.NCP) GO TO 20
C
      WRITE(6,1001) NCP,NCP 
 1001 FORMAT('0',I8,' or more model points provided, only',I8,' used')
C
C
   30 CONTINUE
      NMOD = I - 1
   31 CONTINUE
      DO 90 J = 1,NBAS
         IS1 = NBB(J)
         IS2 = NBA(J)
         IF(NDSK.EQ.0) GO TO 85
            DO 82 NN = 1,NDSK
               IF(NBA(J).EQ.ID1(NN).AND.NBB(J).EQ.ID2(NN)) THEN
                   REWIND 12
                   DO 80 NQ = 1,NN
   80                READ(12,END=81) ADSK,PDSK
                   GO TO 92
   81              CALL ERROR(
     1               'Something wrong with temporary model disk')
               END IF
   82       CONTINUE
   85    CONTINUE
         DO K=1,6
            ORB(K,1) = AEIOWM(K,IS1)
            ORB(K,2) = AEIOWM(K,IS2)
         END DO
         DTSEC = T(1)*3600.D0	! Propagate orbits to beginning of data
         DO 100 I=1,MODPTS
            UT = T(I)
            IF (I.GT.1) DTSEC = (T(I)-T(I-1))*3600.D0
            IF ((STATYP(IS1).EQ.0) .AND. (STATYP(IS2).EQ.0)) THEN
               CALL ATIME(UT,TIHA,NBL(J),0)
               U = -BXY(J) * COS(TIHA/HRAD)
               V = BZ(J) * COSDEC - BXY(J)*SINDEC*SIN(TIHA/HRAD)
            ELSE
               CALL CCTIME(UT,GST)
               GSTRAD = GST/HRAD
               CALL POSITN(RHST(1,1),STX(IS1),STY(IS1),STZ(IS1),
     1            ORB(1,1),STATYP(IS1),GSTRAD,DTSEC)
               CALL POSITN(RHST(1,2),STX(IS2),STY(IS2),STZ(IS2),
     1            ORB(1,2),STATYP(IS2),GSTRAD,DTSEC)
               BXIB = (RHST(1,1) - RHST(1,2)) / WAVE
               BYIB = (RHST(2,1) - RHST(2,2)) / WAVE
               BZIB = (RHST(3,1) - RHST(3,2)) / WAVE
               U = ROT(1,1)*BXIB + ROT(1,2)*BYIB
               V = ROT(2,1)*BXIB + ROT(2,2)*BYIB + ROT(2,3)*BZIB
            END IF
            CALL PHSAMP(U,V,A,PH)	! Call to model calculation
            ADSK(I) = A
            PDSK(I) = -PH	! PHSAMP has opposite sign convention!!
  100    CONTINUE
C        WRITE NEW DISK RECORD 
         REWIND 12
         IF(NDSK.NE.0) THEN
            DO 91 NN = 1,NDSK
   91          READ(12) DUMMY
         END IF
         WRITE(12) ADSK,PDSK
         NDSK = NDSK + 1
         ID1(NDSK) = NBA(J)
         ID2(NDSK) = NBB(J)
C
   92    DO 93 I = 1,MODPTS
            AMP(I,J) = ADSK(I)
   93       PHASE(I,J) = PDSK(I)
   90 CONTINUE
C
C     CALCULATE CLOSURE PHASES
      IF(NBAS.EQ.1) GO TO 110
      DO 105 I = 1,MODPTS
        CLOS(I) = PHASE(I,1) + PHASE(I,2) - PHASE(I,3)
  105   CLOS(I) = PINCH(CLOS(I))
  110 RETURN
C
C
      END

      FUNCTION PINCH(Y)
C
C     CONVERTS ANGLE X (DEGREES) TO THE RANGE   -180 < X < 180
C
      X = Y
      X = AMOD(X,360.0)
      IF(X.GT.180.) X = X - 360.
      IF(X.LT.-180.) X = X + 360.
      PINCH = X
      RETURN
      END

	SUBROUTINE SYSSYM (X, Y, SIZE, STRING, N, ORIENT, ID)
C
C  This is supposed to be a VAX equivalent to IBM routine SYSSYM.
C
	CHARACTER*(*) STRING
	CALL GRSETC (ID,SIZE)
	CALL GRCHAR (ID,.FALSE.,ORIENT,.FALSE.,X,Y,STRING)
C	CALL GRSETC (ID,0.)
	RETURN
	END

      SUBROUTINE TIMING(TIME1,TIME2)
C
C     CONVERTS UT TO VALUE USED FOR PLOTTING
C     SET UP FOR UT,IHA,AND GST AND UTM
C
      PARAMETER (MAXS=20)                !  STATIONS
      PARAMETER (MAXB=MAXS*(MAXS-1)/2)   ! BASELINES
      PARAMETER (MAXT=MAXB*(MAXS-2)/3)   ! maximum number of clos phase
C
C
      REAL*4 ADD(MAXB),MULT(MAXB),PLUS(MAXB)
      DATA HIHA/'IHA'/,HGST/'GST'/,HUT/'UT'/,HUTMOD/'UTM'/
      INTEGER STATYP(MAXS)
      REAL*8 GASTM,FREQ,TINTG,AVG,BW
      REAL*8 STNAME(MAXS),STX(MAXS),STY(MAXS),STZ(MAXS),AEIOWM(6,MAXS)
      REAL*8 SOURCE(2),RAP,DECP,FLUX
      LOGICAL ORBITERS
      COMMON /PARMS/ STNAME,STX,STY,STZ,SOURCE,RAP,DECP,
     +               GASTM,FREQ,FLUX,TINTG,AVG,BW,AEIOWM,STATYP,
     +               ORBITERS
      COMMON /ORDER/ NS,NB,NC,NB1(MAXB),NB2(MAXB),
     +               NC1(MAXT),NC2(MAXT),NC3(MAXT),NVER,NDWPB
      DATA SIDR/1.00273791/,HRAD/3.819718634/
C
C     FIND MEAN LONGITUDE OF GROUND STATIONS TO DETERMINE WHERE TO
C       BREAK PLOTS WITH GST OR UTM TIME AXIS
C
      TIMEA = TIME1
      TIMEC = TIME2
      IF(TIMEA.NE.HGST.AND.TIMEC.NE.HGST.AND.TIMEA.NE.HUTMOD.AND.
     +   TIMEC.NE.HUTMOD) GO TO 5
         XMEAN = 0.0
         YMEAN = 0.0
         NGS = 0
         DO 38 I = 1,NS
            IF (STATYP(I).LE.1) THEN
               VL = DSQRT(STX(I)*STX(I) + STY(I)*STY(I))
               XMEAN = XMEAN + STX(I)/VL
               YMEAN = YMEAN + STY(I)/VL
               NGS = NGS + 1
           END IF
   38       CONTINUE
         IF (NGS.GT.0) THEN
            XMEAN = XMEAN/NGS
            YMEAN = YMEAN/NGS
            HAMEAN = ATAN2(YMEAN,XMEAN)
         ELSE
            HAMEAN = 0.
         END IF
         GSTB = (RAP  + HAMEAN) * HRAD + 12.0
         GSTB = AMOD(GSTB,24.0)
         IF(GSTB.LT.0.0) GSTB = GSTB + 24.0
         UTB = (GSTB - GASTM*HRAD) / SIDR
         UTB = AMOD(UTB,24.0)
         IF(UTB.LT.0.0) UTB = UTB + 24.0
    5 CONTINUE
C
C     SET UP VALUES USED TO CALCULATE TIMES
C
      DO 1 I  = 1,NB
    1    PLUS(I) = 0.0
C
      IF(TIMEA.EQ.HIHA) GO TO 20
      IF(TIMEA.EQ.HGST) GO TO 30
      IF(TIMEA.EQ.HUT.OR.TIMEA.EQ.HUTMOD) GO TO 40
      CALL ERROR('Requested time axis units not understood')
C
   20 CONTINUE
      DO 28 I = 1,NB
         IA = NB1(I)
         IB = NB2(I)
         IF ((STATYP(IA).LE.1) .AND. (STATYP(IB).LE.1)) THEN
            BX = STX(IB) - STX(IA)
            BY = STY(IB) - STY(IA)
            BHA = ATAN2(BY,BX)
C
            STAHA = DATAN2(STY(IA),STX(IA))
            STBHA = DATAN2(STY(IB),STX(IB))
            PLUS(I) = 0.0
C           CHANGE IHA BY 12 HR FOR BACKWARDS BASELINES
            DHA = (STBHA - STAHA) * HRAD
            IF((DHA.GT.-12.0.AND.DHA.LT.0.0).OR.DHA.GT.12.0)
     1         PLUS(I) = 12.0
C
            ADD(I) = (GASTM - RAP - BHA)*HRAD + 6.0
            MULT(I) = SIDR
         ELSE
            PLUS(I) = 0.0
            ADD(I) = 0.0
            MULT(I) = 0.0
         END IF
   28    CONTINUE
      GO TO 100
C
   30 DO 32 I = 1,NB
         ADD(I) = GASTM * HRAD
   32    MULT(I) = SIDR
      GO TO 100
C
   40 DO 42 I = 1,NB
         ADD(I) = 0.0
   42    MULT(I) = 1.0
      GO TO 100
C
 
C
C
C     ENTRY FOR AMPLITUDE PLOTS
C
      ENTRY ATIME(UT,T,IBAS,ITW)
C         ITW = 1 TO ADD PLUS(IBAS)  ---   0 OTHERWISE
      T = UT*MULT(IBAS) + ADD(IBAS)  + ITW*PLUS(IBAS)
      IF(TIMEA.EQ.HUT) RETURN
      IF(TIMEA.EQ.HGST.OR.TIMEA.EQ.HUTMOD) GO TO 50
      T = AMOD(T,24.0)
      IF(T.GT.12.0) T = T - 24.0
      IF(T.LT.-12.0) T = T + 24.0
      RETURN
   50 T = AMOD(T,24.0)
      IF(T.LT.0.0) T = T + 24.0
      IF(T.GT.GSTB.AND.TIMEA.EQ.HGST) T = T - 24.0
      IF(T.GT.UTB.AND.TIMEA.EQ.HUTMOD) T = T - 24.0
      RETURN
C
C
C      CLOSURE PHASE SECTION
C
  100 CONTINUE
      IF(TIMEC.EQ.HGST) GO TO 130
      IF(TIMEC.EQ.HUT.OR.TIMEC.EQ.HUTMOD) GO TO 140
      CALL ERROR('Requested time axis TIMEC not recognized')
C
  130 CADD = GASTM * HRAD
      CMULT = SIDR
      RETURN
C
  140 CADD = 0.0
      CMULT = 1.0
      RETURN
C
C
C
      ENTRY CCTIME(CUT,CT)
      CT = CUT*CMULT + CADD
      IF(TIMEC.NE.HGST.AND.TIMEC.NE.HUTMOD) RETURN
      CT = AMOD(CT,24.0)
      IF(CT.LT.0.0) CT = CT + 24.0
      IF(CT.GT.GSTB.AND.TIMEC.EQ.HGST) CT = CT - 24.0
      IF(CT.GT.UTB.AND.TIMEC.EQ.HUTMOD) CT = CT - 24.0
      RETURN
C
C
C
      END

      SUBROUTINE TSCALE(TMIN,TMAX,INCHHR,LEFT,RIGHT,MAXWID,MINWID,
     +      TCENT,TIME,XCENTER)
C
C     THIS ROUTINE ESTABLISHES TIME AXIS SCALING. 
C
      REAL*4 MAXWID,MINWID
      REAL*4 INCHHR,LEFT,RIGHT
      LOGICAL NOMOD24
      COMMON /TICKS/ WHERE(20),IHR(20),NTT,NOMOD24
      DATA  HIHA/'IHA'/
      TRANGE = TMAX - TMIN
      INCHHR = 2.0
C    
C     CHECK IF THERE IS SO LITTLE DATA THAT THE WIDTH WILL BE SET
C     BY MINWID
C  
      IF (TRANGE*INCHHR .GT. MINWID-1.0) GO TO 5
         WIDTH = MINWID
         GO TO 50
C
C     FOR MORE DATA - SET WIDTH AND INCHHR
C
    5 INCHHR = INCHHR/2.0
      WIDTH = TRANGE*INCHHR + 1.0
      IF (WIDTH .GT. MAXWID)  GO TO 5
C
C     SET UP BOUNDARIES ETC.
C
   50 LEFT = XCENTER - WIDTH/2.0
      RIGHT = XCENTER + WIDTH/2.0
      TICINC = 1.0
      IF (INCHHR .GT. 1.0) TICINC = INCHHR
      TCENT = TMIN + TRANGE/2.0
      TLEFT = TCENT - WIDTH/(2.*INCHHR)
      ILEFT = TLEFT * INCHHR / TICINC
      IF (TLEFT .GT. 0.0) ILEFT = ILEFT + 1
      ITLEFT = ILEFT * TICINC / INCHHR
C
C     SET UP TICKS
C
      WHERE(1) = XCENTER - (TCENT-ITLEFT)*INCHHR
      HR = TCENT + (WHERE(1)-XCENTER)/INCHHR
      IHR(1) = HR + 0.1
      IF(HR.LT.0.0) IHR(1) = HR - 0.2
      I = 1
  100    I = I+1
         WHERE(I) = WHERE(1) + (I-1)*TICINC
         IF(WHERE(I) .GT. RIGHT) GO TO 104
         NTT = I
         IHR(I) = IHR(1) + (I-1)*TICINC/INCHHR
         GO TO 100
  104 CONTINUE
C
C     FOR ALL EXCEPT IHA, TAKE MODULO 24 HR FOR PRINT
C
      IF ((TIME.NE.HIHA) .AND. .NOT.NOMOD24) THEN
         DO 108 I = 1,NTT
         IHR(I) = MOD(IHR(I),24)
         IF(IHR(I).LT.0) IHR(I) = IHR(I) + 24
  108    CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE BASERQ(IRQ,IS1,IS2,FLAG)
C
C     THIS ROUTINE DECODES BASELINE REQUESTS
C
      FLAG=0
      IF(IRQ.LT.100) THEN
         IS1 = IRQ/10
         IS2 = MOD(IRQ,10)
      ELSE
         IS1 = IRQ/100
         IS2 = MOD(IRQ,100)
      END IF
      IF(IS1.LT.IS2) RETURN
         FLAG=1
         WRITE(6,1) 
    1    FORMAT(' Baseline specification must have station numbers in',
     +  ' ascending order')
   
      RETURN
      END

      SUBROUTINE NXTRAT(IB11,IB12,IB21,IB22,IB31,IB32,IB41,IB42,N,LAST)
C
C     THIS ROUTINE DETERMINES THE STATIONS INVOLVED IN THE BASELINES 
C     OF A COMPLETE SERIES OF AMPLITUDE RATIOS.
C
      DATA  NR/0/,I1/1/,I2/2/,I3/3/,I4/3/
      LAST = 0
      NR = NR + 1
      IF(NR.EQ.4) NR = 1
      GO TO (10,20,30) NR
C
   10 I4 = I4 + 1
      IF(I4.NE.N+1) GO TO 12
         I3 = I3 + 1
         I4 = I3 + 1
         IF(I3.NE.N) GO TO 12
            I2 = I2 + 1
            I3 = I2 + 1
            I4 = I3 + 1
            IF(I2.NE.N-1) GO TO 12
               I1 = I1 + 1
               I2 = I1 + 1
               I3 = I2 + 1
               I4 = I3 + 1
               IF(I1.GE.N-2) THEN
                  LAST = 1
                  RETURN
                 END IF
   12 IB11 = I1
      IB12 = I2
      IB21 = I3
      IB22 = I4
      IB31 = I1
      IB32 = I3
      IB41 = I2
      IB42 = I4
      GO TO 100
C
   20 IB11 = I1
      IB12 = I2
      IB21 = I3
      IB22 = I4
      IB31 = I1
      IB32 = I4
      IB41 = I2
      IB42 = I3
      GO TO 100
C
   30 IB11 = I1
      IB12 = I3
      IB21 = I2
      IB22 = I4
      IB31 = I1
      IB32 = I4
      IB41 = I2
      IB44 = I3
C
  100 CONTINUE
      RETURN
      END

      SUBROUTINE REQRAT(NA1,NA2,NA3,NA4,FLAG,
     +                  IB11,IB12,IB21,IB22,IB31,IB32,IB41,IB42)
C
C     DECODES AND CHECKS CLOSURE AMPLITUDE RATIO REQUEST
C
      FLAG = 0
      CALL BASERQ(NA1,IB11,IB12,FLAG1)
      CALL BASERQ(NA2,IB21,IB22,FLAG2)
      CALL BASERQ(NA3,IB31,IB32,FLAG3)
      CALL BASERQ(NA4,IB41,IB42,FLAG4)
      IF(FLAG1.EQ.1. .OR. FLAG2.EQ.1. .OR. FLAG3.EQ.1. .OR. FLAG4.EQ.1.)
     +     THEN
         FLAG=1.0
         RETURN
        END IF
C
C     CHECK VALIDITY
C     
      IF(IB11.EQ.IB21 .OR. IB11.EQ.IB22 .OR. IB12.EQ.IB21 .OR. 
     +   IB12.EQ.IB22)  GO TO 100
      IF(IB31.EQ.IB41 .OR. IB31.EQ.IB42 .OR. IB32.EQ.IB41 .OR.
     +   IB32.EQ.IB42)  GO TO 100
C
      IF(IB11.NE.IB31 .AND. IB11.NE.IB32 .AND. IB11.NE.IB41 .AND.
     +   IB11.NE.IB42)  GO TO 100
      IF(IB12.NE.IB31 .AND. IB12.NE.IB32 .AND. IB12.NE.IB41 .AND.
     +   IB12.NE.IB42)  GO TO 100
      IF(IB21.NE.IB31 .AND. IB21.NE.IB32 .AND. IB21.NE.IB41 .AND.
     +   IB21.NE.IB42)  GO TO 100
      IF(IB22.NE.IB31 .AND. IB22.NE.IB32 .AND. IB22.NE.IB41 .AND.
     +   IB22.NE.IB42)  GO TO 100
C
      RETURN
C
C
  100 CONTINUE
      WRITE(6,110)
  110 FORMAT(' Improper set of baselines')
      FLAG = 1
      RETURN
C
      END

      SUBROUTINE SCALE(YMAX,HTSCAL,NTICKS,INCHJY)
C
C     THIS ROUTINE IS FOR DETERMINATION OF THE VERTICAL SCALE FOR
C     PLOTS.  IT ASSUMES DATA FROM 0 TO VMAX IS TO BE PLOTTED AND
C     FINDS A SCALE FROM 0 TO PMAX SUCH THAT 'NTICKS' TICKS MAY BE
C     USED WTIH EACH TICK LABELED WITH A NUMBER WITH TWO NON-ZERO
C     DIGITS.  NDIG GIVES THE NUMBER OF DIGITS RIGHT OF THE DECIMAL
C     POINT THAT ARE REQUIRED TO PRINT THE SCALE (IF NEGATIVE, IT IS
C     THE NUMBER OF EXTRA ZEROS REQUIRED BEFORE THE DECIMAL)
C
      REAL*4 INCHJY
      COMMON  /VTICKS/ VALS(15),NDIG
C     NTICKS SHOULD BE A SMALL NUMBER - EG LESS THAN 10 ALTHOUGH IT IS
C     NOT RIGIDLY RESTRICTED
      INTEGER*4 IUNIT(15)
      DATA IUNIT /1,2,3,5,8,10,15,20,25,30,40,50,60,75,100/
C
      VMAX = YMAX
      IF(VMAX.EQ.0.0) VMAX = 1.0
C
C     FIND NDIG
      ALOG = ALOG10(VMAX)  
      DIG = 2 - ALOG
      IF(DIG.LT.0) DIG = DIG - 1
      NDIG = DIG
      IF(NDIG.LT.-4) NDIG=-4
      IF(NDIG.GT.5) NDIG=5
C
C     CREATE A NUMBER  10<IVM<100 FROM VMAX
C
      IVM = VMAX * 10.0**NDIG
      DO 10 I = 1,15
         IUN = IUNIT(I)
         IF(IVM.LT.NTICKS*IUN) GO TO 12
   10    CONTINUE
C
   12 CONTINUE
      DIV = 10.0**NDIG
      DO 20 I = 1,NTICKS
   20    VALS(I) = IUN * I / DIV
      PMAX = VALS(NTICKS)
      INCHJY = HTSCAL/PMAX
      RETURN
      END

      SUBROUTINE VSTR(STRING,N,NAM1,NAM2,NAM3,NAM4)
C
C     Set up the station labels for VISPLOT
C
      CHARACTER*60 STRING,RESET
      DATA  RESET / ' ' /
      REAL*8  NAM1,NAM2,NAM3,NAM4
      INTEGER*4 N,ICHAR
C
      STRING = RESET
C
      IF(N.EQ.2.OR.N.EQ.3) THEN
          WRITE(STRING,'(A8)') NAM1      
          ICHAR = LEN1(STRING) + 1
          WRITE(STRING(ICHAR:),'(''-'')')
          ICHAR = ICHAR+1
          WRITE(STRING(ICHAR:),'(A8)') NAM2
          IF(N.EQ.3) THEN
             ICHAR = LEN1(STRING) + 1
             WRITE(STRING(ICHAR:),'(''-'')')
             ICHAR = ICHAR+1
             WRITE(STRING(ICHAR:),'(A8)') NAM3
          END IF
      ELSE IF (N.EQ.4) THEN
          WRITE(STRING,'(''('',A8)') NAM1             
          ICHAR = LEN1(STRING) + 1
          WRITE(STRING(ICHAR:),'(''-'')')
          ICHAR = ICHAR+1
          WRITE(STRING(ICHAR:),'(A8)') NAM2
          ICHAR = LEN1(STRING) + 1
          WRITE(STRING(ICHAR:),'('')*('')')
          ICHAR = ICHAR+3
          WRITE(STRING(ICHAR:),'(A8)') NAM3
          ICHAR = LEN1(STRING) + 1
          WRITE(STRING(ICHAR:),'(''-'')')
          ICHAR = ICHAR+1
          WRITE(STRING(ICHAR:),'(A8)') NAM4
          ICHAR = LEN1(STRING) + 1
          WRITE(STRING(ICHAR:),'('')'')')
       END IF
       RETURN
       END

	SUBROUTINE POSITN(RHST,STX,STY,STZ,AEIOWM,STATYP,GSTRAD,DT)
	REAL*8 RHST(6),STX,STY,STZ,AEIOWM(6),DT,RE,GM,HJ2
	INTEGER STATYP
	COMMON /EARTH/ RE,GM,HJ2
	DATA RE/6370000./, GM/3.986012D14/, HJ2/1.083D-3/
	COST = COS(GSTRAD)
	SINT = SIN(GSTRAD)
	IF (STATYP.LE.1) THEN
	    RHST(1) = STX*COST + STY*SINT	! Recall STX, etc. form
	    RHST(2) = STX*SINT - STY*COST	! a left-handed system;
	    RHST(3) = STZ			! RHST right-handed
	ELSE IF (STATYP.EQ.2) THEN
	    CALL PROPGAT(AEIOWM,DT)
	    CALL CARTES(RHST,GM,AEIOWM(1),AEIOWM(2),AEIOWM(3),
     1		AEIOWM(4),AEIOWM(5),AEIOWM(6))
	END IF
	RETURN
	END

      SUBROUTINE PRTNUM(X,Y,HT,VAL,FMT,ANGLE,ID)
C
C    VAX EQUIVALENT OF IBM PLOT SUBROUTINE 'PRTNUM'
C
      CHARACTER*(*) FMT
      CHARACTER*10  OUT
      OUT='   '
      IF(FMT(2:2) .EQ. 'I') THEN
         IVAL = VAL
         ENCODE(10,FMT,OUT) IVAL
      ELSE
         ENCODE(10,FMT,OUT) VAL
      END IF
      CALL GRCHAR(ID,.FALSE.,ANGLE,.FALSE.,X,Y,OUT)
      RETURN
      END

      SUBROUTINE SYSPLT(X,Y,IPEN,ID)
C
C	Draw vector from last point to (X,Y)
C
C	IPEN=2 : pen down
C	IPEN=3 : pen up
C
      MODE = MOD(IPEN,10) - 1
      CALL GRVECT(ID,MODE,.FALSE.,1,X,Y)
      RETURN
      END
