*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*  2-D real to conjugate symmetric Fourier transformer                        *
*  Utilises TRANS, a                                                          *
*  Generalised 2 dimensional transposer - useful for virtual machines         *
*  Causes 2*N1*N2 page faults of size M1*M2                                   *
*  It will run without trouble if the working memory size is at               *
*  least (MAX(N1,N2)+1)*M1*M2                                                 *
*                                                                             *
*  Copyright (c) Maximum Entropy Data Consultants Ltd.                        *
*           33 North End, Meldreth                                            *
*           Royston SG8 6NR, U.K.                                             *
*                                                                             *
*  Written by Steve Gull    November 1981                                     *
*  Fourier part written December 1981 in OVRO                                 *
*  and revised December 1989 in CIT to persuade the CONVEX to run it faster   *
*  NOTE: Our classier Fouriers were slower on the CONVEX, so swap out this    *
*  this routine by all means.....                                             *
* Here are some notes on the mathematics and storage......                    *
*-----------------------------------------------------------------------------*

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*  The code has to be linked differently on IKAROS and DEIMOS                 *
*    For Deimos link to FOUR1D : for IKAROS link to  FOUR1I (calls VECLIB)    *
*-----------------------------------------------------------------------------*
 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Two-dimensional Fourier transform is defined by
*
*              NX-1  NY-1           2PI i (x * Kx / NX + y * Ky / NY)
*  F       :=   SUM  SUM    f    * e
*   Kx,Ky       x=0  y=0     x,y
*                                                             *
* In a conjugate-symmetric transform, f is real and F     = F
*                                                    Kx,Ky   NX-Kx,NY-Ky
* Storage:
*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Format of real array has first index "x" increasing fastest.
* For example, (NX=16, NY=8)  has the following 128 real numbers
* f(x,y) in row order ("xy" means the x,y component) :
*
*           -----   x
*      00  10  20  30  40  50  60  70  80  90  A0  B0  C0  D0  E0  F0 
*   |  01  11  21  31  41  51  61  71  81  91  A1  B1  C1  D1  E1  F1 
*   |  02  12  22  32  42  52  62  72  82  92  A2  B2  C2  D2  E2  F2 
*   |  03  13  23  33  43  53  63  73  83  93  A3  B3  C3  D3  E3  F3 
*   |  04  14  24  34  44  54  64  74  84  94  A4  B4  C4  D4  E4  F4 
*      05  15  25  35  45  55  65  75  85  95  A5  B5  C5  D5  E5  F5 
*   y  06  16  26  36  46  56  66  76  86  96  A6  B6  C6  D6  E6  F6 
*      07  17  27  37  47  57  67  77  87  97  A7  B7  C7  D7  E7  F7 
*
*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Format of packed conjugate-symmetric 2-D Fourier is transposed in x,y:
*
*      (Re Kx=0, Re Kx=NX/2), Cmplx Kx=1, Cmplx Kx=2,.., Cmplx Kx=NX/2-1
*
* where
*  Re Kx is (Ky=0 , Ky=NY/2), Cmplx Ky=1, Cmplx Ky=2,.., Cmplx Ky=NY-1
* and
*  Cmplx Kx is    Cmplx Ky=0, Cmplx Ky=1, Cmplx Ky=2,.., Cmplx Ky=NY-1 
*
* For example, (NX=16, NY=8)  has the following 64 complex numbers
* F(Kx,Ky) in row order ("xy" means the Kx,Ky Fourier component) :
*
*                 ----- Ky
*          (00,04)  01     02     03   (80,84)  81     82     83
*      |     10     11     12     13     14     15     16     17
*      |     20     21     22     23     24     25     26     27
*      |     30     31     32     33     34     35     36     37
*      |     40     41     42     43     44     45     46     47
*            50     51     52     53     54     55     56     57
*     Kx     60     61     62     63     64     65     66     67
*            70     71     72     73     74     75     76     77
*
* The four real components occupy real locations 0, 1, NY, NY+1
* (i.e. complex locations 0 and NY/2).
*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

*-----------------------------------------------------------------------------*
      SUBROUTINE FOURSG(X,Y,Z,N,ISIGN,IFORM)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
* Purpose:
*         Two-dimensional packed conjugate-symmetric out-of-place
*         Fourier transform, with inverse.
*
*  X    is the input array
*  Y    is the ouput array (can be the same as X)
*  Z    is a work-space array  (needed because I didn't know how 
*       to do this tranpose properly in 1981
*
*  N(2) contains the array dimensions (powers of 2 >= 2)
*  ISIGN is the sign of the transform
*  IFORM  = 0 does real to conjugate symmetric
*  IFORM =  -1 does the inverse
*
*  As set up, is it very slightly faster to use
*  ISIGN = 1 with IFORM = 0
*  and ISIGN=-1 with IFORM=-1 (!)
*
*  The coefficients of FIXRL and the subroutine BITRV
*  are reduced to tables in this version
*  The routine COOL2 is taken directly from the subroutine FOUR2
*  by Norman Brenner (1966, MIT)
*----------------------------------------------------------------------------*

      COMPLEX X(*),Y(*),Z(*)
      INTEGER N(*)
      DATA NWORK/131072/

*     NWORK is the working set size (no. of complexes)
*     set to be ideal for VAX 3600 or CONVEX (1 MByte)

      INCLUDE 'FOURSG.INC'
* Some savings are made by tabling coefficients, hence it needs a table size
      IF(N(1).GT.NSIZE.OR.N(2).GT.NSIZE) THEN
        WRITE(*,*) 'FOURSG maximum size exceed - edit NSIZE in'
     *             ,' FOURSG.INC'
        STOP
      ENDIF
      M1=MIN(NWORK/N(2),N(1)/2)
      M2=MIN(NWORK*2/N(1),N(2))
      N1=N(1)/M1/2
      N2=N(2)/M2
      IF(IFORM.EQ.-1) GOTO 1
      IF(IFORM.NE.0) GOTO 2
      CALL TRAN1(X,Z,M1,N1,M2,N2,ISIGN,IFORM)
      CALL TRAN2(Z,Y,M1,N1,M2,N2,ISIGN,IFORM)
      RETURN
1     CONTINUE
      CALL TRAN1(X,Z,M2,N2,M1,N1,ISIGN,IFORM)
      CALL TRAN2(Z,Y,M2,N2,M1,N1,ISIGN,IFORM)
      RETURN
2     WRITE(6,100) IFORM
100   FORMAT(' IFORM = ',I5,' is not implemented')
      STOP
      END
C
      SUBROUTINE TRAN1(X,Y,M1,N1,M2,N2,ISIGN,IFORM)
C     Stage 1. Perform a block transpose
      COMPLEX X(*),Y(*),XWORKC(1)
      INCLUDE 'FOURSG.INC'
      EQUIVALENCE (XWORK(1),XWORKC(1))
      MM=M1*M2
      MN=M1*N1
      MN2=MN*2
      MN3=MN/2
      MMN2=MM*N2
      MMN=MM*N1
      MMNN=MMN*N2
      IF=IFORM
      IF(IFORM.EQ.-1) IF=1
      IF(IFORM.EQ.-1) MN2=MN
      J=1
      K=1
      DO 4 I4=1,N2
      DO 5 I5=1,M2
      IF(I4.NE.1.OR.I5.NE.1.OR.IFORM.NE.-1) GOTO 8
C
C     First two columns
C
      CALL FOUR1(X,MN,ISIGN,-1)
      CALL FOUR1(X(MN3+1),MN,ISIGN,-1)
      DO 6 I6=1,MN
6     XWORKC(I6)=X(I6)
      DO 7 I6=1,MN
7     X(I6)=CMPLX(REAL(XWORK(I6)),XWORK(I6+MN))
      GOTO 5
8     CALL FOUR1(X(J+(I5-1)*MN),MN2,ISIGN,IF)
5     CONTINUE
      DO 3 I3=1,N1
      DO 2 I2=1,M2
      DO 1 I1=1,M1
      Y(K)=X(J)
      K=K+1
1     J=J+1
2     J=J+MN-M1
      J=J+M1-MMN
3     K=K+MMN2-MM
      J=J+MMN-MN
4     K=K+MM-MMNN
      RETURN
      END
C
      SUBROUTINE TRAN2(X,Y,M1,N1,M2,N2,ISIGN,IFORM)
C     Stage 2. Unscramble the blocks
      INCLUDE 'FOURSG.INC'
      COMPLEX XWORKC(1)
      COMPLEX X(*),Y(*)
      EQUIVALENCE(XWORKC(1),XWORK(1))
      MM=M1*M2
      MN=M2*N2
      MN2=MN*2
      MN3=MN/2
      MMN=MM*N2
      IF=IFORM
      IF(IFORM.EQ.0) IF=1
      IF(IFORM.EQ.0) MN2=MN
      J=1
      K=1
      DO 3 I3=1,N1
      DO 2 I2=1,M1
      DO 1 I1=1,MN
      Y(K)=X(J)
      K=K+1
1     J=J+M1
      IF(I2.NE.1.OR.I3.NE.1.OR.IFORM.NE.0) GOTO 5
C
C      First column contains scrambled reals
C
      DO 7 I4=1,MN
      XWORK(I4)=REAL(Y(I4))
      XWORK(I4+MN)=AIMAG(Y(I4))
7     CONTINUE
      DO 6 I4=1,MN
6     Y(I4)=XWORKC(I4)
      CALL FOUR1(Y,MN,ISIGN,0)
      CALL FOUR1(Y(MN3+1),MN,ISIGN,0)
      GOTO 2
5     CALL FOUR1(Y(K-MN),MN2,ISIGN,IF)
2     J=J+1-MMN
3     J=J+MMN-M1
      RETURN
      END
