      SUBROUTINE MEMCG(OUT,ALPHA,C,TOL,DEF,ACC, LOWER,VALUE,UPPER,CODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*    Investigation of positive maximum of  Q(x) = c + 2x.A.g - x.B.A.x
*
*    where      B = alpha*I + A     ,  A = Memop.{f}.Memtr ,
*               Memop = acc * Opus  ,  Memtr = Tropus * acc
*
*                             -1                             -1
*    Maximum occurs at  x = B  .g  with value  Qmax = c + g.B  .A.g
*
*    Use conjugate gradient estimate y of x to set 
*           lower limit on Qmax  =  Q(y)  =  c + y.B.A.y
*    and obtain an upper limit because B exceeds both alpha*I and A
*
*    Enter with areas
*                     <1>   = image {f}
*                     <26>  = g
*                     <23>  = input        (if OUT=1)
*                     <20>  = default      (if M1 =3)
*                     <22>  = accuracy     (if M2 =0)
*                     <29>  = responsivity (if M3 =1)
*    Exit with area
*                     <23> += y            (if OUT=1)
*
*    Workspace areas
*                     <2>,<25>,<26>,<27>,<28>
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    OUT     I     I      -         Flag for updating x (1 is "update")
*    ALPHA   R*8   I      -         Entropic multiplier alpha, > 0
*    C       R*8   I      -         Offset (not so negative that Qmax<0)
*    TOL     R*8   I      -         Tolerance factor for ending, > 0
*    DEF     R*8   I      -         Default  (if M1=3)
*    ACC     R*8   I      -         Accuracy (if M2=0)
*    LOWER   R*8     O    -         Lower limit on Qmax
*    VALUE   R*8     O    -         Estimate of    Qmax
*    UPPER   R*8     O    -         Upper limit on Qmax
*    CODE    I       O    -         Return code
*     Return codes:
*                0 = tolerance limit achieved or g tiny      (OK)
*                1 = maximum number of iterations reached    (damaging)
*                2 = upper limit about to go negative        (dangerous)
*                3 = lower limit about to be out of range    (dangerous)
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    MEMP        Initialise
*    MEMQ        Application of matrix A
*    MEMR        Update workspace
*    MEMS        Update gradient
*    MEMT        Update multipliers
*    UEXIT       Error exit
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS               8 Jan 1989     Version 1.5
*   JS              19 Jan 1989     Version 1.52
*   JS              26 Jan 1989     Version 1.6
*   JS              17 Jun 1989     Version 2.00 with pos/neg
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER OUT,LOOP,CODE,LIST
      PARAMETER (LIST=40)
      REAL*8 ALPHA,C,TOL,DEF,ACC,P,Q,R,S,T,UPPER,VALUE,LOWER
      REAL*8 EPS,BIG,ZERO
      PARAMETER (ZERO=0., EPS=1.D-20, BIG=1.D20)
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,51)
      IF(L1.GE.2) WRITE(IOUT,52)
      IF(ALPHA.LE.0.) CALL UEXIT('Conjugate gradient needs alpha > 0')
      CALL MEMP
      LOWER=C
      UPPER=BIG
      CODE=1
      DO 5 LOOP=1,LIST
        CALL MEMQ(DEF,ACC,P)
        IF(P.LT.EPS) THEN
          UPPER=LOWER
          CODE=0
          GOTO 9
        ENDIF
        P=1./P
        CALL MEMR(ALPHA,P,R,S,T)
        IF(S.LT.EPS .OR. T.LT.EPS) THEN
          UPPER=LOWER
          CODE=0
          GOTO 9
        ENDIF
        Q=1./(ALPHA*S+T)
        CALL MEMS(Q)
        IF(OUT.EQ.1) THEN
          CALL MEMMUL(27,25,ACC)
          CALL MEMT(Q)
        ENDIF
        R=LOWER+MIN(1./(ALPHA*P),R)
        IF(R.LE.0.) THEN
          LOWER=0.
          CODE=2
          GOTO 9
        ENDIF
        UPPER=MIN(UPPER,R)
        IF(LOWER+Q.GT.UPPER) THEN
          LOWER=UPPER
          CODE=3
          GOTO 9
        ENDIF
        LOWER=LOWER+Q
        IF(L1.GE.2) WRITE(IOUT,53) LOOP,LOWER,UPPER
        IF(UPPER.LE.(1.+TOL)*LOWER) THEN
          CODE=0
          GOTO 9
        ENDIF
    5 CONTINUE
    9 CONTINUE
      IF(LOWER.GT.0.) THEN
        VALUE=LOWER+MAX(MIN(TOL*LOWER,(UPPER-LOWER)/2.),ZERO)
      ELSE
        VALUE=UPPER/2.
      ENDIF
      IF(L1.GE.2) WRITE(IOUT,54) CODE,VALUE
      IF(L0.GE.1) WRITE(IOUT,55)
   51 FORMAT('   ConjGrad...')
   52 FORMAT('   ConjGrad     lower                 upper')
   53 FORMAT(' ',I4,1PE24.15,E24.15)
   54 FORMAT('          ( code ',I1,' )  return ',1PE24.15)
   55 FORMAT('   ...ConjGrad')
      END


      SUBROUTINE MEMDET(ALPHA,C,TOL,DEF,ACC, 
     *           LOWDET,VALDET,UPDET,CODDET, LOWTR,VALTR,UPTR,CODETR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*     Values of
*
*          c + log determinant(B/alpha) = c + SUM log( 1+eval[A]/alpha )
*     and
*                         -1                   -1
*          trace((B/alpha)  .A)  =  alpha * r.B  .A.r 
*
*                        = alpha * maximum of( Q(x) = 2x.A.r - x.B.A.x )
*
*     where      B = alpha*I + A     ,  A = Memop.{f}.Memtr
*                Memop = acc * Opus  ,  Memtr = Tropus * acc
*
*
*    Enter with areas
*                     <1>   = image f
*                     <26>  = random normal vector r
*                     <20>  = default      (if M1 =3)
*                     <22>  = accuracy     (if M2 =0)
*                     <29>  = responsivity (if M3 =1)
*
*    Workspace areas
*                     <2>,<25>,<26>,<27>,<28>
*
* Method:
*   Conjugate gradient maximisation of Q(x),
*   with sophisticated use of the scalars it generates.
*                        -1
*   To calculate  trace(B  .A) , use the same algebra as in MEMCG,
*   with a random source vector r.
*
*   To calculate  log(det B) , use the recurrence relation
*
*    npoly, {ppoly[i],qpoly[i] for i=0..npoly-1
*
*   for orthogonal polynomials of B orthogonal over A
*
*        r.P (B).A.P (B).r = 0   ,   for  j != k  and  j,k <= npoly
*           j       k
*   normalised to P(0)=1.  Recurrence relation can be derived from the
*   conjugate gradient scalars and is defined by
*
*            m = 0               ;   P (x) = 1 
*                                     0
*     for i=0..npoly-1
*            m += ppoly[i]*P (x) ; P   (x) = P (x) - x*qpoly[i]*m
*                           i       i+1       i
*
*   Use roots rpoly[] and Gaussian weights wpoly[] such that
*     npoly-1
*       SUM   wpoly  * f(rpoly )   approximates   r.f(B).r
*       i=0        i          i
*   with equality if f is a polynomial up to order 2*npoly-1.
*
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ALPHA   R*8   I      -         Entropic multiplier alpha, > 0
*    C       R*8   I      -         Offset (positive, in fact)
*    TOL     R*8   I      -         Tolerance factor for ending, > 0
*    DEF     R*8   I      -         Default  (if M1=3)
*    ACC     R*8   I      -         Accuracy (if M2=0)
*    LOWDET  R*8     O    -         Lower limit on  c + logdet
*    VALDET  R*8     O    -         Estimate of     c + logdet
*    UPDET   R*8     O    -         Upper limit on  c + logdet
*    CODDET  I       O    -         Return code for logdet
*    LOWTR   R*8     O    -         Lower limit on trace
*    VALTR   R*8     O    -         Estimate of    trace
*    UPTR    R*8     O    -         Upper limit on trace
*    CODETR  I       O    -         Return code for trace
*     Return codes:
*                0 = tolerance limit achieved or r tiny      (OK)
*                1 = maximum number of iterations reached    (damaging)
*                2 = upper limit about to go negative        (dangerous)
*                3 = lower limit about to be out of range    (dangerous)
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    ...         MECOMP     I    Sizes and flags
*
* External calls:
*    MEMP        Initialise
*    MEMQ        Application of matrix A
*    MEMR        Update workspace
*    MEMS        Update source vector dQ/dx
*    MEMDE1      Roots rpoly of characteristic polynomial
*    MEMDE2      Gaussian weights wpoly
*    UEXIT       Error exit
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS              11 Dec 1988     Merged version
*   JS               9 Jan 1989     MEMRUN switch, ALPHA scale
*   JS              29 Jan 1989     MEMRUN switch removed
*   JS              15 Feb 1989     Diagnostic flag
*   JS              17 Jun 1989     Version 2.00 with pos/neg
*
* Notes:
*   (1)  If  tol  is set too small, the returned values will be too near
*        the lower limit.  (Otherwise, the returned values might be 
*        unduly biassed upwards by a poor upper limit if the procedure 
*        terminated prematurely.)
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER K,LOOP,CODDET,CODETR,NPOLY,LIST
      PARAMETER (LIST=40)
      REAL*8 ALPHA,C,TOL,DEF,ACC,P,Q,R,S,T,X,Y,Z,NEXT,UPTR,VALTR,LOWTR
      REAL*8 UPDET,VALDET,LOWDET,EPS,BIG,ZERO
      REAL*8 PPOLY(0:LIST-1),QPOLY(0:LIST-1),
     *       RPOLY(0:LIST)  ,WPOLY(0:LIST-1)
      PARAMETER (ZERO=0., EPS=1.D-20, BIG=1.D20)
      INCLUDE 'MECOMP.INC'
      IF(L0.GE.1) WRITE(IOUT,51)
      IF(ALPHA.LE.0.) CALL UEXIT('LogDet procedure needs alpha > 0')
* Special case of ALPHA very large would be
*  VALDET=C; CALL MEMQ(DEF,ACC,VALTR)

* Initialise
      CALL MEMP
      LOWTR=0.
      UPTR=BIG
      CODETR=1
      LOWDET=C
      UPDET=BIG
      CODDET=1
      NPOLY=0
      DO 5 LOOP=1,LIST
* Conventional conjugate gradient
        CALL MEMQ(DEF,ACC,P)
        IF(P.LT.EPS) THEN
          UPTR=LOWTR
          CODETR=0
          CODDET=0
          GOTO 9
        ENDIF
        P=1./P
        CALL MEMR(ALPHA,P,R,S,T)
        IF(S.LT.EPS .OR. T.LT.EPS) THEN
          UPTR=LOWTR
          CODETR=0
          CODDET=0
          GOTO 9
        ENDIF
        Q=1./(ALPHA*S+T)
        CALL MEMS(Q)
* Limits on good degrees of freedom (with early termination possible)
        IF(CODETR.EQ.1) THEN
          Z=LOWTR+MIN(1./(ALPHA*P),R)
          IF(Z.LE.0.) THEN
            LOWTR=0.
            CODETR=2
          ENDIF
        ENDIF
        IF(CODETR.EQ.1) THEN
          UPTR=MIN(UPTR,Z)
          IF(LOWTR+Q.GT.UPTR) THEN
            LOWTR=UPTR
            CODETR=3
          ENDIF
        ENDIF
        IF(CODETR.EQ.1) THEN
          LOWTR=LOWTR+Q
          IF(UPTR.LE.(1.+TOL)*LOWTR) THEN
            CODETR=0
          ENDIF
        ENDIF
* Limits on log determinant (with early termination possible)
        IF(CODDET.EQ.1) THEN
          PPOLY(NPOLY)=P
          QPOLY(NPOLY)=Q
          NPOLY=NPOLY+1
          CALL MEMDE1(ALPHA,PPOLY,QPOLY,RPOLY,NPOLY)
          CALL MEMDE2(ALPHA,PPOLY,QPOLY,RPOLY,WPOLY,NPOLY)
          X=1./(ALPHA*P*R)
          IF(X.GT.1.) X=1.+LOG(X)
          Y=LOWDET+R*X
          IF(Y.LE.0.) THEN
            CODDET=2
          ENDIF
        ENDIF
        IF(CODDET.EQ.1) THEN
          UPDET=MIN(UPDET,Y)
          NEXT=C
          DO 1 K=0,NPOLY-1
            NEXT=NEXT+WPOLY(K)*LOG(RPOLY(K)/ALPHA)
    1     CONTINUE
          IF((NEXT.LT.LOWDET).OR.(NEXT.GT.UPDET)) THEN
            CODDET=3
          ENDIF
        ENDIF
        IF(CODDET.EQ.1) THEN
          LOWDET=NEXT
          IF(UPDET.LE.(1.+TOL)*LOWDET) THEN
            CODDET=0
          ENDIF
        ENDIF
* Exit if early termination of both "good" and "logdet"
        IF(CODETR.NE.1 .AND. CODDET.NE.1) GOTO 9
    5 CONTINUE
* Final results
    9 CONTINUE
      LOWTR=LOWTR*ALPHA
      UPTR=UPTR*ALPHA
      IF(LOWTR.GT.0.) THEN
        VALTR=LOWTR+MAX(MIN(TOL*LOWTR,(UPTR-LOWTR)/2.),ZERO)
      ELSE
        VALTR=UPTR/2.
      ENDIF
      LOWDET=MAX(LOWDET,ZERO)
      VALDET=LOWDET+MAX(MIN(TOL*LOWDET,(UPDET-LOWDET)/2.),ZERO)
      IF(L0.GE.1) WRITE(IOUT,55)
   51 FORMAT('   LogDet...')
   55 FORMAT('   ...LogDet')
      END


      SUBROUTINE MEMDE1(ALPHA,PPOLY,QPOLY,RPOLY,NPOLY)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*                  Roots of interleaving polynomials.
*
* Enter with:
*   1)   alpha = positive lower limit to all roots
*   2)   recurrence relation npoly, ppoly[0..npoly-1], qpoly[0..npoly-1]
*        of polynomials up to current order
*   3)   roots rpoly[0..npoly-2] of previous polynomial
*
*   Exit with:
*        Roots rpoly[0..npoly-1] of current polynomial
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ALPHA   R*8   I      -         Positive lower limit
*    PPOLY   R*8   I     (0:*)      Recurrence relation coeffs
*    QPOLY   R*8   I     (0:*)      Recurrence relation coeffs
*    RPOLY   R*8   I O   (0:*)      Roots of interleaving polynomials
*    NPOLY   I     I      -         Order of polynomials
*
* Globals:
*     -
*
* External calls:
*     -
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS              29 Jan 1989     Loop limit bigger: no linear polish
*
* Notes:
*   (1) Polynomial evaluation is protected against overflow :-
*       Eigenvalues of 2x2 recurrence matrix have product equal to unity
*       and negative sum, so that any large eigenvalue is negative real,
*       and it suffices to check if sum of the two elements
*       being recursed is too BIG.
*   (2) Because of the frequency of multiple nearly equal roots, the
*       polynomials may NOT usefully be approximated by linear or
*       quadratic forms: binary chop is best.
*   (3) If inaccurate root determination is suspected, 
*       increase the LOOP limit.
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,J,NPOLY,LOOP,LIST
      REAL*8 PPOLY(0:*),QPOLY(0:*),RPOLY(0:*)
      REAL*8 ALPHA,PARITY,D,X,M,G,A,B,EPS,BIG
      PARAMETER (LIST=40)
      PARAMETER (EPS=1.D-20, BIG=1.D20)
* Treat incoming roots as upper limits to successive roots
      PARITY=1.
      DO 1 J=NPOLY-1,1,-1
        RPOLY(J)=RPOLY(J-1)
        PARITY=-PARITY
    1 CONTINUE
* New overall lower limit
      RPOLY(0)=ALPHA
* New overall upper limit
      D=RPOLY(0)
      X=RPOLY(NPOLY-1)
    2 CONTINUE
        X=X+D
        D=D*2.
        M=0.
        G=1.
        DO 3 I=0,NPOLY-1
          M=M+PPOLY(I)*G
          G=G-QPOLY(I)*M*X
          IF(M+G.GE.BIG) THEN
            M=M*EPS
            G=G*EPS
          ENDIF
    3   CONTINUE
      IF(PARITY*G.GE.0.) GOTO 2
      RPOLY(NPOLY)=X
* For each interval ...
      PARITY=1.
      DO 9 J=0,NPOLY-1
* Binary chop to near root
        A=RPOLY(J)
        B=RPOLY(J+1)
        DO 5 LOOP=1,LIST
          X=(A+B)/2.
          M=0.
          G=1.
          DO 4 I=0,NPOLY-1
            M=M+PPOLY(I)*G
            G=G-QPOLY(I)*M*X
            IF(M+G.GE.BIG) THEN
              M=M*EPS
              G=G*EPS
            ENDIF
    4     CONTINUE
          IF(PARITY*G.GT.0.) THEN
            A=X
          ELSE
            B=X
          ENDIF
    5   CONTINUE
        RPOLY(J)=X
        PARITY=-PARITY
    9 CONTINUE
      END

      SUBROUTINE MEMDE2(ALPHA,PPOLY,QPOLY,RPOLY,WPOLY,NPOLY)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*                      Weights of top polynomial.
*
* Enter with:
*   1)   recurrence relation npoly, ppoly[0..npoly-1], qpoly[0..npoly-1]
*        of polynomials up to current order
*   2)   roots rpoly[0..npoly-1] of top polynomial
*
* Exit with:
*        Weights wpoly[0..npoly-1] of top polynomial
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ALPHA   R*8   I      -         Positive lower limit
*    PPOLY   R*8   I     (0:*)      Recurrence relation coeffs
*    QPOLY   R*8   I     (0:*)      Recurrence relation coeffs
*    RPOLY   R*8   I     (0:*)      Roots of interleaving polynomials
*    WPOLY   R*8     O   (0:*)      Weights of top polynomial
*    NPOLY   I     I      -         Order of polynomials
*
* Globals:
*     -
*
* External calls:
*     -
*
* History:
*   John Skilling    7 Dec 1988     MEMSYS3 version
*   JS              29 Jan 1989     WPOLY protection
*   JS              13 Mar 1989     ZERO parameter, V1.71
*
* Notes:
*-----------------------------------------------------------------------
      IMPLICIT CHARACTER*1 (A-Z)
      INTEGER I,J,NPOLY
      REAL*8 PPOLY(0:*),QPOLY(0:*),RPOLY(0:*),WPOLY(0:*)
      REAL*8 ALPHA,PARITY,X,M,N,G,H,DM,DG,EPS,BIG,ZERO
      PARAMETER (EPS=1.D-20, BIG=1.D20, ZERO=0.)
      PARITY=1.
* For each root ...
      DO 9 J=0,NPOLY-1
* Gradient dg and dual polynomial h at root
        X=RPOLY(J)
        M=0.
        DM=0.
        DG=0.
        G=1.
        N=1.
        H=-QPOLY(0)
        DO 1 I=0,NPOLY-1
          M=M+PPOLY(I)*G
          DM=DM+PPOLY(I)*DG
          N=N+PPOLY(I)*H*X
          G=G-QPOLY(I)*M*X
          DG=DG-QPOLY(I)*(DM*X+M)
          H=H-QPOLY(I)*N
          IF(M+G.GT.BIG) THEN
            M=M*EPS
            DM=DM*EPS
            N=N*EPS
            G=G*EPS
            DG=DG*EPS
            H=H*EPS
          ENDIF
    1   CONTINUE
* Write weight, rescaled from calculating g.f(B).A.g to g.f(A).g
        IF(ABS(RPOLY(J)-ALPHA).GT.EPS*ALPHA) THEN
          WPOLY(J)=MAX( (H/DG)/(RPOLY(J)-ALPHA) , ZERO)
        ELSE
          WPOLY(J)=0.
        ENDIF
        PARITY=-PARITY
    9 CONTINUE
      END
