      SUBROUTINE VRAND(K,N)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*          First call initialises strongly (and expensively)
*                     before proceeding.
*          If N <=0,  initialise weakly   (but cheaply)
*          If N > 0,  set vector of normally distributed random numbers
*                     ST(i) = Normal(0,1)  ,  i=K..K+N-1
*
* Method:
*    0)  First call hardwires random lookup table with 12345 as a seed.
*        Pre-calculate NN normal numbers in GAUSS, and
*        two permutations PERM1 and PERM2, of 1,2,...,NN.
*    1)  N <= 0 : Use K as a seed.
*        Take a randomly offset COPY of GAUSS.
*        Initiate random sequences ( + or - )  and  ( PERM1 or PERM2 ).
*    2)  N > 0  : Use K as a start address for the output.
*        Read up to NN numbers from COPY.
*        If more are needed, replace COPY by one of the signed
*        permutations  ( + or - )( PERM1 or PERM2 )COPY , and
*        read up to NN more numbers.  Continue until output is complete.
*        
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    K       I     I      -         Seed (if N<=0) or 
*                                    start address in ST (if N>0)
*    N       I     I      -         Weak initialisation (if N<=0) or
*                                    length of output vector (if N>0)
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*     ST        MECOMS        O  Output vector
*
* External calls:
*    VGAUSS     Set normal vector by direct generation of random numbers
*    VPERM      Set random permutation
*
* History:
*   John Skilling   10 Jan 1989     Polished for MEMSYS3 version 1.5
*   John Skilling   30 Jan 1989     Automatic hardwired start for V1.6
*
* Notes:
*   (1) First call will automatically set up random firmware.
*   (2) Subsequent initialisation calls are "weak", using N <= 0
*       to use the firmware lookup tables.
*   (3) The random numbers produced by weak initialisation are
*       permutations of only 2*NN individual samples (the factor of 2
*       arising from sign flips), but should often be adequate.
*   (4) The number NN of permuted samples is fairly arbitrary:
*       487 is a prime unlikely to bear much relation to anything else.
*   (5) If these numbers are not adequately random, or if there is not
*       enough storage available, call VGAUSS instead.
*       VGAUSS has the same specification, except that there is no
*       distinction between strong and weak initialisation.
*-----------------------------------------------------------------------
      INCLUDE 'MECOMS.INC'
      PARAMETER (IM=11979,IA=430,IC=2531,NN=487)
      REAL GAUSS(NN),COPY(NN),WORK(NN)
      INTEGER PERM1(NN),PERM2(NN),KW(NN)
      LOGICAL START
      SAVE GAUSS,COPY,PERM1,PERM2,IFLAG,ICOPY,START
      DATA START/.TRUE./

      IF(START) THEN
*
* Initialise normal vector GAUSS and permutations PERM1 and PERM2
*
        K0=12345
        CALL VGAUSS(GAUSS,K0,0)
        CALL VGAUSS(GAUSS,1,NN)
* Normalise to take out the worst systematic error
        Z=0.
        DO 1 I=1,NN
          Z=Z+GAUSS(I)*GAUSS(I)
    1   CONTINUE
        Z=SQRT(NN/Z)
        DO 2 I=1,NN
          GAUSS(I)=GAUSS(I)*Z
          COPY(I)=GAUSS(I)
    2   CONTINUE
* Permutations 
        JSEED=K0
        CALL VPERM(JSEED,PERM1,KW,NN)
        JSEED=MOD(JSEED*IA+IC,IM)
        CALL VPERM(JSEED,PERM2,KW,NN)
        START=.FALSE.
      ENDIF

      IF(N.LE.0) THEN

* Initialise switch and pointer along GAUSS
        IFLAG=MOD(K,IM)
        J=MOD(IFLAG*IA+IC,NN)
* Move offset version of GAUSS to COPY
        ICOPY=1
        DO 3 I=1,NN-J
          COPY(I)=GAUSS(I+J)
    3   CONTINUE
        J=J-NN
        DO 4 I=1-J,NN
          COPY(I)=GAUSS(I+J)
    4   CONTINUE

      ELSE

*
* N > 0, so write ST
*
* Pointer along destination vector ST
        IST=K
* Remaining number to be read
        NREM=N
   11   IF(NREM.LE.0) RETURN
* Possibly repermute COPY
          IF(ICOPY.EQ.1) THEN
* Permute randomly with PERM1 or PERM2. Put result temporarily in WORK
            IFLAG=MOD(IFLAG*IA+IC,IM)
            IF(IFLAG.LE.IM/2) THEN
              DO 12 I=1,NN
                WORK(I)=COPY(PERM1(I))
   12         CONTINUE
            ELSE
              DO 13 I=1,NN
                WORK(I)=COPY(PERM2(I))
   13         CONTINUE
            ENDIF
* Flip sign randomly, returning result to COPY
            IFLAG=MOD(IFLAG*IA+IC,IM)
            IF(IFLAG.LE.IM/2) THEN
              DO 14 I=1,NN
                COPY(I)=WORK(I)
   14         CONTINUE
            ELSE
              DO 15 I=1,NN
                COPY(I)=-WORK(I)
   15         CONTINUE
            ENDIF
          ENDIF
* Read as much as possible of COPY and update pointers
          LEN=MIN(NREM,NN-ICOPY+1)
          DO 16 I=0,LEN-1
            ST(IST+I)=COPY(ICOPY+I)
   16     CONTINUE
          ICOPY=ICOPY+LEN
          IST=IST+LEN
          NREM=NREM-LEN
* Don't read beyond the end of COPY
          IF(ICOPY.GT.NN) ICOPY=1
        GOTO 11
      ENDIF
      END


      SUBROUTINE VGAUSS(ST,K,N)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*          If N <= 0, initialise.
*
*          If N > 0 , set vector of normally distributed random numbers
*                     ST(i) = Normal(0,1)  ,  i=K..K+N-1
*
* Method:
*    1)  N <= 0 : Use K as a seed.
*        Pre-calculate two shuffling tables of 97 integers
*    2)  N > 0  : Use K as a start address for the output.
*        Generate two interleaved random sequences, using them to
*        give (x,y) in the square (-1<x<1,-1<y<1).  Obtain normally
*        distributed numbers from those (x,y) within the unit circle.
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    ST      R       O    *         Output vector
*    K       I     I      -         Seed (if N<=0) or 
*                                    start address in ST (if N>0)
*    N       I     I      -         Initialise (if N<=0) or
*                                    length of output vector (if N>0)
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    -
*
* External calls:
*    -
*
* History:
*   John Skilling   13 Dec 1988     Polished for MEMSYS3
*
* Notes:
*   (1) First call MUST have N <= 0 to initialise the shuffling tables.
*   (2) This is a good generator, with sequences derived from
*       "Numerical Recipes" (ex-Knuth).
*-----------------------------------------------------------------------
      REAL ST(*)
      INTEGER R1(0:96),R2(0:96)
      PARAMETER (M1=259200,J1=7141,K1=54773,Z=7.7160494E-6)
      PARAMETER (M2=243000,J2=4561,K2=51349)
      SAVE R1,R2,I1,I2

      IF(N.LE.0) THEN
* Initialise 97 pairs of integers, seeded by K
        I1=MOD(K1+K,M1)
        I1=MOD(J1*I1+K1,M1)
        I2=MOD(I1,M2)
        DO 1 L=0,96
          I1=MOD(J1*I1+K1,M1)
          I2=MOD(J2*I2+K2,M2)
          R1(L)=I1
          R2(L)=I2
    1   CONTINUE

      ELSE

* Fill ST with as many pairs of normal reals as necessary,
        N1=K
        N2=N1+N-1
        DO 3 I=N1,N2,2
    2     CONTINUE
*  by using pairs of reals lying within the unit circle.
            I1=MOD(J1*I1+K1,M1)
            I2=MOD(J2*I2+K2,M2)
            L=(97*I2)/M2
            V1=R1(L)*Z-1.
            R1(L)=I1
            L=(97*I1)/M1
            V2=R2(L)*Z-1.
            R2(L)=I2
            RR=V1*V1+V2*V2
            IF (RR.GE.1.) GOTO 2
          FAC=SQRT(-2.*LOG(RR)/RR)
          ST(I)=V1*FAC
          IF(I.LT.N2) ST(I+1)=V2*FAC
    3   CONTINUE
      ENDIF
      END



      SUBROUTINE VPERM(SEED,P,R,N)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* Purpose:
*             P :=  Random permutation of 1,2,...,N
*
* Method:
*    1)  Set lookup table of values 1,2,...,N
*    2)  Generate random numbers in [1,N] and bring the corresponding
*        entry into the output vector P (unless it was used earlier).
*    3)  Every so often, remove garbage by squashing the lookup table:
*        optimal speed is achieved by doing this when the garbage of
*        previously read numbers fills about 1/4 of the table.
*
* Parameters:
*   ARGUMENT TYPE  I/O  DIMENSION   DESCRIPTION
*    SEED    I     I      -         Seed for random number generator
*    P       I       O    N         Output vector
*    R       I      (O)   N         Workspace
*    N       I     I      -         Length of vectors
*
* Globals:
*   VARIABLE  COMMON BLOCK  I/O  DESCRIPTION
*    -
*
* External calls:
*    -
*
* History:
*   John Skilling   13 Dec 1988     Polished for MEMSYS3
*
* Notes:
*   (1) This uses a good generator, with sequences derived from
*       "Numerical Recipes" (ex-Knuth).
*-----------------------------------------------------------------------
      IMPLICIT INTEGER (A-Z)
      PARAMETER (M=714025,IA=1366,IC=150889)
      INTEGER P(N),R(N),L(0:96)
* M*N must not overflow
      IF( (M*N)/N .NE. M )  STOP ' Random generator overflow'
* Initialise 97 random integers in [0,M)
      IX=MOD(IC+SEED,M)
      DO 1 J=0,96
        IX=MOD(IA*IX+IC,M)
        L(J)=IX
    1 CONTINUE
      IX=MOD(IA*IX+IC,M)
      IY=IX
* Fill all points
      DO 2 K=1,N
        R(K)=K
    2 CONTINUE
      N2=N
    3 CONTINUE
* For the last fraction of output vector P ...
      N1=N2-(N2+3)/4
      IF(N1.GE.1) THEN
        DO 5 I=N2,N1,-1
* Get a new, previously unread, random within the current vector ..
    4     CONTINUE
            J=(97*IY)/M
            IY=L(J)
            K=1+(IY*N2)/M
            IX=MOD(IA*IX+IC,M)
            L(J)=IX
            IF(R(K).LT.0) GOTO 4
* .. Bring the corresponding unread number into P and blank it off
          P(I)=R(K)
          R(K)=-1
    5   CONTINUE
* then collect garbage by 
* squashing the fraction remaining unread and blanking the rest
        J=1
        DO 6 I=1,N2
          IF(R(I).GE.0) THEN
            R(J)=R(I)
            J=J+1
          ENDIF
    6   CONTINUE
        DO 7 I=J,N2
          R(I)=-1
    7   CONTINUE
* Prepare to look at next top fraction
        N2=N1-1
        GOTO 3
      ENDIF
* Finally, collect any remaining unread number
      DO 8 I=1,N2
        P(I)=R(I)
    8 CONTINUE
      END
